/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/******************************************************************************************/
/* NAME: macamudf.sas                                                                     */
/* VERSION: 5.4                                                                         */
/*                                                                                        */
/* DESCRIPTION: Macro to update or insert Campaign UDF records which are not              */
/*              included part of the new campaign record insert process                   */
/* Parameter:                                                                             */
/*    camp_char_udf - Physical table name for Campaign_Char_Udf                           */
/*    camp_num_udf  - Physical table name for Campaign_Num_Udf                            */
/*    camp_dt_udf   - Physical table name for Campaign_Date_udf                           */
/*                                                                                        */
/* PRODUCT: MA                                                                            */
/* USAGE: %macamudf                                                                       */
/*                                                                                        */
/******************************************************************************************/
/******************************************************************************************/
/*                                                                                        */
/* History:                                                                               */
/*   27APR2007 - Revised first version of the macro.                                      */
/*                                                                                        */
/* DATE       Defect No  Comments                                                         */
/* 14Oct2008  S0504678   SEQ_NO and CHECKLIST_FLG removed from Campaign UDF tables        */
/*                                                                                        */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                 */
/*                                                                                        */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the     */
/*                       master table. This is required when default UDF values are       */
/*                       reset to blank.                                                  */
/*                                                                                        */
/* 06Jan2012  S0782404   Fix the CAMP UDF table population                                */
/* 23Aug2013  S0992864   Add dswriteopts to data modify                                   */
/******************************************************************************************/
%macro macamudf(camp_char_udf=, camp_num_udf=, camp_dt_udf=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA09;
    %put Last Modified on 23Aug2013;
    %put MACAMUDF macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;


/* S0782404 - During initial publish all the campaign udf records are inserted by the dynamic source created in mapub script.
   Hence nothing to be processed from this macro */
%if &initial_pub.=Y %then
%do;
    %return;
%end;


/* Update Campaign_Char_Udf or insert records for the existing */
/* Campaign_Char_Udf: Update or Insert */
/* After the initial publish no new UDF records can be added in the respective tables.  Only value
   updates are possible. Hence re-use the same udf cnt variable here. */
%if &camp_cudf_cnt. > 0 %then
%do;
    data &libnm..&camp_char_udf.;
        modify &libnm..&camp_char_udf. (&dswriteopts.) work.&camp_char_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_SK CHAR_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 823;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =823;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_CHAR_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* Campaign_Num_Udf: Update or Insert */
%if &camp_nudf_cnt. > 0 %then
%do;
    data &libnm..&camp_num_udf.;
        modify &libnm..&camp_num_udf. (&dswriteopts.) work.&camp_num_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_SK NUM_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 825;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =825;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_NM_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%if &camp_dudf_cnt. > 0 %then
%do;
    /* Campaign_Num_Udf: Update or Insert */
    data &libnm..&camp_dt_udf.;
        modify &libnm..&camp_dt_udf. (&dswriteopts.) work.&camp_dt_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_SK DATE_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_= 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 827;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =827;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_DT_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%if &MADebug. eq Y %then
%do;
    %put MACAMUDF macro End Time - %sysfunc(datetime(), datetime24.2);
%end;
%mend;
