/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/* NAME: madttera.sas                                                                    */
/* VERSION: 5.4                                                                        */
/* DESCRIPTION: This script populates the Dynamic Treatments records to TERADATA tables  */
/*                                                                                       */
/* Parameters: Macro variables                                                           */
/*    Macro variables:                                                                   */
/*        dbconnect - Explicit Pass-through connect statement                            */
/*        targetschema - Target DB location                                              */
/*        templibname - Libref points to the temporary DB library                        */
/*        tempschema - Temporary location to upload the input tables                     */
/*                                                                                       */
/*                                                                                       */
/* PRODUCT: MA                                                                           */
/* USAGE: %madttera                                                                      */
/*                                                                                       */
/*****************************************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/* DATE       Defect No  Comments                                                        */
/* 12Dec2008  S0504678   First version of the code                                       */
/*                                                                                       */
/* 03Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 06May2010  S0647068   Change the insert logic to prevent remote constraint error      */
/*                                                                                       */
/* 06May2010  CI5.4      Populate CI_Package_X_Treatment for offer arbitration campaign  */
/*                                                                                       */
/* 06Oct2010  S0701120   Remove arbitration flag special handling                        */
/*****************************************************************************************/

%macro madttera(dbconnect=, targetschema=, templibname=, tempschema=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MADTTERA Current Version of the code: 5.4.1;
    %put Last Modified on 06May2010;
    %put MADTTERA macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Created Process DTTM variable */
data _null_;
    call symput('pdttm',put(datepart(datetime()),yymmdd10.)||' '||put(timepart(datetime()),tod8.));
run;

/* Insert the new records to the respective Dynamic Treatment tables */
proc sql noerrorstop;

&dbconnect;;

    /* Exit the process if the pass-through connect statement fails */
    %if &SQLXRC. ne 0 %then
    %do;
        %let MAError=753;
        %let SYSCC=100000;
        %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
        %return;
    %end;

    /* S0701120 arbitration flag check is removed */
    %if &_g_cpxt_tbl_cnt. > 0 %then
    %do;
        /* Step 2: Populate CI_PACKAGE_X_TREATMENT with Package_hash_val */
        /* This table has to be populated for both Static and dynamic treatments so always expect to have some records in it */

        /* 2a): Index creation not required here for Teradata as the Index has been created
       along with the table upload itself */

        /* 2b): Insert nonexisting records into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_PACKAGE_X_TREATMENT Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (
            LOCK TABLE &targetschema..&_g_cpxt_tbl. FOR WRITE
            MERGE INTO &targetschema..&_g_cpxt_tbl. AS tt
            USING (
                select
                    package_sk,
                    package_hash_val,
                    treatment_sk,
                    contributing_cell_package_sk,
                    sequence_no
                from
                     &tempschema.&_g_cpxt_unq.
            ) AS st
            ON (
                    tt.package_sk                   = st.package_sk
                and tt.package_hash_val             = st.package_hash_val
                and tt.treatment_sk                 = st.treatment_sk
                and tt.contributing_cell_package_sk = st.contributing_cell_package_sk
                and tt.sequence_no                  = st.sequence_no
            )
            WHEN NOT MATCHED THEN INSERT (
                package_sk,
                package_hash_val,
                treatment_sk,
                contributing_cell_package_sk,
                sequence_no
            )
            VALUES (
                st.package_sk,
                st.package_hash_val,
                st.treatment_sk,
                st.contributing_cell_package_sk,
                st.sequence_no
            )
        )by TERADATA;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_cpxt_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        execute (commit) by TERADATA;

        /* Drop temp table */
        execute ( drop table &tempschema.&_g_cpxt_unq. )by TERADATA;

        execute (commit) by TERADATA;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_PACKAGE_X_TREATMENT Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end;/* End of Step 2 */

    /* Step 3: Populate CI_DYNAMIC_TREATMENT_ATTRIBUTE table */
    %if &_g_dt_tbl_cnt. > 0 %then
    %do;
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Index Creation: %sysfunc(datetime(), datetime24.2);
        %end;

        /* Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (
            LOCK TABLE &targetschema..&_g_dt_tbl. FOR WRITE
            MERGE INTO &targetschema..&_g_dt_tbl. AS tt
            USING (
                select
                    cell_package_sk,
                    treatment_sk,
                    package_hash_val,
                    treatment_hash_val
                from
                     &tempschema.&_g_da_unq.
            ) AS st
            ON (
                      tt.cell_package_sk    = st.cell_package_sk
               and    tt.treatment_sk       = st.treatment_sk
               and    tt.package_hash_val   = st.package_hash_val
               and    tt.treatment_hash_val = st.treatment_hash_val
            )
            WHEN NOT MATCHED THEN INSERT (
                    cell_package_sk,
                    treatment_sk,
                    package_hash_val,
                    treatment_hash_val
            )
            VALUES (
                    st.cell_package_sk,
                    st.treatment_sk,
                    st.package_hash_val,
                    st.treatment_hash_val
            )

        ) by TERADATA;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_dt_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        execute (commit) by TERADATA;

        /* Drop temp table */
        execute ( drop table &tempschema.&_g_da_unq. )by TERADATA;

        execute (commit) by TERADATA;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end;/* End of Step 3 */

    /* Step 4: Populate the CI_TREATMENT_CHAR_UDF table after removing the records already exists */
    %if &_g_char_udf_cnt. > 0 %then
    %do;
        /* Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_TREATMENT_CHAR_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (
            LOCK TABLE &targetschema..&_g_char_udf_tbl. FOR WRITE
            MERGE INTO &targetschema..&_g_char_udf_tbl. AS tt
            USING (
                select
                    treatment_sk,
                    char_udf_nm,
                    treatment_hash_val,
                    char_udf_val,
                    char_ext_column_nm
                from
                     &tempschema.&_g_cu_unq.
            ) AS st
            ON (
                      tt.treatment_sk       = st.treatment_sk
                and   tt.treatment_hash_val = st.treatment_hash_val
                and   tt.char_udf_nm        = st.char_udf_nm
            )
            WHEN NOT MATCHED THEN INSERT (
                    treatment_sk,
                    char_udf_nm,
                    treatment_hash_val,
                    char_udf_val,
                    char_ext_column_nm,
                    processed_dttm
            )
            VALUES (
                    st.treatment_sk,
                    st.char_udf_nm,
                    st.treatment_hash_val,
                    st.char_udf_val,
                    st.char_ext_column_nm,
                    TIMESTAMP %str(%')&pdttm.%str(%')
            )
        )by TERADATA;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_char_udf_tbl.].  SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        execute (commit) by TERADATA;

        /* Drop temp table */
        execute ( drop table &tempschema.&_g_cu_unq. )by TERADATA;

        execute (commit) by TERADATA;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_TREATMENT_CHAR_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of Step 4:CI_TREATMENT_CHAR_UDF table processing */

    /* Step 5: Populate the CI_TREATMENT_NUM_UDF table after removing the records already exists */
    %if &_g_num_udf_cnt. > 0 %then
    %do;
        /* Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_TREATMENT_NUM_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (
            LOCK TABLE &targetschema..&_g_num_udf_tbl. FOR WRITE
            MERGE INTO &targetschema..&_g_num_udf_tbl. AS tt
            USING (
                select
                    treatment_sk,
                    num_udf_nm,
                    treatment_hash_val,
                    num_udf_val,
                    num_ext_column_nm
                from
                     &tempschema.&_g_nu_unq.
            ) AS st
            ON (
                      tt.treatment_sk       = st.treatment_sk
                and   tt.treatment_hash_val = st.treatment_hash_val
                and   tt.num_udf_nm        = st.num_udf_nm
            )
            WHEN NOT MATCHED THEN INSERT (
                    treatment_sk,
                    num_udf_nm,
                    treatment_hash_val,
                    num_udf_val,
                    num_ext_column_nm,
                    processed_dttm
            )
            VALUES (
                    st.treatment_sk,
                    st.num_udf_nm,
                    st.treatment_hash_val,
                    st.num_udf_val,
                    st.num_ext_column_nm,
                    TIMESTAMP %str(%')&pdttm.%str(%')
            )
        )by TERADATA;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_num_udf_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        execute (commit) by TERADATA;

        /* Drop temp table */
        execute ( drop table &tempschema.&_g_nu_unq. )by TERADATA;

        execute (commit) by TERADATA;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_TREATMENT_NUM_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

    %end; /* End of Step 5: CI_TREATMENT_NUM_UDF table processing */

    /* Step 6: Populate the CI_TREATMENT_DATE_UDF table after removing the records already exists */
    %if &_g_dt_udf_cnt. > 0 %then
    %do;
        /* Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_TREATMENT_DATE_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (
            LOCK TABLE &targetschema..&_g_dt_udf_tbl. FOR WRITE
            MERGE INTO &targetschema..&_g_dt_udf_tbl. AS tt
            USING (
                select
                    treatment_sk,
                    date_udf_nm,
                    treatment_hash_val,
                    date_udf_val,
                    date_ext_column_nm
                from
                     &tempschema.&_g_du_unq.
            ) AS st
            ON (
                      tt.treatment_sk       = st.treatment_sk
                and   tt.treatment_hash_val = st.treatment_hash_val
                and   tt.date_udf_nm        = st.date_udf_nm
            )
            WHEN NOT MATCHED THEN INSERT (
                    treatment_sk,
                    date_udf_nm,
                    treatment_hash_val,
                    date_udf_val,
                    date_ext_column_nm,
                    processed_dttm
            )
            VALUES (
                    st.treatment_sk,
                    st.date_udf_nm,
                    st.treatment_hash_val,
                    CAST(st.date_udf_val AS TIMESTAMP),
                    st.date_ext_column_nm,
                    TIMESTAMP %str(%')&pdttm.%str(%')
            )
        )by TERADATA;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_dt_udf_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        execute (commit) by TERADATA;

        /* Drop temp table */
        execute ( drop table &tempschema.&_g_du_unq. )by TERADATA;

        execute (commit) by TERADATA;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_TREATMENT_DATE_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of CI_TREATMENT_DATE_UDF table processing */

disconnect from TERADATA;

quit;

%if &MADebug. eq Y %then
%do;
    %put MADTTERA macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
