/****************************************************************************************/

/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.                       */

/*                     All Rights Reserved.                                             */

/*       This software is protected by copyright laws and                               */

/*                    international treaties                                            */

/*                                                                                      */

/*               U.S. Government Restricted Rights                                      */

/* Use, duplication, or disclosure of this software and related                         */

/*    documentation by the U.S. government is subject to the                            */

/*  Agreement with SAS Institute and the restrictions set forth                         */

/*       in FAR 52.227-19, Commercial Computer Software -                               */

/*                Restricted Rights ( June 1987 )                                       */

/*                                                                                      */

/****************************************************************************************/

/* NAME: mahswsql                                                                       */

/* VERSION: 5.4                                                                         */

/* DESCRIPTION: The macro is invoked from a stored proc and                             */

/* accepts a dataset name and no. of concurrent processes as                            */

/* input parameters. The purpose of the macro is to set up                              */

/* concurrent processes for all subjects in the dataset without                         */

/* exceeding the maximum permissible no. of processes.                                  */

/* Multiple queries for variables belonging to the same subject                         */

/* are executed sequentially while a concurrent process is set                          */

/* up for each unique subject.                                                          */

/* The concurrency is achieved by using the                                             */

/* MP CONNECT feature in SAS consisting of asynchronous                                 */

/* executions of RSUBMIT-ENDRSUBMIT blocks. Within each such                            */

/* block, mahstsql macro is invoked to generate metadata.                               */

/*                                                                                      */

/* PARAMETERS: Following are input parameters to the macro:                             */

/*        histo_ds - SAS dataset name (library.name).                                   */

/*        mametalibname - The MAMETA libname statement.                                 */

/*        madebug - Y/N value.                                                          */

/*        log_path - location of status.txt file for logging execution status.          */

/*        outtable - The MAMETA table name (only required for the old way).             */

/*        varlen - Maximum variable length (only required for the old way).             */

/*        mode - Whether to Replace or Append (only required for the old way).          */

/*        mp_flg - Y/N value indicating concurrent processing or                        */

/*        otherwise (Default: Y = concurrent processing).                               */

/*        max_procs - a numeric indicating max. no. of                                  */

/*        permissible concurrent processes. Defaulted to 1.                             */

/*        (Default: 1 = no MP CONNECT usage).                                           */

/*                                                                                      */

/* PRODUCT: MA                                                                          */

/* USAGE: %mahswsql(histo_ds,mametalibname,madebug,log_path,outtable=,varlen=,mode=,    */

/*        mp_flg=Y,max_procs=1);                                                        */

/*                                                                                      */

/****************************************************************************************/

/* History:                                                                             */

/* 24FEB06          First version of the macro.                                         */

/*                                                                                      */

/* 31Jul09 S0597899 Modified to remove the space %nrbquote() functtion                  */

/* 23Jun10 S0384412 add new parameter univar in macro parameters list and pass it while */

/*                  calling mahstsql macro                                              */

/* 25Jun10 S0594364 Use a common macro for getting the row count of tables              */

/* 05Apr11 S0735566 add missing macro name                                              */

/****************************************************************************************/

%macro mahswsql(histo_ds, mametalibname, madebug, log_path, outtable=, varlen=, mode=, mp_flg=Y, max_procs=1,univar=);



    /* Exit from macro in case of a previous error. */

    %if (&MAError ne %str()) %then %do;

        %return;

    %end;



    %let mameta1=%index(&mametalibname.,%str(%")) ;



    %if &mameta1 > 0 %then %do;

        %let mametalibname = %sysfunc(translate(&mametalibname.,"'",'"'));

    %end;



    /* For the old XML way, set the concurrent processing to sequential

       if it has not been set so already. */

    %if (&mp_flg=N) %then %do;

        %if (&max_procs > 1) %then %let max_procs = 1;

    %end;



    /* Define name of dataset to which input dataset will be copied. */

    %let histo_ds1 = WORK.histo1;



    /* Get the no. of observations in the input dataset. */

    /* S0594364 */

    %marowcnt( &histo_ds. , obs_cnt);

    %if (%upcase(&madebug.) = Y) %then %put No. of records = [&obs_cnt.];



    /* If there are no observations, initialize global variables, write out the status

       and return to the calling stored proc. */

    %if &obs_cnt. < 1 %then %do;

        %let MAError = 004;

        %let MAMsg = Macro [&sysmacroname.]. No records to process. Exiting Macro.;

        %return;

    %end;



    /* Create a copy of the input dataset by changing few column values to uppercase

       to avoid any problems in sorting and value comparisons. This will be fast as

       the input dataset is expected to have records of the order of few hundreds. */

    data &histo_ds1.;

        set &histo_ds.;

        datetime = upcase(datetime);

        datetype = upcase(datetype);

        /* In the XML for the old sequential method, name of the input and output tables

           as well as variable length and mode are not available while the XML for the

           new concurrent method has separate tags for these. */

        %if (&mp_flg=N) %then %do;

            intable = 'view1';

            outtable = upcase("&outtable.");

            varlen = &varlen.;

            mode = upcase("&mode.");

        %end;

        %else %do;

            outtable = upcase(outtable);

            mode = upcase(mode);

        %end;

    run;



    /* Now, sort the dataset by output table. This will ensure that multiple queries

       for the same subject are in sequence. */

    proc sort data = &histo_ds1.;

        by outtable;

    run;



    /* Associate each unique subject (identified by OUTTABLE) with an unique index value

       to facilitate processing. The dataset has already been sorted on outtable. */

    data &histo_ds1.;

        length tab_idx 3.;

        set &histo_ds1. end = done;

        by outtable;



        if first.outtable then do;

            tab_idx + 1;

        end;

        output;



        /* Store the no. of subjects in a macro variable. */

        if done then do;

            call symputx('subj_cnt',tab_idx);

        end;

    run;



    %if (%upcase(&madebug.) = Y) %then %put No. of Subject Tables = &subj_cnt.;



    /* Check the maximum permissible no. of concurrent processes.

       The value would decide sequential or concurrent processing. */

    %if (&max_procs. < 2) %then %do;        /* Perform Sequential processing. */

        filename histtemp temp lrecl=32767;



        data _null_; /* Data step to generate Sequential processing code. */

            retain idx_no outtabnm;

            length line $ 32767 line1 $ 32767;

            j = 1;

            length tab_mode $ 10;

            retain tab_mode '';

            set &histo_ds1. end = done;

            by outtable;



            file histtemp;       /* Write to file. */



            /* Store the Replace or Append mode for the first row of a subject table

               in a variable. */

            if first.outtable then do;

                tab_mode = mode;

            end;



            if idx_no ^= tab_idx then do; /* A new subject. */

                if _n_ = 1 then outtabnm = outtable;

                if _n_ > 1 then do;

                    put ' %mend; ';

                    put ' %mac_inc; ';

                    put '   ';

                    /* Log the status for each Subject execution in the new method. */

                    %if (&mp_flg=Y) %then %do;

                        put ' %mastatus( ' "&log_path.status.txt" ',context=' outtabnm '); ' ;

                    %end;

                    put '    ';

                    put '    ';

                end;



                /* Initialize idx_no with value of tab_idx

                   and write out code to the file for each new subject. */

                idx_no = tab_idx;

                outtabnm = outtable;



                put '%put  ************************************************************* ; ';

                put '%put  SUBJECT TABLE = ' outtable '; ';

                put '%put  ************************************************************* ; ';

                put '    ';



                /* Embed the include file code inside a macro for better error handling. */

                put ' %macro mac_inc; ';

                put '/* Exit from macro in case of a previous error. */ ';

                put ' %if (&MAError ne %str()) %then %do; ';

                put '     %return; ';

                put ' %end; ';

                put ' %else %do; ';

                put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                put '         %let MAError = 000; ';

                put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                put '         %return; ';

                put '     %end; ';

                put ' %end; ';

                put '   ';



                put ' proc delete data = base; run; ' ;

                put '    ';

            end;





            /* Scan the QUERY column and write out statements to the file. */

            line1 = scan( query, j, ';' );

            do while( line1 ^= '' );

                i = 1;

                line1 = trim(line1) || ';';

                line = scan(line1, i, '09'x );

                if line = '' then put line1 ';' ;

                do while( line ^= '' );

                      put line '0D'x;

                      i + 1;

                      line = scan( line1, i, '09'x );

                end;

                j + 1;

                line1 = scan( query, j, ';' );

            end;



            put '/* Check for errors in the above SQL. */ ';

            put ' %maerrinc(mahswsql); ';

            put '/* Exit from macro in case of an error. */ ';

            put ' %if (&MAError ne %str()) %then %do; ';

            put '     %return; ';

            put ' %end; ';

            put ' %else %do; ';

            put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

            put '         %let MAError = 000; ';

            put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

            put '         %return; ';

            put '     %end; ';

            put ' %end; ';



            if (datetime = 'NONE') then do;

                put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem '), ' Nbins ' , ' start ' , ' inc ' ,h' _n_ ' , %str(), excludeMissing=N , datetype=%str(),univar=' "&univar." '); ' ;

                put '    ';



                put '/* Exit if the above macro execution resulted in errors. */ ';

                put ' %if (&MAError ne %str()) %then %do; ';

                put '     %return; ';

                put ' %end; ';

                put ' %else %do; ';

                put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                put '         %let MAError = 000; ';

                put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                put '         %return; ';

                put '     %end; ';

                put ' %end; ';

                put '    ';



                put ' proc append base = base data = h' _n_ ' ; ' ;

                put ' run; ' ;

                put '    ';



                put '/* Check for errors in the above step. */ ';

                put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                put '/* Exit from macro in case of an error. */ ';

                put ' %if (&MAError ne %str()) %then %do; ';

                put '     %return; ';

                put ' %end; ';

                put ' %else %do; ';

                put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                put '         %let MAError = 000; ';

                put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                put '         %return; ';

                put '     %end; ';

                put ' %end; ';

                put '    ';

            end;

            else do;

                if (datetype = 'WHOLE') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ' , h' _n_+(-1) 'w, '  datetime ' , excludeMissing=N , datetype=Whole , univar=' "&univar." ' ); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'w; ' ;

                    put ' run; ' ;



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

                if (datetype = 'YEAR') then do;

                    /* S0597899 */

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ' , h' _n_+(-1) 'y, '  datetime ' , excludeMissing=N , datetype=Year , univar=' "&univar." ' ); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'y; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

                  if (datetype = 'MONTH') then do;

                    put ' %mahstsql( ' intable ' , ' var  ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ', h' _n_+(-1) 'm, '  datetime ' , excludeMissing=N , datetype=Month , univar= ' "&univar." '); ' ;



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'm; ' ;

                    put ' run; ' ;



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

                if (datetype = 'DAY') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ', h' _n_+(-1) 'd, '  datetime ' , excludeMissing=N , datetype=Day , univar= ' "&univar." '); ' ;



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'd; ' ;

                    put ' run; ' ;



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

            end;

            put '    ';



            if last.outtable then do;

                if trim(tab_mode) = 'REPLACE' then do;

                    put ' data ' outtable ' ; ' ;

                    put ' set base; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=DATA STEP); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

                else do;

                    put ' proc append base = ' outtable ' data = base; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

            end;



            if done then do;

                put ' %mend; ';

                put ' %mac_inc; ';

                put '    ';

                /* Log the status for each Subject execution in the new method. */

                %if (&mp_flg=Y) %then %do;

                    put ' %mastatus( ' "&log_path.status.txt" ',context=' outtabnm '); ' ;

                %end;

                put '    ';

            end;

        run; /* End of data step to generate Sequential processing code. */



    %end; /* End of Perform Sequential processing. */

    %else %do;    /* Perform concurrent processing. */



        /* Reset the value of maximum no. concurrent processes if it exceeds 2 x no. of CPUs. */

        %if (&max_procs > %eval(2*&SYSNCPU)) %then %let max_procs = %eval(2*&SYSNCPU);



        /* If the above count does not exceed the maximum permissible no. of concurrent processes,

           then schedule all of them to run concurrently. Otherwise, split the processing

           such that the limit is not exceeded. */

        %if (&subj_cnt. <= &max_procs.) %then %do;    /* Within max. limit. */

            %put Subjects[&subj_cnt.] not exceeding Maximum Processes[&max_procs.];

            filename histtemp temp mod lrecl=32767;



            data _null_;  /* Data step to generate RSUBMIT code. */

                length line $ 32767 line1 $ 32767;

                j = 1;

                length tab_mode $ 10;

                retain tab_mode '';

                retain idx_no outtabnm;

                set &histo_ds1. end = done;

                by outtable;



                /* Store the Replace or Append mode for the first row of a subject table

                   in a variable. */

                if first.outtable then do;

                    tab_mode = mode;

                end;



                file histtemp;             /* Write to file. */



                if idx_no ^= tab_idx then do; /* A new subject. */

                    if _n_ = 1 then outtabnm = outtable;

                    /* End the RSUBMIT block for the previous subject. */

                    if _n_ > 1 then do;

                        put ' %mend; ';

                        put ' %mac_inc; ';

                        put '   ';

                        put ' %mastatus( ' "&log_path.status.txt" ',context=' outtabnm '); ' ;

                        put '    ';

                        put 'endrsubmit; ';

                        put '    ';

                    end;

                    else do; /* If this is the first record in the dataset. */

                        put '%put ----------------------------------------------; ';

                        put '%put Starting MP CONNECT Sessions                     ; ';

                        put '%put Start Time - %sysfunc(datetime(), datetime21.); ';

                        put '%put ----------------------------------------------; ';

                        put '    ';

                    end;



                    /* Initialize idx_no with value of tab_idx

                       and write out RSUBMIT code to the file for each new subject. */

                    idx_no = tab_idx;

                    outtabnm = outtable;



                    put '%put  ************************************************************* ; ';

                    put '%put  SESSION ' tab_idx  '; ';

                    put '%put  SUBJECT TABLE = ' outtable '; ';

                    put '%put  ************************************************************* ; ';

                    put '    ';



                    put 'rsubmit job' tab_idx 'macvar = mvar_' tab_idx;

                    put 'sascmd = " !sascmdv -noobjectserver " ' ;

                    put 'wait=no;'; /* End RSUBMIT statement. */

                    put '    ';



                    put ' /* Call the macro to declare and initialize global variables. */ ';

                    put ' %maspinit; ';



                    put '%let log_path = ' "&log_path" ';' ;



                    put '%let madebug = ' "%UPCASE(&madebug.)" ';' ;

                    /* Metadata Library */

                    put " &mametalibname. ;" ;



                    if exist( 'base' ) then put ' proc delete data = base; run; ' ;

                    put '    ';



                    /* Embed the include file code inside a macro for better error handling. */

                    put ' %macro mac_inc; ';

                    put '/* Exit from macro in case of a previous error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '   ';

                end;



                /* Scan the QUERY column and write out statements to the file. */

                line1 = scan( query, j, ';' );

                do while( line1 ^= '' );

                    i = 1;

                    line1 = trim(line1) || ';';

                       line = scan(line1, i, '09'x );

                    if line = '' then put line1 ';' ;

                    do while( line ^= '' );

                          put line '0D'x;

                          i + 1;

                        line = scan( line1, i, '09'x );

                       end;

                    j + 1;

                       line1 = scan( query, j, ';' );

                end;



                put '    ';

                put '/* Check for errors in the above SQL. */ ';

                put ' %maerrinc(mahswsql); ';

                put '/* Exit from macro in case of an error. */ ';

                put ' %if (&MAError ne %str()) %then %do; ';

                put '     %return; ';

                put ' %end; ';

                put ' %else %do; ';

                put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                put '         %let MAError = 000; ';

                put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                put '         %return; ';

                put '     %end; ';

                put ' %end; ';

                put '   ';



                if (datetime = 'NONE') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem '), ' Nbins ' , ' start ' , ' inc ' ,h' _n_ ' , %str(), excludeMissing=N , datetype=%str() , univar= ' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ ' ; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

                else do;

                  if (datetype = 'WHOLE') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ' , h' _n_+(-1) 'w, '  datetime ' , excludeMissing=N , datetype=Whole , univar= ' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'w; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                  if (datetype = 'YEAR') then do;

                    /* S0597899 S0735566*/

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ' , h' _n_+(-1) 'y, '  datetime ' , excludeMissing=N , datetype=Year , univar= ' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'y; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                  if (datetype = 'MONTH') then do;

                    put ' %mahstsql( ' intable ' , ' var  ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ', h' _n_+(-1) 'm, '  datetime ' , excludeMissing=N , datetype=Month , univar= ' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'm; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                  if (datetype = 'DAY') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ', h' _n_+(-1) 'd, '  datetime ' , excludeMissing=N , datetype=Day , univar= ' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'd; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                end;

                put '    ';



                if last.outtable then do;

                    if trim(tab_mode) = 'REPLACE' then do;

                        put ' data ' outtable ' ; ' ;

                        put ' set base; ' ;

                        put ' run; ' ;

                        put '    ';



                        put '/* Check for errors in the above step. */ ';

                        put ' %maerrinc(mahswsql, proc_nm=DATA STEP); ';

                        put '/* Exit from macro in case of an error. */ ';

                        put ' %if (&MAError ne %str()) %then %do; ';

                        put '     %return; ';

                        put ' %end; ';

                        put ' %else %do; ';

                        put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                        put '         %let MAError = 000; ';

                        put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                        put '         %return; ';

                        put '     %end; ';

                        put ' %end; ';

                        put '    ';

                    end;

                    else do;

                        put ' proc append base = ' outtable ' data = base; ' ;

                        put ' run; ' ;

                        put '    ';



                        put '/* Check for errors in the above step. */ ';

                        put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                        put '/* Exit from macro in case of an error. */ ';

                        put ' %if (&MAError ne %str()) %then %do; ';

                        put '     %return; ';

                        put ' %end; ';

                        put ' %else %do; ';

                        put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                        put '         %let MAError = 000; ';

                        put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                        put '         %return; ';

                        put '     %end; ';

                        put ' %end; ';

                        put '    ';

                    end;

                end;



                if done then do;

                    put ' %mend; ';

                    put ' %mac_inc; ';

                    put '   ';

                    put ' %mastatus( ' "&log_path.status.txt" ',context=' outtabnm '); ' ;

                    put '    ';

                    put 'endrsubmit; ';

                    put '    ';

                end;

            run;  /* End of data step to generate RSUBMIT code. */



            /* Write post-MP CONNECT session code to histtemp file. */

            data _null_;

                   file histtemp ;

                   put '    ';

                   put 'waitfor _all_ '

                   %do i=1 %to &subj_cnt.;  /* Add all job names to WAITFOR statement */

                       " job&i."

                   %end;

                ';' /* End WAITFOR statement */

                ;   /* End current PUT statement */



                put '    ';

                put ' /* Get the logs for all the server sessions into the client session. */ ' ;

                %do i=1 %to &subj_cnt.;     /* Add all job names to RGET statement to get logs. */

                    put " rget job&i.; ";

                    put '    ';

                %end;



                put ' /* Disconnect from each server session. */ ' ;

                %do i=1 %to &subj_cnt.;     /* Add all job names to SIGNOFF statement */

                    put "signoff job&i.; ";

                    put '    ';

                %end;



                put '    ';

                put '%put --------------------------------------------; ';

                put '%put Ending MP CONNECT Sessions                     ; ';

                put '%put End Time - %sysfunc(datetime(), datetime21.); ';

                put '%put --------------------------------------------; ';

            run; /* End of data step to generate post MP CONNECT session code. */

        %end;         /* End of Within max. limit. */

        %else %do;    /* Exceeded max. limit. */

            %put Subjects[&subj_cnt.] exceeding Maximum Processes[&max_procs.];

            filename histtemp temp mod lrecl=32767;



            data _null_;  /* Data step to generate RSUBMIT code. */

                length line $ 32767 line1 $ 32767;

                j = 1;

                length tab_mode $ 10;

                retain tab_mode '';

                retain sess_no idx_no proc_cnt 0;

                retain outtabnm;

                set &histo_ds1. end = done;

                by outtable;



                /* Store the Replace or Append mode for the first row of a subject table

                   in a variable. */

                if first.outtable then do;

                    tab_mode = mode;

                end;



                file histtemp;



                if idx_no ^= tab_idx then do; /* A new subject. */

                    if _n_ = 1 then outtabnm = outtable;

                    idx_no = tab_idx;

                    if _n_ > 1 then do;

                           put ' %mend; ';

                           put ' %mac_inc; ';

                           put '   ';

                        put ' %mastatus( ' "&log_path.status.txt" ',context=' outtabnm '); ' ;

                        put '    ';

                        put 'endrsubmit; ';

                        put '    ';

                    end;

                    else do;  /* If this is the first record in the dataset. */

                        put '%put ----------------------------------------------; ';

                        put '%put Starting MP CONNECT Sessions                     ; ';

                        put '%put Start Time - %sysfunc(datetime(), datetime21.); ';

                        put '%put ----------------------------------------------; ';

                        put '    ';

                    end;



                    if proc_cnt = &max_procs. then do;    /* IF max. no. of processes reached. */

                        put ' /* Wait for the completion of all the server sessions. */ ' ;

                        put 'waitfor _all_ '

                        %do i=1 %to &max_procs.;  /* Add all job names to WAITFOR statement */

                            " job&i."

                        %end;

                        ';' /* end WAITFOR statement */

                        ;   /* end current PUT statement */



                        put '    ';

                        put ' /* Get the logs for all the server sessions into the client session. */ ' ;

                        do i=1 to proc_cnt;     /* Add all job names to RGET statement to get logs.*/

                            put    'rget job' i ';' ;

                            put '    ';

                        end;



                        put ' /* Disconnect from each server session. */ ' ;

                        do i=1 to proc_cnt;     /* SIGNOFF from all the sessions. */

                            put    'signoff job' i ';' ;

                            put '    ';

                        end;



                        put '    ';

                        put '%put --------------------------------------------; ';

                        put '%put Ending MP CONNECT Sessions                     ; ';

                        put '%put End Time - %sysfunc(datetime(), datetime21.); ';

                        put '%put --------------------------------------------; ';

                        put '    ';



                        /* If there are more subjects to be processed, start another

                           set of MP CONNECT sessions. */

                        if (idx_no <= &subj_cnt.) then do;

                            put '%put ----------------------------------------------; ';

                            put '%put Starting MP CONNECT Sessions                     ; ';

                            put '%put Start Time - %sysfunc(datetime(), datetime21.); ';

                            put '%put ----------------------------------------------; ';

                            put '    ';

                        end;



                        /* Reset values for the new set of MP CONNECT sessions. */

                        proc_cnt = 0;

                        sess_no = 0;

                    end;  /* End of IF max. no. of processes reached. */



                    /* Increment process count, initialize idx_no with value of tab_idx

                       and write out RSUBMIT code to the file for each new subject. */

                    proc_cnt + 1;

                    sess_no + 1;

                    outtabnm = outtable;



                    put '%put ************************************************************** ; ';

                    put '%put SESSION ' sess_no '; ';

                    put '%put SUBJECT TABLE = ' outtable '; ';

                    put '%put ************************************************************** ; ';

                    put '    ';



                    put 'rsubmit job' sess_no 'macvar = mvar_' sess_no;

                    put 'sascmd = " !sascmdv -noobjectserver " ' ;

                    put 'wait=no;'; /* End RSUBMIT statement. */

                    put '    ';



                    put ' /* Call the macro to declare and initialize global variables. */ ';

                    put ' %maspinit; ';



                    put '%let log_path = ' "&log_path" ';' ;



                    put '%let madebug = ' "%UPCASE(&madebug.)" ';' ;

                    /* Metadata Library */

                    put " &mametalibname. ;" ;



                    if exist( 'base' ) then put ' proc delete data = base; run; ' ;

                    put '    ';



                    /* Embed the include file code inside a macro for better error handling. */

                    put ' %macro mac_inc; ';

                    put '/* Exit from macro in case of a previous error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '   ';

                end;  /* End of IF a new subject. */



                /* Scan the QUERY column and write out statements to the file. */

                line1 = scan( query, j, ';' );

                do while( line1 ^= '' );

                    i = 1;

                    line1 = trim(line1) || ';';

                    line = scan(line1, i, '09'x );

                    if line = '' then put line1 ';' ;

                    do while( line ^= '' );

                          put line '0D'x;

                          i + 1;

                          line = scan( line1, i, '09'x );

                    end;

                    j + 1;

                    line1 = scan( query, j, ';' );

                end;



                put '/* Check for errors in the above SQL. */ ';

                put ' %maerrinc(mahswsql); ';

                put '/* Exit from macro in case of an error. */ ';

                put ' %if (&MAError ne %str()) %then %do; ';

                put '     %return; ';

                put ' %end; ';

                put ' %else %do; ';

                put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                put '         %let MAError = 000; ';

                put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                put '         %return; ';

                put '     %end; ';

                put ' %end; ';

                put '   ';



                if (datetime = 'NONE') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem '), ' Nbins ' , ' start ' , ' inc ' ,h' _n_ ' , %str(), excludeMissing=N , datetype=%str(), univar=' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ ' ; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                end;

                else do;

                  if (datetype = 'WHOLE') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ' , h' _n_+(-1) 'w, '  datetime ' , excludeMissing=N , datetype=Whole , univar=' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'w; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                  if (datetype = 'YEAR') then do;

                    /* S0597899 */

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ' , h' _n_+(-1) 'y, '  datetime ' , excludeMissing=N , datetype=Year , univar=' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'y; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                  if (datetype = 'MONTH') then do;

                    put ' %mahstsql( ' intable ' , ' var  ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ', h' _n_+(-1) 'm, '  datetime ' , excludeMissing=N , datetype=Month , univar=' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'm; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                  if (datetype = 'DAY') then do;

                    put ' %mahstsql( ' intable ' , ' var ' , %nrbquote(' dataitem ') , ' Nbins ' , ' start ' , ' inc ', h' _n_+(-1) 'd, '  datetime ' , excludeMissing=N , datetype=Day , univar=' "&univar." '); ' ;

                    put '    ';



                    put '/* Exit if the above macro execution resulted in errors. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';



                    put ' proc append base = base data = h' _n_ +(-1) 'd; ' ;

                    put ' run; ' ;

                    put '    ';



                    put '/* Check for errors in the above step. */ ';

                    put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                    put '/* Exit from macro in case of an error. */ ';

                    put ' %if (&MAError ne %str()) %then %do; ';

                    put '     %return; ';

                    put ' %end; ';

                    put ' %else %do; ';

                    put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                    put '         %let MAError = 000; ';

                    put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                    put '         %return; ';

                    put '     %end; ';

                    put ' %end; ';

                    put '    ';

                  end;

                end;

                put '    ';



                if last.outtable then do;

                    if trim(tab_mode) = 'REPLACE' then do;

                        put ' data ' outtable ' ; ' ;

                        put ' set base; ' ;

                        put ' run; ' ;

                        put '    ';



                        put '/* Check for errors in the above step. */ ';

                        put ' %maerrinc(mahswsql, proc_nm=DATA STEP); ';

                        put '/* Exit from macro in case of an error. */ ';

                        put ' %if (&MAError ne %str()) %then %do; ';

                        put '     %return; ';

                        put ' %end; ';

                        put ' %else %do; ';

                        put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                        put '         %let MAError = 000; ';

                        put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                        put '         %return; ';

                        put '     %end; ';

                        put ' %end; ';

                        put '    ';

                    end;

                    else do;

                        put ' proc append base = ' outtable ' data = base; ' ;

                        put ' run; ' ;

                        put '    ';



                        put '/* Check for errors in the above step. */ ';

                        put ' %maerrinc(mahswsql, proc_nm=PROC APPEND); ';

                        put '/* Exit from macro in case of an error. */ ';

                        put ' %if (&MAError ne %str()) %then %do; ';

                        put '     %return; ';

                        put ' %end; ';

                        put ' %else %do; ';

                        put '     %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do; ';

                        put '         %let MAError = 000; ';

                        put '         %let MAMsg = Macro[mahswsql]. There was an error in this or a called macro. Check the log for details.; ';

                        put '         %return; ';

                        put '     %end; ';

                        put ' %end; ';

                        put '    ';

                    end;

                end;



                if done then do;

                    put ' %mend; ';

                    put ' %mac_inc; ';

                    put '   ';

                    put ' %mastatus( ' "&log_path.status.txt" ',context=' outtabnm '); ' ;

                    put '    ';

                    put 'endrsubmit; ';

                    put '    ';



                    put ' /* Wait for the completion of all the server sessions. */ ' ;

                    put 'waitfor _all_ ';

                    do i=1 to proc_cnt;  /* Add all job names to WAITFOR statement. */

                        put 'job' i;

                    end;

                    put ';' ; /* End WAITFOR statement. */



                    put '    ';

                    put ' /* Get the logs for all the server sessions into the client session. */ ' ;

                    do i=1 to proc_cnt;     /* Add all job names to RGET statement to get logs.*/

                        put    'rget job' i ';' ;

                        put '    ';

                    end;



                    put ' /* Disconnect from each server session. */ ' ;

                    do i=1 to proc_cnt;     /* SIGNOFF from all the sessions. */

                        put    'signoff job' i ';' ;

                        put '    ';

                    end;



                    put '    ';

                    put '%put --------------------------------------------; ';

                    put '%put Ending MP CONNECT Sessions                     ; ';

                    put '%put End Time - %sysfunc(datetime(), datetime21.); ';

                    put '%put --------------------------------------------; ';

                end;

            run;   /* End of data step to generate RSUBMIT code. */

        %end; /* End of Exceeded max. limit. */

    %end; /* End of Perform concurrent processing. */



    options autosignon;

    %include histtemp;                /* Include the temp file in the main flow. */

    filename histtemp clear;        /* Deassign the file reference. */



%mend;

