/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: mabcmeta.sas                                            */
/* VERSION: 5.4                                                */
/* DESCRIPTION: Common Data Interface macro to read BC and       */
/*              common model metadata from input stream/Payload  */
/*              URI to create the master table                   */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     bc_meta_tbl - BC metadata table name                      */
/*     bulkdatalibref - XML Payload URI for bulkdata             */
/*     bulkdatamap - XML Map file for Payload URI                */
/*     cdi_meta_tbl - Master metadata table name                 */
/*     Inputstream - Input stream name                           */
/*     dm_meta_tbl -data model metadata table name               */
/*     data_tbl - Actual data table name                         */
/*     stpcall - CH/RH/PT Stp which calls the mabcmeta           */
/*     xml_pl_map - Payload map name                             */
/*     xml_pl_ref - Payload libref name                          */
/*     xmlflag - Y/N to indicate whether to use XML Stream       */
/*     updatemode - B/T Bulk or Trickling                        */
/*     Retrials -No of retry attempts for SWA connection failure */
/*     RetryInterval-Interval between retrails                   */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %mabcmeta                                              */
/*                                                               */
/*****************************************************************/
/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 30Dec2008  S0471986   Modified the error message to reflect the more accurate reason  */
/*                                                                                       */
/* 23Feb2009  S0558672   New input stream (XML) from MA with list of rsptracking_cd      */
/*                                                                                       */
/* 03Jun2009  S0584561   Support for new SWA Client                                      */
/*                                                                                       */
/* 30Jun2009  S0587037   Added 1. Retry logic for SWA                                    */
/*                             2. new parameters Retrials,RetryInterval                  */
/*                                                                                       */
/* 03Jul2009  S0591871   Set Syscc value for error condition 732 so that to print the    */
/*                       Success with Error message status for SWA. Also don't write     */
/*                       bad records to .csv file for SWA.                               */
/*                                                                                       */
/* 21Jan2010  S0634713   Refresh the Contact Records for the removed cell                */
/*                                                                                       */
/* 14May2010  Taskid-57851 MA5.4Iteration5-Update Stored Process for CH "exclude" status */
/* 25Jun2010  S0594364  Use a common macro for getting the row count of tables           */
/*****************************************************************************************/


%macro mabcmeta(bc_meta_tbl=, bulkdatalibref=,bulkdatamap=, cdi_meta_tbl=, Inputstream=, dm_meta_tbl=,
                data_tbl=, retrials=, retryinterval=, stpcall=, xml_pl_map=, xml_pl_ref=, xmlflag=, updatemode=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MABCMETA - Current Version of the code: 5.4.1;
    %put Last Modified on 14May2010;
%end;

/* Exit the process if there is an error from the calling macro */
%if (&MAError. ne %str()) %then
%do;
    %return;
%end;

/* Step 1: Create SAS dataset from XML Stream */

/* S0634713: The Commnication_SK macro variable is used to find out the list of cell_package_sk
   which needs to be refreshed.  This overrides the defect S0558672 fix so the MA XML stream
   reading part is removed */

/* S0558672: List of resptracking_cd from MA CH call. This helps to reset the customer
   records in CH table and contact counts in case of 0 customer */

/* Taskid-57851 */
%if %upcase(&action.) eq UPOP or %upcase(&action.) eq USTA %then
%do;
    %let xmllib=%str(libname &Inputstream. xml);
    %macrtlib(&xmllib.,MABCMETA);

    /* Exit the process if there is an error */
    %if (&MAError ne %str()) %then
    %do;
        %return;
    %end;
    /* Covert XML stream to SAS data set */
    proc copy in=&Inputstream. out=work;
    run;

    /* Clear the XML library */
    libname &Inputstream.;

    /*Exit the process if there is an error */
    %if &SYSERR. > 4 %then
    %do;
        %let MAError = 722;
        %let MAMsg = Macro[&sysmacroname.]. PROC Copy failed - while attempting to create dataset out of XML stream. Check the log for details.;
        %return;
    %end;

    /* Exit the process if the ch_status input table is empty */
    %marowcnt( &ch_status_tbl. , ch_status_cnt);
    %if &ch_status_cnt. le 0 %then
    %do;
        %let MAError = 733;
        %let Syscc = 100000;
        %let MAMsg = Macro[&sysmacroname.]. Ch_Status input table is empty and could not continue the process. Check the log for details.;
    %end;
    %return; /* The rest of the processing in this macro is not needed for MA processing */
%end;

%if %upcase(&xmlflag.) eq Y  %then
%do;
        %let xmllib=%str(libname &Inputstream. xml);
        %macrtlib(&xmllib.,MABCMETA);

        /* Exit the process if there is an error */
        %if (&MAError ne %str()) %then
        %do;
               %return;
        %end;

        /* Covert XML stream to SAS data set
           for trickling insert it will result in 3 tables (CCSConnect,CCSDM and Chupdate)
           for bulk update it will result in 2 tables (CCSConnect, CCSDM)*/
        proc copy in=&Inputstream. out=work;
        run;

        /* Clear the XML library */
        libname &Inputstream.;

        /*Exit the process if there is an error */
        %if &SYSERR. > 4 %then
        %do;
            %let MAError = 722;
            %let MAMsg = Macro[&sysmacroname.]. PROC Copy failed - while attempting to create dataset out of XML stream. Check the log for details.;
            %return;
        %end;

        /* Exit the process if any of the input table is empty */
        data _null_;
                call symputx('bc_meta_obs_cnt', attrn(open("&bc_meta_tbl."),"NOBS"));
                call symputx('dm_meta_obs_cnt', attrn(open("&dm_meta_tbl."),"NOBS"));
        run;

        %if &bc_meta_obs_cnt.= 0 or &dm_meta_obs_cnt.=0 or &SYSERR. > 4 %then
        %do;
            %let MAError = 733;
            %let Syscc = 100000;
            %let MAMsg = Macro[&sysmacroname.]. Input table is empty and could not continue the process. Check the log for details.;
            %return;
        %end;

        /* Convert the BCID and SUBID to string so that it can have leading zeros for single digit code
           This code block of additional processing can be removed once the we are sure that the content
           of the CCSConnect and CCSDM XML data are sorted by bcid and subid respectively before streaming in */
        data &bc_meta_tbl. (drop=bcidn);
            length bcid $2.;
            set &bc_meta_tbl. (rename=(bcid=bcidn));
            if lengthn(left(put(bcidn,2.))) = 1 then
            do;
                bcid='0'||left(bcidn);
            end;
            else if lengthn(left(put(bcidn,2.))) = 2 then
            do;
                bcid=left(bcidn);
            end;
        run;

        data &dm_meta_tbl. (drop=subidn bcidn);
            length subject_id bcid $2.;
            set &dm_meta_tbl. (rename=(subject_id=subidn bcid=bcidn));
            if lengthn(left(put(subidn,2.))) = 1 then
            do;
                subject_id='0'||left(subidn);
            end;
            else if lengthn(left(put(subidn,2.))) = 2 then
            do;
                subject_id=left(subidn);
            end;
            if lengthn(left(put(bcidn,2.))) = 1 then
            do;
                bcid='0'||left(bcidn);
            end;
            else if lengthn(left(put(bcidn,2.))) = 2 then
            do;
                bcid=left(bcidn);
            end;
        run;

        /*Exit the process if there is an error */
        %if &SYSERR. > 4 %then
        %do;
            %let MAError = 723;
            %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;
            %return;
        %end;

        /* Read Bulk update data from the XML URI */
        %if %upcase(&updatemode.) eq B %then
        %do;
            filename &xml_pl_ref. url "&bulkdatalibref." ;
            filename &xml_pl_map. url "&bulkdatamap.";

            %let blklib=%str(libname &xml_pl_ref. xml xmlmap=&xml_pl_map.);
            %macrtlib(&blklib.,MABCMETA);

            /* Exit the process if there is an error */
            %if (&MAError ne %str()) %then
            %do;
                   %return;
            %end;

            /* Covert XML URI to SAS data set */
            proc copy in=&xml_pl_ref. out=work;
            run;

            /* Clear the XML library */
            libname &xml_pl_ref.;

            /*Exit the process if there is an error */
            %if &SYSERR. > 4 %then
            %do;
                %let MAError = 722;
                %let MAMsg = Macro[&sysmacroname.]. PROC Copy failed - while attempting to create dataset out of XML stream. Check the log for details.;
                %return;
            %end;
            /* Exit the process if the data table is empty */
            /* S0594364 */
            %marowcnt( &data_tbl. , data_obs_cnt);
            /* Special Treatment For SDM that don't return Error even though no records results to process */
            %if &data_obs_cnt. = 0 %then
            %do;
                %let MAError = 732;
                %let MAMsg = Macro[&sysmacroname.]. Input table is empty and could not continue the process. Check the log for details.;
                %return;
            %end;

            %if &SYSERR. > 4 %then
            %do;
                %let MAError = 733;
                %let Syscc = 100000;
                %let MAMsg = Macro[&sysmacroname.]. Error reading Input data table and could not continue the process. Check the log for details.;
                %return;
            %end;
         %end; /* End of reading Bulk Update data */
         %else %if %upcase(&updatemode.) eq D %then /* S0584561 SWA Support */
         %do;
            /* S0587037 */
            %let RetryId=0;                                             /* Retry loop counter 0 - Default Attempt*/
            %let RetryAttempts=%eval(&retrials + 1);                    /* RetryAttempts = 1(default) + no of Retries */
            %do %while(&RetryId. lt &RetryAttempts.);                   /* in case of error loop until Retry Attempts */
                %put Attempt [&RetryId.];                               /* Print Retry Attempt number */
                %macrtlib(%unquote(&bulkdatalibref.),MABCMETA);
                %if (&MAError ne %str()) AND (&RetryId. ne (&RetryAttempts. - 1)) %then
                %do;
                    data _null_;
                        call sleep(&RetryInterval.,1);                  /* Wait for Retry interval */
                    run;
                    %let RetryId=%eval(&RetryId. + 1);                  /* increment the retry counter */
                    %let MAError=;                                      /* reset MAError before retry */
                %end;
                %else
                    %let RetryId=&RetryAttempts.;
            %end;

            /* After all retries , Exit the process if there is an error */
            %if (&MAError ne %str()) %then
            %do;
                %return;
            %end;
         %end;
%end; /* End of reading XML Stream */


/* STEP 2: Process the input data to create bcid/subject_id and Cell_Package_sk  */
/* S0584561 - Create the inputset in work area for further processing instead of overwritting the
   input table passed down for the processing */

data &data_tbl.(drop=resptracking_cd _rtcln) work._bc_subid(keep=bcid subject_id _rtcln);
    length bcid subject_id $2 cell_package_sk 8;
    %if %length(&inputtable.) > 0 %then
    %do;
        set &inputtable.;
    %end;
    %else
    %do;
        set &data_tbl.;
    %end;
    _rtcln=lengthn(left(put(resptracking_cd,14.)));
    bcid=substrn(left(put(resptracking_cd,14.)),(_rtcln-4)+1,2);
    subject_id=substrn(left(put(resptracking_cd,14.)),(_rtcln-2)+1,2);
    cell_package_sk=substrn(left(put(resptracking_cd,14.)),1,(_rtcln-4));
run;

/*Exit the process if there is an error */
%if &SYSERR. > 4 %then
%do;
    %let MAError = 723;
    %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;
    %return;
%end;

/*STEP 3: Create a temp table with only the distinct BC_ID and Subject_ID.  This is to
          avoid the 256 character length if at all we get large number of BC's and Subject_id's          */
proc sort data=&data_tbl.;
    by bcid subject_id;
run;

/* Remove the duplicates from temp table */
proc sort data=work._bc_subid nodupkey;
    by bcid subject_id;
run;

/*STEP 4: Join the above table with CCSConnect and then with CCSDM to retain only those
          BC's and Subject_ID's for which we have got records in the Input table*/
proc sort data=&bc_meta_tbl.;
    by bcid;
run;

/* Sort the Subject Metadata table */
proc sort data=&dm_meta_tbl.;
    by bcid subject_id;
run;

/* Exit the process if there is an error */
%if &SYSERR. > 4 %then
%do;
    %let MAError = 724;
    %let MAMsg = Macro[&sysmacroname.]. Proc Sort failed - while processing metadata table. Check the log for details.;
    %return;
%end;

/* Drop the additional records from BC Metadata table */
data &bc_meta_tbl.;
    merge &bc_meta_tbl.(in=a) work._bc_subid(in=b keep=bcid);
    by bcid;
    if a and b then
        output;
run;

/*Exit the process if there is an error */
%if &SYSERR. > 4 %then
%do;
    %let MAError = 725;
    %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to process Merge stmt. Check the log for details.;
    %return;
%end;

/* Drop the additional records from Data Model Metadata table */
data &dm_meta_tbl.;
    merge &dm_meta_tbl.(in=a) work._bc_subid(in=b);
    by bcid subject_id;
    if a and b then
        output;
run;

/*Exit the process if there is an error */
%if &SYSERR. > 4 %then
%do;
    %let MAError = 725;
    %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to process Merge stmt. Check the log for details.;
    %return;
%end;

/* Create a master metadata table by joining BC and DM tables */
data &cdi_meta_tbl.;
    merge &bc_meta_tbl.(in=a) &dm_meta_tbl.(in=b);
    by bcid;
    if a and b then
        output;
run;

/*Exit the process if there is an error */
%if &SYSERR. > 4 %then
%do;
    %let MAError = 725;
    %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to process Merge stmt. Check the log for details.;
    %return;
%end;

/* S0594364 */
%marowcnt( &cdi_meta_tbl. , meta_obs_cnt);
/* Special Treatment For SDM that don't return Error even though no records results to process */
%if &meta_obs_cnt. = 0 %then
%do;
    /* S0591871: The SYSCC value is set so that the CICommon can handle the 732 special code
       and create Success with Error message status */
    %let SYSCC = 100000;
    %let MAError = 732;
    %let MAMsg = Macro[&sysmacroname.]. Data not available for the BC in use. Check the log for details.;

    /* Call print obs macro to print Data table to a file */
    /* S0591871: Print the bad records to .csv file only for SDM and RDM clients*/
    %if %upcase(&updatemode.) eq B or %upcase(&updatemode.) eq T %then
    %do;
        %maprnobs(badrsp=Y,fl_loc=MAMISC, srctbl=&data_tbl., stpcall=&stpcall.);
    %end;

    %return;
%end;

%mend;
