/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/*******************************************************************************/
/* NAME: macdibl.sas                                                           */
/* VERSION: 5.4                                                                */
/* DESCRIPTION: Common Data Interface macro to bulkload/Upload inputtable to   */
/*              data base schema for further processing. This macro is         */
/*              introduced to assign the correct subjec_key data type to the   */
/*              input table received from XML stream based on the SAS formats  */
/*                                                                             */
/* Parameters                                                                  */
/* Macro variables:                                                            */
/*     action - USTA/USTADTTM/UPOP/UPOPDTTM                                    */
/*     bcid - 2 digit BC id                                                    */
/*     data_tbl - Actual data table name                                       */
/*     dbdsoptions - Data set options for bulk loading                         */
/*     in_tbl_nm - Input table name without the library                        */
/*     dbmstemp - Y or Null                                                    */
/*     dbtempschema - Data base temporary schema                               */
/*     execdt - Execute Datetime                                               */
/*     libstmt - Libname for CH database table                                 */
/*     matables - Input table library                                          */
/*     stpcall - CH/RH/PT Storeprocess which invoked this macro                */
/*     sub_id - 2 digit subject id                                             */
/*     sub_key - Comma separated subject_key for other than MA client          */
/*     sub_key_fmt - SAS Formats associated with the above key columns         */
/*     templibname - db temp libname                                           */
/*     updatemode - B/T Bulk or Trickling                                      */
/*     xmlflag - Y/N to indicate whether to use XML Stream                     */
/*                                                                             */
/*                                                                             */
/* PRODUCT: MA                                                                 */
/* USAGE: %macdibl                                                             */
/*                                                                             */
/*******************************************************************************/
/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 17Jan2008 - First Version of the Code                                                 */
/*                                                                                       */
/* S0479890 - Assign correct data type and format to avoid any possible                  */
/* upload and implicit type casting errors                                               */
/*                                                                                       */
/* S0486525 - Modifications to support NETEZZA                                           */
/*                                                                                       */
/* S0486300 - Modifications to support SPDS - sinmvp                                     */
/*                                                                                       */
/* DATE       Defect No  Comments/updated by                                             */
/* 04Sep2008  S0528358   Create explicit UPI while bulkloading input table to Teradata   */
/*                       schema to avoid default index creation - sinmvp                 */
/* 04Nov2008  S0532953   Include the error check below the drop tables blocks within     */
/*                       the its corresponding %do %end blocks                           */
/*                                                                                       */
/* 23Feb2009  S0558672   Upload only the cell_package_sk in the dist_cpk table           */
/*                                                                                       */
/* 22Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 15Jul2009 S0515661   SPDS Temp Libname changes                                        */
/*                                                                                       */
/* 11Jan2010 S0625365   Teradata use all key columns in the primary index                */
/*                                                                                       */
/* 21Jan2010 S0634713   Refresh the Contact Records for the removed cell                 */
/*                                                                                       */
/* 24Mar2010 S0651614   Aster Partition Key Change                                       */
/*                                                                                       */
/* 19Jul2010 S0684982   Aster Number Partition Key                                       */
/*                                                                                       */
/* 14Dec2010 S0724281   Modify file statement to add lrecl=32767                         */
/*                                                                                       */
/* 28Jun2011 S0744863   Added support for GREENPLUM                                      */
/*                                                                                       */
/* 09Dec2011 S0791301  create / refer all the table/column names in upper case           */
/* 03May2012 S0859079  move semicolon after data statement after if condn.               */
/*****************************************************************************************/

/* Macro introduced to fix the subject variable's data type issues from SDM and RDM clients */

%macro macdibl(action=, bcid=, data_tbl=, dbdsoptions=, stpcall=,
               sub_id=, sub_key=, sub_key_fmt=);
/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA02;
    %put Last Modified on 03May2012;
%end;

/* Step 1: Check any errors from calling macro and exit immediately if any */
%if (&MAError. ne %str()) %then
%do;
    %return;
%end;

/* S0651614 S0684982 */
%global MppDBKey;
%if &dbengine = ASTER %then
%do;
    %let MppDBKey = %str(dbtype=(CELL_PACKAGE_SK ='NUMERIC(10)' ) partition_key='CELL_PACKAGE_SK') ;
%end;
%else %if &dbengine = GREENPLM %then
%do;
    %let MppDBKey = %str(dbtype=(CELL_PACKAGE_SK='NUMERIC(10)' ) distributed_by='distributed by (CELL_PACKAGE_SK)');
%end;


/* Step 2: Pharse the comma separated sub_key and sub_key_fmt to create individual variabls.
           Also create the variable data type based on the format */

%let maxnumkey=0;
%let done=0;

/* Read comma separated subject_key in to individual macro variables */
%if &action.=UPOPDTTM or &action.=USTADTTM %then
%do;
    %do %until (&done=1);
       %let maxnumkey = %eval(&maxnumkey+1);
       %let chsubjectidcol&maxnumkey.=%qscan(%bquote(&sub_key.),&maxnumkey.,',');
       /* adding a single white space to the subject_key_fmt to read the leading and trailing blanks
          correctly using the qscan function */
       %let subkeyfmt&maxnumkey.=%qscan(%bquote(%str( )&sub_key_fmt.%str( )),&maxnumkey.,',');

       /* Find the length of each variable */
       %if &&subkeyfmt&maxnumkey. ne %str() %then
       %do;
           %if %qsubstr(%bquote(%left(&&subkeyfmt&maxnumkey.)),1,1) eq %str($) %then
           %do;
               data _null_;
                    fmt=compress("&&subkeyfmt&maxnumkey.","$1234567890","K");
                    call symput("subkeylen&maxnumkey.",strip(fmt)||strip('.'));
                run;
           %end;
           %else
           %do;
                %let subkeylen&maxnumkey. =%str(8.);
           %end;
        %end;

       %if &&chsubjectidcol&maxnumkey.= %str() and &&subkeyfmt&maxnumkey.=%str() %then
       %do;
           %let done=1;
           %let maxnumkey=%eval(&maxnumkey-1);
       %end;
    %end;

    /* Step 2: Subset the input table by bcid & sub_id.  Also rename the subject_id(s) to the target table
               column name and assign the correct data type */
    /* This WORKLIB is introduced make a generic reference for SPDS temp library as
       well other engine work area */
    /* S0515661 */
    /*data &worklib..in_tbl_&sub_id.;*/
    data &worklib..&unqintbl.&sub_id.;
    %do i= 1 %to &maxnumkey.;
        %if &&subkeyfmt&i. ne %str() %then
        %do;
            length &&chsubjectidcol&i. &&subkeylen&i.;
            format &&chsubjectidcol&i. &&subkeyfmt&i.;
        %end;
    %end;
        set &data_tbl;
        %do i= 1 %to &maxnumkey.;
            %if &&subkeyfmt&i. ne %str() %then
            %do;
                &&chsubjectidcol&i. = SUBJECT_ID&i.;
            %end;
        %end;
        where BCID="&bcid." and SUBJECT_ID="&sub_id.";
        rename
        %do i= 1 %to &maxnumkey.;
            %if &&subkeyfmt&i. eq %str() %then
            %do;
                SUBJECT_ID&i.=&&chsubjectidcol&i.
            %end;
        %end;
        ; /* Termination for rename statement */
        drop
        %do i= 1 %to &maxnumkey.;
            %if &&subkeyfmt&i. ne %str() %then
            %do;
                SUBJECT_ID&i.
            %end;
        %end;
        ; /*Termination for drop statement */
    run;

%end; /* End of If statement for both Step1 and 2 */
%if &SYSERR. > 4 or (&SYSCC. >=2 and &SYSCC. ne 4) %then
%do;
    %let MAError = 729;
    %let MAMsg = Macro[&sysmacroname.]. Error in DATA STEP. Check the log for details.;
    %return;
%end;

/* Step 3: Upload the input / new subset table to db schema when dbengine is not SASSPDS */

/* Upload the input table for action= UPOP/UPOPDTTM/USTADTTM plus when action=USTA and
   popexists=0.  When action=USTA and popexists=1 only the tiny table (which contains
   only the distinct cell_package_sk) gets uploaded to avoid possible performance
   issues later especialy with DB2 */

/* Upload code block if the input table from XML stream */
%if &dbengine. ne SASSPDS %then
%do;
    %if (&action. eq UPOPDTTM) or (&action. eq USTADTTM) %then
    %do; /* Drop the table from target schema if exists already */
        %if %sysfunc( exist( &templibname..&unqintbl.&sub_id. ) ) %then
        %do;
            proc sql noerrorstop;
               drop table &templibname..&unqintbl.&sub_id.;
            quit;
            /* S0532953 Included the error check within the same block         */

            %if &SYSDBRC. ne 0 %then
            %do;
                %if &dbengine. eq NETEZZA %then
                %do;
                    %if &SYSDBRC. ne 108 %then
                    %do;
                        %let MAError = 750;
                        %let MAMsg = Macro[&sysmacroname]. Error dropping input table from db location. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
                        %return;
                    %end;
                %end;
                %else
                %do;
                    %let MAError = 750;
                    %let MAMsg = Macro[&sysmacroname]. Error dropping input table from db location. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
                    %return;
                %end;
            %end;
        %end;

        /* i) Special treatment for Teradata to create index file */
        /* S0724281   Modify file statement to add lrecl=32767 */
        %if &dbengine. eq TERADATA %then
        %do;
            filename chsrc catalog 'work.temp.chtemp.source';

            data _null_;
                file chsrc flowover lrecl=32767;
                put " data &templibname..&unqintbl.&sub_id.
                      (&dbdsoptions DBCREATE_TABLE_OPTS=%str(%')UNIQUE PRIMARY INDEX (CELL_PACKAGE_SK,"
                %do i=1 %to &maxnumkey.;
                    %if &i. > 1 %then
                    %do;
                        ","
                    %end;
                    "&&chsubjectidcol&i."
                %end;

                /* S0625365 Add other key columns to the list of key columns to avoid the duplicate rows if cell_pack & sub are repeated */
                %if %upcase(&stpcall.) eq CH and &action. eq UPOPDTTM %then
                %do;
                    ", CDTTM "
                %end;
                %else %if %upcase(&stpcall.) eq PT %then
                %do;
                    ", TREATMENT_SK, TREATMENT_HASH_VAL, PTDTTM "
                %end;
                %else %if %upcase(&stpcall.) eq RH %then
                %do;
                    ", TREATMENT_SK, TREATMENT_HASH_VAL, RESPONSE_CD, RDTTM "
                %end;

                ") %str(%') );";
            run;
            /* Insert the newly created Data statement for Teradata */
            %inc chsrc;
                /* S0515661 */
                /*set in_tbl_&sub_id.;*/
                set &unqintbl.&sub_id.;
            run;
        %end;
        /* S0651614 */
        %else %if &dbengine. eq ASTER or &dbengine. eq GREENPLM %then
        %do;
            data &templibname..&unqintbl.&sub_id.
            %if %length(&dbdsoptions.) > 1 %then
            %do;
                ( &MppDBKey. &dbdsoptions. );
            %end;
                ; /* Termination for the data statement */
                set &unqintbl.&sub_id.;
            run;

        %end;
/*        %else %if &dbengine. eq GREENPLM %then*/
/*        %do;*/
/*            data &templibname..&unqintbl.&sub_id.*/
/*            %if %length(&dbdsoptions.) > 1 %then*/
/*            %do;*/
/*                ( &DistributionKey. &dbdsoptions. );*/
/*            %end;*/
/*                 ;*/ /* Termination for the data statement */
/*                set &unqintbl.&sub_id.;*/
/*            run;*/

/*        %end;*/
        %else
        %do;
            data &templibname..&unqintbl.&sub_id.
            %if %length(&dbdsoptions.) > 1 %then
            %do;
                ( &dbdsoptions. )
            %end;
                ; /* Termination for the data statement */
                /* S0515661 */
                /*set in_tbl_&sub_id.;*/
                set &unqintbl.&sub_id.;
            run;
        %end;
    %end; /* End for uploading input table for SDM and RDM */
    %else %if (&action. eq USTA) or (&action. eq UPOP) %then
    %do;
        /* Read comma separated subject_key in to individual macro variables
           This is need during Terdata Index creation */
        %do %until (&done=1);
            %let maxnumkey = %eval(&maxnumkey+1);
            %let chsubjectidcol&maxnumkey.=%qscan(%bquote(&sub_key.),&maxnumkey.,',');

            %if &&chsubjectidcol&maxnumkey.= %then
            %do;
                %let done=1;
                %let maxnumkey=%eval(&maxnumkey-1);
            %end;
        %end;
        /* Drop the table from target schema if exists already */
        %if %sysfunc( exist( dbtmplib.%upcase(&unqintbl.) ) ) %then
        %do;
            proc sql noerrorstop;
                drop table &templibname..%upcase(&unqintbl.);
            quit;
            /* S0532953 Included the error check within the same block         */
            %if &SYSDBRC. ne 0 %then
            %do;
                %if &dbengine. eq NETEZZA %then
                %do;
                    %if &SYSDBRC. ne 108 %then
                    %do;
                        %let MAError = 750;
                        %let MAMsg = Macro[&sysmacroname]. Error dropping input table from db location. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
                        %return;
                    %end;
                %end;
                %else
                %do;
                     %let MAError = 750;
                     %let MAMsg = Macro[&sysmacroname]. Error dropping input table from db location. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
                     %return;
                %end;
            %end;
        %end;

        /* Upload code block for MA */
        /* S0528358: Create explicit UPI for Teradata to avoid the default index creation */
        /* S0724281   Modify file statement to add lrecl=32767 */
        %if &dbengine. eq TERADATA %then
        %do;
            filename chsrc catalog 'work.temp.chtemp.source';

            data _null_;
                file chsrc flowover lrecl=32767 ;
                put " data &templibname..&unqintbl.
                      (&dbdsoptions DBCREATE_TABLE_OPTS=%str(%')UNIQUE PRIMARY INDEX (CELL_PACKAGE_SK,"
                %do i=1 %to &maxnumkey.;
                    %if &i. > 1 %then
                    %do;
                        ","
                    %end;
                    "&&chsubjectidcol&i."
                %end;
                ") %str(%') );";
            run;
            /* Insert the newly created Data statement for Teradata */
            %inc chsrc;
                set &data_tbl.;
            run;
        %end;
        /* S0651614 */
        %else %if &dbengine. eq ASTER or &dbengine. eq GREENPLM %then
        %do;
            data &templibname..&unqintbl.
            %if %length(&dbdsoptions.) > 1 %then
            %do;
                ( &MppDBKey. &dbdsoptions. );
            %end;
                ; /* Termination for the data statement */
                set &data_tbl.;
            run;
        %end;
/*        %else %if &dbengine. eq GREENPLM %then*/
/*        %do;*/
/*            data &templibname..&unqintbl.&sub_id.*/
/*            %if %length(&dbdsoptions.) > 1 %then*/
/*            %do;*/
/*                ( &DistributionKey. &dbdsoptions. );*/
/*            %end;*/
/*                 ; */ /* Termination for the data statement */
/*                set &data_tbl.;*/
/*            run;*/
/**/
/*        %end;*/
        %else
        %do;
            data &templibname..&unqintbl.
            %if %length(&dbdsoptions.) > 1 %then
            %do;
                ( &dbdsoptions. )
            %end;
                ; /* Termination for the data statement */
                set &data_tbl.;
            run;
        %end;
    %end; /* End for uploading MA input table */

    /* Upload the tinny table which only has the distinct cell_package_sk */
    %if %upcase(&stpcall.) eq CH and ( (&action. eq USTA) or (&action. eq UPOP)) %then
    %do;
        %if %sysfunc( exist( &templibname..%upcase(&unqintbl._CPK ) ) ) %then
        %do; /* Drop the table from target schema if exists already */
            proc sql noerrorstop;
                drop table &templibname..%upcase(&unqintbl._CPK);
            quit;
            /* S0532953 Included the error check within the same block         */
            %if &SYSDBRC. ne 0 %then
            %do;
                %if &dbengine. eq NETEZZA %then
                %do;
                    %if &SYSDBRC. ne 108 %then
                    %do;
                        %let MAError = 750;
                        %let MAMsg = Macro[&sysmacroname]. Error dropping input table from db location. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
                        %return;
                    %end;
                %end;
                %else
                %do;
                     %let MAError = 750;
                     %let MAMsg = Macro[&sysmacroname]. Error dropping input table from db location. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
                     %return;
                %end;
            %end;
        %end;

        /* S0651614 S0859079*/
        %if &dbengine. eq ASTER or &dbengine. eq GREENPLM %then
        %do;
            data &templibname..&unqintbl._CPK
            %if %length(&dbdsoptions.) > 1 %then
            %do;
                ( &MppDBKey. &dbdsoptions. )
            %end;
                ; /* Termination for the data statement */
                 set &unqintbl._CPK;
            run;
        %end;
/*        %else %if &dbengine. eq GREENPLM %then*/
/*        %do;*/
/*            data &templibname..&unqintbl._CPK*/
/*            %if %length(&dbdsoptions.) > 1 %then*/
/*            %do;*/
/*                ( &DistributionKey. &dbdsoptions. );*/
/*            %end;*/
/*                 ; */ /* Termination for the data statement */
/*                 set &unqintbl._cpk;*/
/*            run;*/
/*        %end;*/
        %else
        %do;
            /* CPK is kept in caps so that works fine for NETEZZA */
            data &templibname..&unqintbl._CPK
            %if %length(&dbdsoptions.) > 1 %then
            %do;
                ( &dbdsoptions. )
            %end;
                ;/* Termination for the data statement */
                 /* S0558672: Upload only the cell_package_sk */
                 /* set dist_cpk (keep=cell_package_sk); */
                 /* S0634713 cell_package_sk is read out of the ci_cell_package table and not from XML */
                 set &unqintbl._CPK;
            run;
        %end;
    %end; /* End for uploading distinct cell_package_sk */

    /* If temp table did not load successfully, no point in continuing */
    %if &SYSDBRC. ne 0 %then
    %do;
        %let MAError=751;
        %let syscc=100000;
        %let MAMsg=Macro[&sysmacroname.] Error in uploading input table to database. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
        %return;
    %end;

%end; /* End for Step 3: upload block */
%mend;
