/*****************************************************************/
/* Copyright (c) 2007 - 2012 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/*****************************************************************/
/* NAME: macdiche.sas                                            */
/* VERSION: 5.41                                                 */
/* DESCRIPTION: Common Data Interface macro to Update Contact    */
/*              History table using Explicit Pass through        */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - USTA/USTADTTM/UPOP/UPOPDTTM                      */
/*     chschema - Database schema                                */
/*     chtable - CH Physical column name                         */
/*     data_tbl - Actual data table name                         */
/*     dbconnect - Data base explicit pass through connect stmt  */
/*     dbengine - Data base engine from connect stmt             */
/*     libname - Library name where CH table resides             */
/*     sub_id - 2 digit subject id value from metadata table     */
/*     sub_key - CH physical column name of primary key          */
/*     subjectidcol - Pyhsical column name in inputtable         */
/*     tempschema - Data base temporary schema                   */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macdiche                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* S0480043 - Added the tempschema to identify the uploaded table correctly for the      */
/* RDM/SDM client when the dbtempschema is set with a valid value                        */
/*                                                                                       */
/* S0479722 - Updated the select distinct subquery (used for the MA campaign process)    */
/* which caused a major performance issue to refer the skinny subset table which will    */
/* have only distinct cell_package_sk                                                    */
/*                                                                                       */
/* S0486525 - Modifications to support NETEZZA                                           */
/* a) Added code to determine the source and target table.                               */
/* b) Added code to wrap the targettble and sourcetbl variables                          */
/*    with double quotes, that's how explicit passthru works here                        */
/* c) Added code for treating datetime value in Netezza                                  */
/* d) Added code to update contact hsitory table for SDM                                 */
/*                                                                                       */
/* S0486300 - Modifications to support SPDS                                              */
/*                                                                                       */
/* S0498541 - Convert function used with style code 121 for OLEDB connection to process  */
/* the datetime value correct irrespective of the format in database                     */
/*                                                                                       */
/* S0499640 - Fix to handle the DBMSTEMP=YES option (global temporary schema to upload   */
/* table) correct                                                                        */
/*                                                                                       */
/* S0513401 - Neoview support                                                            */
/*            1. SQLXRC after dbconnect statement                                        */
/*            2. Convert the datetime value to proper format                             */
/*            3. Commit not required for Neoview                                         */
/*                                                                                       */
/* DATE      Defect No  Comments                                                         */
/* 17Sep2008 S0521156   Date-time values for SASSPDS,ODBC connection needs to be handled */
/*                      separately, so that the datetime number value is passed to the   */
/*                      SQL processing                                                   */
/*                                                                                       */
/* 15Oct2008 S0504678   Package_Hash_Val added to CH tble for Dynamic Treatment project  */
/*                                                                                       */
/* 18Dec2008 S0534266   Special processing block for NEOVIEW delete                      */
/*                                                                                       */
/* 29Dec2008 S0548795   Remove the double quote (") around Netezza table names to be able*/
/*                      to work with case insensitive table names                        */
/*                                                                                       */
/* 16Jan2009 S0540093   Re-arranged the %if block to avoid the unnecessary COMMIT        */
/*                      transaction during Index creation                                */
/*                                                                                       */
/* 26Feb2009 S0504678   Package_Hash_Val added to CH tble to support Dynamic Treatments  */
/*                      for Selection campaigns                                          */
/*                                                                                       */
/* 04Jun2009 S0583077  Removed the drop table block                                      */
/*                                                                                       */
/* 22Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 15Jul2009 S0515661   SPDS Temp Libname changes                                        */
/*                                                                                       */
/* 20Apr2010 S0652355   remove commit for netezza                                        */
/*                                                                                       */
/* 05May2010 S0663006   Add ASTER db specific code                                       */
/*                                                                                       */
/* 19May2010  Taskid-57851 MA5.4Iteration5-Update Stored Process for CH "exclude" status */
/*                                                                                       */
/* 21Jul2010 S0608258   Change the connect statement error check                         */
/*                                                                                       */
/* 23Sep2010 S0693256   new column support CONTACT_DT                                    */
/*                                                                                       */
/* 12Nov2010 S0715116   Add 2 new columns External_Contact_Info_ID1 & ID2                */
/*                                                                                       */
/* 28Jun2011 S0744863   Added support for GREENPLUM                                      */
/*                                                                                       */
/* 09Dec2011 S0791301  create / refer all the table/column names in upper case           */
/*                                                                                       */
/* 13Aug2013 S1000559  Add COMMIT for DB2 after each DELETE in macdiche                  */
/*****************************************************************************************/
%macro macdiche(action=, chschema=, chtable=, data_tbl=, dbconnect=, dbengine=, libname=, sub_id=,
               sub_key=, subjectidcol=, tempschema= );
/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MACDICHE Current Version of the code: 541MA09;
    %put Last Modified on 13Aug2013;
    %put MACDICHE macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Initialize the process variables to zero */
%let maxnumkey=0;
%let done=0;
%let popexists=0;

/* Step 2: Read comma separated subject_key in to individual macro variables */
%do %until (&done=1);
   %let maxnumkey = %eval(&maxnumkey+1);
   %let chsubjectidcol&maxnumkey.=%qscan(%bquote(&sub_key.),&maxnumkey.,',');
   %if &subjectidcol. ne %str() %then
       %let subjectidcol&maxnumkey.=%qscan(%bquote(&subjectidcol.),&maxnumkey.,',');

   %if &&chsubjectidcol&maxnumkey.= %then
   %do;
       %let done=1;
       %let maxnumkey=%eval(&maxnumkey-1);
   %end;
%end;

%let dbengine = %upcase(&dbengine.) ;

/* Create space delimited subject key to use in a required places instead of resolving individual variables */
data _null_;
    call symput('sub_key_sp',tranwrd("&sub_key.",","," "));
run;

%if &SYSERR. > 4 %then
%do;
     %let MAError = 729;
     %let MAMsg = Macro[&sysmacroname.]. Error in DATA STEP. Check the log for details.;
     %return;
%end;

/* Step 3: Assign input table name to common variable as it is different for databases */
%if &action. eq USTA or &action. eq UPOP %then
%do;
    /* S0515661 */
    %let subintbl=&unqintbl.;
%end;
%else %if &action. eq UPOPDTTM or &action. eq USTADTTM %then
%do;
    /* S0515661 */
    %let subintbl=&unqintbl.&sub_id.;
%end;

/* Step 3a: Find out the popexists to branch the process to update or insert block correctly */
/* The tiny table for SPDS is created as dist_cpk in SPDTEMP domain so using the name directly */
%if &action. eq USTA %then
%do;
    proc sql noprint;
        select 1 into:popexists
            from &libname..&chtable. t
            where t.CELL_PACKAGE_SK in(select s.CELL_PACKAGE_SK from
            /* S0515661 */
            &templibname..&subintbl._CPK s);
    quit;

    /* Exit the process if there is any error */
    %if &SQLRC. > 4 %then
    %do;
        %let MAError = 752;
        %let MAMsg = Macro[&sysmacroname]. Error in executing PROC SQL query, SQLRC [&SQLRC]. Check the log for details.;
        %return;
    %end;
%end;
/* S0534266: Special Processing block for NEOVIEW Delete to set the MULTI COMMIT ON */
%else %if &action. eq UPOP and %upcase(&dbengine.) eq NEOVIEW %then
%do;
    filename chdel catalog "work.temp.chdel.source";

    data _null_;
        file chdel flowover lrecl=32767;
        set &templibname..&subintbl._CPK end=eof;
        if _n_=1 then
        do;
            put ' execute(set transaction multi commit on) by &dbengine. ;';
        end;
        put ' execute(delete from &targettbl. where';
        put '     CELL_PACKAGE_SK=' CELL_PACKAGE_SK ') by &dbengine. ;';

        if eof then
        do;
            put ' execute(set transaction multi commit off) by &dbengine. ;';
        end;
    run;
%end;
/* S0714067: Special processing for Mainframe DB2 and the same applies to GREENPLUM as well */
/* S1000559 */
%else %if &action. eq UPOP and %upcase(&dbengine.) eq DB2 or %upcase(&dbengine.) eq GREENPLM %then
%do;
    filename chdel catalog "work.temp.chdel.source";

    data _null_;
        file chdel flowover lrecl=32767;
        set &templibname..&subintbl._CPK end=eof;
        put " ";
        put ' execute(delete from &targettbl. where';
        put '     CELL_PACKAGE_SK=' CELL_PACKAGE_SK ') by &dbengine. ;';
        %if %upcase(&dbengine.) eq DB2 %then
        %do;
            put ' execute(COMMIT) by &dbengine. ;';
        %end;
        put " ";
    run;
%end;
/* Step 4: For SPDS Explicit Passthrough is used but the input table is not uploaded.  So
           sort the input table and retain only the required variables */
/* This block removed for new defect for SPDS */

/* Step 5: Find out the popexists for MA campaigns is moved to macdiupl.sas script part of the defect S0479722 */

/* Step 7: Assign Input and target table to common varaible as it is different for db */
/* For SPDS the schema name will supplied in the connect statement */
%if &dbengine. eq SASSPDS %then
%do;
    %let targettbl = &chtable.;
    %let sourcetbl = &templibname..&subintbl.;
%end;
/* Netezza: Schema is not supported in Neteeza, so a separate database is used for the temporary processings.
   The database= option is included part of the connect to statement. The targettbl variable
   will have only the table name */
/* S0548795 comment out the double quote table names */
%else %if %upcase(&dbengine.) eq NETEZZA %then
%do;
    %let targettbl = &chtable.;
    %let sourcetbl = &tempschema.&subintbl.;
%end;
%else
%do;
    %let targettbl = &chschema..&chtable.;
    %let sourcetbl = &tempschema.&subintbl.;
%end;

/* Taskid-57851 action specific processing separated into different macros */
%if &action. eq USTADTTM %then
%do;
    %macheusd(dbengine=&dbengine.);
    %if &MAError. ne %str() %then
    %do;
        %return;
    %end;
%end;
%if (&action. eq USTA and &popexists=1 ) %then
%do;
     %macheust(dbengine=&dbengine.);
    %if &MAError. ne %str() %then
    %do;
        %return;
    %end;
%end;
%else %if &action. eq UPOP or &action. eq UPOPDTTM or (&action. eq USTA and &popexists=0)  %then
%do;
    proc sql noerrorstop;
        &dbconnect.;;
        /* Exit the process if the pass-through connect statement fails */
        /* S0608258: NEOVIEW returns a -1 even in case of successfull connection and SQLXMSG check results in
           operand error when the connection is not available. Add SQLRC condition check along with SQLXRC to
            bypass this issue */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=753;
            %let syscc=100000;
            %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %put &MAMsg;
            %return;
        %end;

        /* Step 10: If Contact population is to be updated by UPOP first do a full delete */
        %if &action. eq UPOP %then
        %do; /* S0548795: Netezza specific delete block with the double quote table name is removed */
            /* S0534266: Special Processing block for NEOVIEW Delete to set the MULTI COMMIT ON */
            %if %upcase(&dbengine.) eq NEOVIEW or %upcase(&dbengine.) eq DB2 or %upcase(&dbengine.) eq GREENPLM %then
            %do;
                %include chdel;
            %end;
            /* S0663006 */
            %else %if %upcase(&dbengine.) eq ASTER %then
            %do;
                execute(delete from &targettbl.
                        using &sourcetbl._CPK
                        where &targettbl..CELL_PACKAGE_SK =&sourcetbl._CPK.CELL_PACKAGE_SK
                ) by &dbengine.;

            %end;
            %else
            %do;
                  execute(delete from &targettbl.
                             where CELL_PACKAGE_SK in
                             %if &dbengine. ne SASSPDS %then
                             %do;
                                 (select CELL_PACKAGE_SK from &sourcetbl._CPK )
                             %end;
                             %else %if &dbengine. eq SASSPDS %then
                             %do;
                                /* S0515661 */
                                 (select CELL_PACKAGE_SK from &&templibname..&unqintbl._CPK )
                             %end;
                         ) by &dbengine.;

            %end;
            /* Check for any errors in update process: A non Zero return code is set by the
                data base if there is no rows to process (er. 1403 in Oracle and 100 in DB2).  So
                use the SQLRC > 4 condition in conjection with SQLXRC*/
            %if &dbengine. ne SASSPDS %then
            %do;
                %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
                %do;
                   %let MAError=757;
                   %let SYSCC=100000;
                   %let MAMsg=Macro[&sysmacroname.] Error deleting Contact records from Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                   %return;
                %end;
            %end;
            %else %if &dbengine. eq SASSPDS %then
            %do;
                %if &SQLRC. > 4 %then
                %do;
                   %let MAError=757;
                   %let SYSCC=100000;
                   %let MAMsg=Macro[&sysmacroname.] Error in deleting Contact records from Contact History target table. SQLRC=[&SQLRC.]  Check the log for details.;
                   %return;
                %end;
            %end;
            /* S0652355 */
            %if (&dbengine. eq ORACLE) or (&dbengine. eq DB2) or (&dbengine. eq TERADATA) %then
            %do;
                execute (commit) by &dbengine.;
            %end;
        %end; /* End of full delete */
        /* S0693256 - insert new column CONTACT_DT */
        execute(insert into &targettbl.
                 (
                  &sub_key., /* Comma separated subject key */
                  CELL_PACKAGE_SK,
                  CONTACT_HISTORY_STATUS_CD,
                  PACKAGE_HASH_VAL,
                  CONTACT_DTTM,
                  CONTACT_DT
                  %if &action. eq UPOPDTTM %then
                  %do;
                      ,
                      EXTERNAL_CONTACT_INFO_ID1,
                      EXTERNAL_CONTACT_INFO_ID2
                  %end;
                 )
                select
                  &sub_key.,
                  CELL_PACKAGE_SK,
                  CHSTATUS_CD,
                  PACKAGE_HASH_VAL,
                  %if &action. eq UPOP or (&action. eq USTA and &popexists=0) %then
                  %do;
                      /* %str(%')&CHSTATUS_CD%str(%'), */
                      %if &dbengine. eq ORACLE %then
                      %do;
                          TO_TIMESTAMP(%str(%')&cdttm%str(%'),'YYYY-MM-DD HH24:MI:SS.FF'),
                          TO_DATE( %str(%')&ctdate%str(%')  ,  'YYYY-MM-DD' )
                      %end;
                      %else %if &dbengine. eq TERADATA  or &dbengine. eq NEOVIEW %then
                      %do;
                          TIMESTAMP %str(%')&cdttm%str(%'),
                          DATE %str(%')&ctdate%str(%')
                      %end;
                      %else %if &dbengine. eq SASSPDS %then
                      %do;
                          &execdt.,  /*S0521156 - use execdt for MA updates */
                          datepart(&execdt.)
                      %end;
                      %else %if  %upcase(&dbengine.) eq NETEZZA %then
                      %do;
                          TIMESTAMP(%str(%')&cdttm%str(%')),
                          DATE(%str(%')&ctdate%str(%'))
                      %end;
                      %else %if %upcase(&dbengine.) eq OLEDB %then
                      %do;
                          CONVERT(DATETIME,%str(%')&cdttm%str(%'),121),
                          CONVERT(DATE,REPLACE(%str(%')&ctdate%str(%'),'-','/'),111)
                      %end;
                      %else
                      %do;
                          %str(%')&cdttm%str(%'),
                          %str(%')&ctdate%str(%')
                      %end;
                  %end;
                  %else %if &action. eq UPOPDTTM %then
                  %do;
                      %if &dbengine. eq ORACLE %then
                      %do;
                          TO_TIMESTAMP(CDTTM,'YYYY-MM-DD HH24:MI:SS.FF'),
                          TO_DATE(ctdate,'YYYY-MM-DD'),
                      %end;
                      %else %if &dbengine. eq TERADATA or  &dbengine. eq NEOVIEW %then
                      %do;
                          CAST(CDTTM AS TIMESTAMP),
                          CAST(ctdate AS DATE),
                      %end;
                      %else %if  %upcase(&dbengine.) eq NETEZZA %then
                      %do;
                          TIMESTAMP(CDTTM),
                          DATE(CTDATE),
                      %end;
                      %else %if %upcase(&dbengine.) eq OLEDB %then
                      %do;
                          CONVERT(DATETIME,CDTTM,121),
                          CONVERT(DATE,REPLACE(CTDATE,'-','/'),111),
                      %end;
                      %else %if &dbengine. eq SASSPDS %then
                      %do;/*S0521156-convert the string dttm to number value */
                          /*input( put(input(substr(cdttm,1,10),YYMMDD10.),DATE9.)||':'||substr(cdttm,12), datetime24.3 ),*/
                          /*for spds use already uploaded contact_dttm column*/
                          contact_dttm,
                          datepart(contact_dttm),
                      %end;
                      %else
                      %do;
                          CDTTM,
                          CTDATE,
                      %end;
                      EXTERNAL_CONTACT_INFO_ID1,
                      EXTERNAL_CONTACT_INFO_ID2
                  %end;
                from &sourcetbl.
               )by &dbengine.;

        /* Check for any errors in the Insert process */
        %if &dbengine. ne SASSPDS %then
        %do;
            %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
            %do;
               %let MAError=758;
               %let SYSCC=100000;
               %let MAMsg=Macro[&sysmacroname.] Error inserting contacts in to Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %end;
        %end;
        %else %if &dbengine. eq SASSPDS %then
        %do;
            %if &SQLRC. > 4 %then
            %do;
               %let MAError=758;
               %let SYSCC=100000;
               %let MAMsg=Macro[&sysmacroname.] Error inserting contacts in to Contact History target table. SQLRC=[&SQLRC.]. Check the log for details.;
            %end;
        %end;
        /* Call the print obs macro to print the input table records to a file only for other clients*/
        %if &action.=UPOPDTTM and &MAError. ne %str() %then
        %do;
            %maprnobs(action=&action.,badrsp=N, expimplflag=E, fl_loc=MAMISC, srclib=&templibname. , srctbl=&subintbl.,
                      stpcall=CH, sub_key=%str(&sub_key.), tgtlib=&libname., tgttbl=&chtable., updatemode=&updatemode.,
                      xmlflag=&xmlflag.);
            %return;
        %end;
        /* S0652355 */
        %if (&dbengine. eq ORACLE) or (&dbengine. eq DB2) or (&dbengine. eq TERADATA) %then
        %do;
            execute (commit) by &dbengine.;
        %end;
         disconnect from &dbengine.;
    quit; /* End of Explicit Pass-through Proc SQL block */
%end;
%if &MADebug. eq Y %then
%do;
    %put MACDICHE macro End Time - %sysfunc(datetime(), datetime24.2);
%end;
%mend;
