/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: macdichi.sas                                            */
/* VERSION: 5.4                                                  */
/* DESCRIPTION: Common Data Interface macro to Update Contact    */
/*              History table using Implicit Pass through        */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - USTA/USTADTTM/UPOP/UPOPDTTM                      */
/*     chtable - CH Physical column name                         */
/*     data_tbl - Actual data table name                         */
/*     dbengine - Data base engine from connect stmt             */
/*     libname - Library name where CH table resides             */
/*     libstmt - Libname statement for CH table                  */
/*     sub_id - 2 digit subject id value from metadata table     */
/*     sub_key - CH physical column name of primary key          */
/*     subjectidcol - Pyhsical column name in inputtable         */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macdichi                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 17Sep2008  S0521156   Date-time values for SASSPDS,ODBC connection needs to be handled*/
/*                       separately, so that the datetime number value is passed to the  */
/*                       SQL processing for SDM/RDM updates. MA updates will use the     */
/*                       macro variable execdt                                           */
/*                                                                                       */
/* 15Oct2008  S0504678   Package_Hash_Val added to CH tble for Dynamic Treatment project */
/*                                                                                       */
/* 30Dec2008  S0472135   Code added to remove the temp tables in case of any error       */
/*                                                                                       */
/* 12Feb2009  S0561995   Change the Package_Hash_val null value assignment               */
/*                                                                                       */
/* 26Feb2009 S0504678   Package_Hash_Val added to CH tble to support Dynamic Treatments  */
/*                      for Selection campaigns                                          */
/*                                                                                       */
/* 14May2010  Taskid-57851 MA5.4Iteration5-Update Stored Process for CH "exclude" status */
/*                                                                                       */
/* 23Sep2010 S0693256   new column support CONTACT_DT                                    */
/*                                                                                       */
/* 12Nov2010 S0715116   Add 2 new columns External_Contact_Info_ID1 & ID2                */
/*                                                                                       */
/* 09Dec2011 S0791301  create / refer all the table/column names in upper case           */
/*****************************************************************************************/

%macro macdichi(action=, chtable=, data_tbl=, dbengine=, libname=, libstmt=, sub_id=, sub_key=,
                subjectidcol=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA02;
    %put Last Modified on 09Dec2011;
    %put MACDICHI macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Initialize the process variables to zero */
%let maxnumkey=0;
%let done=0;
%let popexists=0;

/* Step 2: Read comma separated subject_key in to individual macro variables */
%do %until (&done=1);
   %let maxnumkey = %eval(&maxnumkey+1);
   %let chsubjectidcol&maxnumkey.=%qscan(%bquote(&sub_key.),&maxnumkey.,',');
    %if &subjectidcol. ne %str() %then
       %let subjectidcol&maxnumkey.=%qscan(%bquote(&subjectidcol.),&maxnumkey.,',');

   %if &&chsubjectidcol&maxnumkey.= %then
   %do;
       %let done=1;
       %let maxnumkey=%eval(&maxnumkey-1);
   %end;
%end;

/* Create space delimited subject key to use in a required places instead of resolving individual variables */
data _null_;
    call symput('sub_key_sp',tranwrd("&sub_key.",","," "));
run;

/* Step 3: Assign share variable */
%if &dbengine.=SQLSRV or &dbengine.=ODBC %then
    %let share = 0;
%else
%do;
    %if %index(%upcase(&libstmt), SERVER =) > 0 or %index(%upcase(&libstmt), SERVER=) >0 %then
        %let share = 1;
    %else
        %let share = 0;
%end;

/* Step 4: Assign input/target table name to common variable as it is different for databases */
%if &action. eq USTA or &action. eq UPOP %then
%do;
    %let subintbl=&templibname..&unqintbl.;
    %let targettbl=&libname..&chtable.;
%end;
%else %if &action. eq UPOPDTTM or &action. eq USTADTTM %then
%do;
    %let subintbl=&templibname..&unqintbl.&sub_id.;
    %let targettbl=&libname..&chtable.;
%end;

/* Step 5: If user requests to update status in MA make sure a population already exists on CH*/
%if &action. eq USTA %then
%do;
    /* The Select didstinct sub-query might be performance intensive try revising if possible */
    proc sql noprint;
        select distinct 1 into:popexists
            from &targettbl. t
            where t.CELL_PACKAGE_SK in(select distinct(s.CELL_PACKAGE_SK)
                                         from &subintbl._CPK s);
    quit;

    %if &SQLRC. > 4 %then
    %do;
        %let MAError = 724;
        %let MAMsg = Macro[&sysmacroname]. Error in executing PROC SQL query, SQLRC [&SQLRC]. Check the log for details.;
        %return;
    %end;
%end;

/* Step 6: Sort the input table and retain only the required variabls */
/* This step is removed part of the defect S0521156 fix as the input tables
   are uploaded to the database schema (similar to explicit pass-through processing)
   and further used from there*/

/* Step 7: Implicit pass through block to process the CH updates */
proc sql
%if &share = 1 %then UNDO_POLICY=NONE;
%do;
;
%end;

/* Step 8:  For MA Client action=USTA update all the contacts exist in target table for */
/* the given set of cell_package_sk from in the input table */
/* Package_Hash_Val doesn't get changed during selection campaign status update */
/* Taskid-57851 - update CONTACT_HISTORY_STATUS_CD using the CHSTATUS_CD from &sourcetbl. */
/* existing code for action USTADTTM is doing the same thing so removing the &action. eq USTA and &popexists=1 block */

/* Step 9: Update Contact records from Input table for other than MA Client */
/* Multiple sub-query to update individual column might be performance intensive consider
   revising this using Modify Statement */
/* Package_Hash_Val doesn't get changed during failed contacts update from SDM */
/* Taskid-57851 - update CONTACT_HISTORY_STATUS_CD using the CHSTATUS_CD from &sourcetbl. */
%if &action. eq USTADTTM %then
%do;
    update &targettbl. ch
        set
            CONTACT_HISTORY_STATUS_CD =
                (select CHSTATUS_CD from &subintbl. intbl
                        where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK AND
                        %do i=1 %to &maxnumkey.;
                            %if &i. > 1 %then
                            %do;
                                AND
                            %end;
                            ch.&&chsubjectidcol&i. = intbl.&&chsubjectidcol&i. /* both intbl and ch have the same column names as per the above Proc Sort*/
                        %end;
                 ),
             CONTACT_DTTM =
                 (select
                        /*S0521156-convert the string dttm to number value */
                        input( put(input(scan(CDTTM,1,' '),YYMMDD10.),DATE9.)||':'||scan(CDTTM,2,' '), datetime24.3 )
                        from &subintbl. intbl
                        where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK AND
                        %do i=1 %to &maxnumkey.;
                            %if &i. > 1 %then
                            %do;
                                AND
                            %end;
                            ch.&&chsubjectidcol&i. = intbl.&&chsubjectidcol&i.
                        %end;
                  )
        where exists
             (select 1 from &subintbl. intbl
                     where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK AND
                     %do i=1 %to &maxnumkey.;
                         %if &i. > 1 %then
                         %do;
                             AND
                         %end;
                         ch.&&chsubjectidcol&i. = intbl.&&chsubjectidcol&i.
                     %end;
              );

    /* Check for errors in update process and exit */
    %if &SQLRC. > 4 %then
    %do;
        %let MAError = 730;
        %let MAMsg = Macro[&sysmacroname.] Error in Updating Contact History target table SQLRC:&SQLRC. Check the log for details.;

        %if &action.=USTADTTM and &MAError. ne %str() %then
        %do;
            /* Call the print obs macro to print the input table records to a file */
            %maprnobs(action=&action.,badrsp=N, expimplflag=I, fl_loc=MAMISC, srclib=&templibname. , srctbl=&subintbl.,
                  stpcall=CH, sub_key=%str(&sub_key.), tgtlib=&libname., tgttbl=&chtable., updatemode=&updatemode.,
                  xmlflag=&xmlflag.);
        %end;
        %return;
    %end;
%end;/* End of Update Contact records from Input table */

%if (&action. eq USTA and &popexists=1 ) %then
%do;
    update &targettbl. ch
        set
            CONTACT_HISTORY_STATUS_CD =
                (select CHSTATUS_CD from &sourcetbl._CPK intbl where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and intbl.CHSTATUS_CD IS NOT NULL),
            CONTACT_DTTM = &execdt.
        where exists
             (select 1 from &sourcetbl._CPK intbl where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and intbl.CHSTATUS_CD IS NOT NULL);

    /* Check for errors in update process and exit */
    %if &SQLRC. > 4 %then
    %do;
        %let MAError = 730;
        %let MAMsg = Macro[&sysmacroname.] Error in Updating Contact History target table SQLRC:&SQLRC. Check the log for details.;
        %return;
    %end;
%end;/* End of Update Contact records from Input table */


/* Step 10: If Contact population is to be updated by UPOP first do a full delete */
%if &action. eq UPOP %then
%do;
    delete from &targettbl. ch
        where ch.CELL_PACKAGE_SK in
            (select s.CELL_PACKAGE_SK
                 from &subintbl._CPK s);

    /* Check for errors in update process and exit */
    %if &SQLRC. > 4 %then
    %do;
        %let MAError = 731;
        %let MAMsg = Macro[&sysmacroname.] Error in Deleting Contact records in Contact History target table SQLRC:&SQLRC. Check the log for details.;
        %return;
    %end;

%end; /* End of of full delete */

/* Step 11: Insert the new records into Contact History  */
/* Package_Hash_Val is added for both selection and decision campaigns */
/* Taskid-57851 - use ch_status colum from input table instead of chstatus_cd parameter */
%if &action. eq UPOP or &action. eq UPOPDTTM or (&action. eq USTA and &popexists=0) %then
%do;
    /* S0693256 - insert new column CONTACT_DT */
    insert into &targettbl.
    (
         &sub_key.,
         CELL_PACKAGE_SK,
         CONTACT_HISTORY_STATUS_CD,
         PACKAGE_HASH_VAL,
         CONTACT_DTTM,
         CONTACT_DT
         /* 12Nov2010 */
         %if &action. eq UPOPDTTM %then
         %do;
             ,
             EXTERNAL_CONTACT_INFO_ID1,
             EXTERNAL_CONTACT_INFO_ID2
         %end;
     )
     select
        &sub_key.,
        CELL_PACKAGE_SK,
        CHSTATUS_CD,
        PACKAGE_HASH_VAL,
        %if (&action. eq USTA and &popexists. = 0) or &action. eq UPOP %then
        %do;
            &execdt. , /*S0521156 - use execdt for MA updates */
            datepart(&execdt.)
        %end;
        %else %if (&action. eq UPOPDTTM) %then
        %do;
            /*S0521156-convert the string dttm to number value */
            /*input( put(input(scan(cdttm,1,' '),YYMMDD10.),DATE9.)||':'||scan(cdttm,2,' '), datetime24.3 )*/
            CONTACT_DTTM,
            datepart(CONTACT_DTTM),
            EXTERNAL_CONTACT_INFO_ID1,
            EXTERNAL_CONTACT_INFO_ID2
        %end;
    from &subintbl.;

    /* Insert the new records into Contact History  */
    %if &SQLRC. > 4 %then
    %do;
        %let MAError = 732;
        %let MAMsg = Macro[&sysmacroname.] Error in Inserting Contact records in Contact History target table SQLRC:&SQLRC. Check the log for details.;

        /* Call the print obs macro to print the input table records to a file only for other clients*/
        %if &action.=UPOPDTTM %then
        %do;
            %maprnobs(action=&action.,badrsp=N, expimplflag=I, fl_loc=MAMISC, srclib=&templibname. , srctbl=&subintbl.,
                      stpcall=CH, sub_key=%str(&sub_key.), tgtlib=&libname., tgttbl=&chtable., updatemode=&updatemode.,
                      xmlflag=&xmlflag.);
        %end;
    %end;
%end; /* End of Insert in to CH table*/

quit;

%if &MADebug. eq Y %then
%do;
    %put MACDICHI macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
