/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: macdichw.sas                                            */
/* VERSION: 5.4                                                  */
/* DESCRIPTION: Common Data Interface wrapper macro for CH       */
/*              process to create dynamic source code file       */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - USTA/USTADTTM/UPOP/UPOPDTTM                      */
/*     bulkdatalibref - XML Payload URI for bulkdata             */
/*     bulkdatamap - XML Map file for Payload URI                */
/*     chschema - Database schema                                */
/*     chstatus_cd - Contact Status code                         */
/*     chsubjectidcol -Comma separated CH table physical col name*/
/*     chtable - CH Physical table name                          */
/*     execdt - Execute Datetime                                 */
/*     dbconnect - Data base explicit pass through connect stmt  */
/*     dbdsoptions - Data set options for bulk loading           */
/*     dbmstemp - Y or Null                                      */
/*     dbtempschema - Data base temporary schema                 */
/*     inputtable - Input table with libname <<libname.tablename>*/
/*     libstmt - Libname statement for CH table                  */
/*     matables - Libname statement for inputtable               */
/*     subjectidcol - Input table physical column name           */
/*     xmlflag - Y/N to indicate whether to use XML Stream       */
/*     updatemode - B/T Bulk or Trickling                        */
/*                                                               */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macdichw                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* S0479722 - drop the tiny table (*_cpk) created for MA processing                      */
/*                                                                                       */
/* S0479890 - added new macro call macdibl to upload the subject level records for SDM   */
/* RDM client.  Also added new param bcid to maconcnt macro call                         */
/*                                                                                       */
/* S0470452 - Print the error message in the SAS log and silently fail if input table    */
/* doesn't exist or the subject_id is not part of the input table                        */
/*                                                                                       */
/* S0486525 - Modifications to support NETEZZA                                           */
/* a) added &dbengine. eq NETEZZA to call macro macdiche                                 */
/* b) Code added to call %machkdbc macro incase dbconnect macro variable is not passed   */
/*                                                                                       */
/* S0486300 - Modifications to support SPDS                                              */
/*                                                                                       */
/* S0499640 - Fix to handle the DBMSTEMP=YES option (global                              */
/* temporary schema to upload table) correct                                             */
/*                                                                                       */
/* S0513401 - Added engine check for NEOVIEW to call macdiche                            */
/*                                                                                       */
/* S0513401 - passed targetschema,chschema as macro parameters to macdiche,because for   */
/* Neoview the resource template is not available and we assign this parameter           */
/*  in the madbconn macro                                                                */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 04Sep2008  S0528358   Fix for Teradata UPI creation during MA bulkload.               */
/*                       Included sub_key parameter to pass in the chsubjectidcol        */
/*                       values to the macdibl macro                                     */
/*                                                                                       */
/* 15Sep2008  S0529138   Multipart key for the subjectid is delimited by space when      */
/*                       passed to Stp for MA CH processing.  This is changed to comma   */
/*                       delimited to be consistent with the SDM and RDM CH processing   */
/*                                                                                       */
/* 15Oct2008  S0536278   Assign correct formats to all key columns in the input table    */
/*                       to avoid mismatch in data type after bulkload                   */
/*                                                                                       */
/* 15Oct2008  S0504678   Add formats for the PACKAGE_HASH_VAL                            */
/*                                                                                       */
/* 19Nov2008  S0504678   PACKAGE_HASH_VAL length increased to 32                         */
/*                                                                                       */
/* 30Dec2008  S0472135   Code added to remove the temp tables in case of any error       */
/*                                                                                       */
/* 23Feb2009  S0558672   New input stream (XML) from MA with list of rsptracking_cd      */
/*                                                                                       */
/* 03Jun2009  S0584561   Creating the input set in work location rather than working on  */
/*                       same input table                                                */
/*                                                                                       */
/* 04Jun2009  S0583077   Added new macro parameter(perfflag)                             */
/*                       New counts process machcnt in place of maconcnt                 */
/*                       Calling machcnt for each subject in each BC                     */
/*                                                                                       */
/* 02Jul2009  S0591527   a) Subset input table to have only a particular BC specific recs*/
/*                       when multiple BC records are part of the single input set       */
/*                       b) Process every BC/Subject records part of the input set even  */
/*                       one BC/Subject fails from the single batch file and wirte the   */
/*                       execution status to a error report file                         */
/*                                                                                       */
/* 15Jul2009 S0515661   SPDS Temp Libname changes                                        */
/*                                                                                       */
/* 21Jan2010 S0634713   Refresh the Contact Records for the removed cell                 */
/*                                                                                       */
/* 06May2010 S0663006   Add ASTER db support                                             */
/*                                                                                       */
/* 14May2010 Taskid-57851 MA5.4Iteration5-Update Stored Process for CH "exclude" status  */
/*                                                                                       */
/* 25Oct2010 S0709657   Reutrn the empty table error back in GUI                         */
/*                                                                                       */
/* 28Jun2011 S0744863   Added support for GREENPLUM                                      */
/*                                                                                       */
/* 09Dec2011 S0791301  create / refer all the table/column names in upper case           */
/*                                                                                       */
/* 14Dec2011  S0799554  Reset the MAERROR code to the original one, so to handle the     */
/*                      SDM special handling correctly                                   */
/* 15Jul2013  S0970573 Delete temp tables when perfflg=N                                 */
/* 14Aug2013  S1000764 use targetschema for machcnt                                      */
/*****************************************************************************************/

/* removed dbconnect from the parameter list */

%macro macdichw(action=, bulkdatalibref=, bulkdatamap=, chsubjectidcol=, chtable=,
                dbdsoptions=, dbmstemp=, dbtempschema=, execdt=, inputtable=, libstmt=,
                matables=, perfflag=, updatemode=, xmlflag= );

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MACDICHW - Current Version of the code: 541MA09;
    %put Last Modified on 14Aug2013;
    %put MACDICHW macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Definitions*/

/* Global variable to keep track of the error codes */
%global syscc_old maerror_old mamsg_old;

/* Global DS options */
%let dbengine = %upcase(%qscan(&libstmt.,3,%str( )));
%madsopts(%str(&libstmt.), libstmt, operation=CH);

/* Define Stream Name */
%let InputStream=CHStream;

/* Define Business Context Metadata table name */
%let bc_meta_tbl=%str(WORK.CCSConnect);

/* Define Data Model Metadata table name */
%let dm_meta_tbl=%str(WORK.CCSDM);

/* S0558672 Define TrackingCode table name */
/* Taskid-57851 */
/*%let tc_tbl=%str(WORK.TRACKINGCODEDATA);*/
%let ch_status_tbl=%str(CH_STATUS);

/* Define Data table name */
/* S0584561 Create inputset in the work location */
%let data_tbl=%str(WORK.CHUPDATE);

/* Define XML Payload fileref */
%let xml_pl_ref=CHUPDATE;

/* Define XML Payload Mapfile */
%let xml_pl_map=plmap;

/* Define Combined Meta table */
%let cdi_meta_tbl=%str(WORK.CDIMETATBL);

/* Define Dynamic Source fileref */
%let cdisrc=CDICHSrc;

/* Define the Report Counts table name*/
%let cp_tbl_nm=CI_CELL_PACKAGE;
%let cpt_tbl_nm=CI_CELL_PACKAGE_TREATMENT_CNT;
%let f_con_cd=_30;

/* Fileref to create dynamic source code file */
filename &cdisrc. catalog "work.CICDI.&cdisrc..Source";

/* S0583077 - if perfflag is not set then set it to default(D)*/
%if &perfflag. eq %str() %then %let perfflag =D;

/* Exit process if any error */
%if &SYSERR. > 4 %then
%do;
    %let MAError =720;
    %let MAMsg=Macro[&sysmacroname.]. Error while attempting to define variables. Check the log for details.;
    %return;
%end;

/* Step 2: Code Block for Trickling and Bulk update */
%if %upcase(&xmlflag.) eq Y and &updatemode. ne %str() %then
%do;
    /* Macro to read the BC Connect  & Subject meta table to create master table for dynamic
       source code generation */
    %mabcmeta(bc_meta_tbl=&bc_meta_tbl., bulkdatalibref=&bulkdatalibref., bulkdatamap=&bulkdatamap.,
              cdi_meta_tbl=&cdi_meta_tbl., Inputstream=&inputstream., dm_meta_tbl=&dm_meta_tbl.,
              data_tbl=&data_tbl., stpcall=CH, xml_pl_map=&xml_pl_map., xml_pl_ref=&xml_pl_ref.,
              xmlflag=&xmlflag., updatemode=&updatemode.);

    /* Return to the calling macro if MAError is already set to a non-zero value */
    %if (&MAError. ne %str()) %then
    %do;
        %return;
    %end;


    /* Convert the Numeric column type to Character if need be and create other
       additional coloumns which is missing in the input table */

    /* S0536278 &  S0504678 apply formats to all key columns in input table */
    /* S0583077 Apply format to contact_dttm */
    data &data_tbl. (rename=(CHSTATUS=CHSTATUS_CD P_HASH_VAL=PACKAGE_HASH_VAL));
        attrib CELL_PACKAGE_SK length=8 format=f10.
               CHSTATUS length=$3 format=$3.
               CONTACT_DTTM length=8 format=DATETIME24.3
               P_HASH_VAL length=$32 format=$32.;
        set &data_tbl.;
           CHSTATUS=STRIP(CHSTATUS_CD);
           P_HASH_VAL=STRIP(PACKAGE_HASH_VAL);

        drop CHSTATUS_CD PACKAGE_HASH_VAL;
    run;

    /* Exit the process if there is an error */
    %if &SYSERR. > 4 %then
    %do;
        %let MAError = 725;
        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create input table. Check the log for details.;
        %return;
    %end;

   /* Source Code creation Data block and might be the last block in this script and
     do any necessary processing above this block or inside this itself */
    data _null_;
        file &cdisrc. flowover lrecl=32767;
        put ' ';
        put ' %macro ' "&cdisrc." ';'; /* The macro header */
    run;

    /* S0591527: a) Apply the bc_data where ever needed in the dynamic source code
                 b) New macdists macro introduced to print the execution status of BC/Subject level data */
    data _null_;
        file &cdisrc. flowover lrecl=32767 mod;
        set &cdi_meta_tbl. end=last;
        by bcid;
        if FIRST.BCID=1 then /* Only for the first BCID in the group */
        do;
            put ' ';
            put ' %let targetschema =' targetschema';';
            put ' /***** Macro Calls for Business Context ID: ' bcid ' and Subject ID: ' subject_id '*****/;';
            put ' ';
            put ' ';
            put ' %macdiupl(action= ' "&action." ', bcid= ' bcid ', chtable= ' chtable ' , dbconstmt= ' %nrbquote(dbconnect) ' , ';
            put '           dbdsoptions=' %nrbquote(dbdsoptions)' , dbmstemp= ' dbmstemp ', dbtempschema= ' dbtempschema ', data_tbl=%str(' "&data_tbl." ') ,';
            put '           libstmt=' %nrbquote(libstmt)', stpcall=CH, sub_id= ' subject_id ', sub_key=%str(' subject_key'), xmlflag= ' "&xmlflag" ');';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', inputnm= %str(' "&data_tbl." '), fl_loc=MAMISC, stpcall=CH);';
            put ' ';
            put '     /* Go To end of the BC processing */ ';
            put '     %goto ExitBCID_' bcid  '; ';
            put ' %end; ';
            put ' ';
            put ' %macdibl(action= ' "&action." ', bcid= ' bcid ', data_tbl= &bc_data., dbdsoptions=' %nrbquote(dbdsoptions)',';
            put '          stpcall=CH, sub_id= ' subject_id ', sub_key=%str(' subject_key'), sub_key_fmt= %str(' subject_key_fmt') ); ';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '     %do; ';
            put '         /* Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '     %end; ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '     %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0663006 */
            put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or &dbengine. eq GREENPLM or';
            put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
            put ' %do; ';
            put '     %macdiche(action= ' "&action." ',chschema= &targetschema, chtable= ' chtable ', data_tbl= &bc_data. ,';
            put '               dbconnect=  %nrbquote(&dbconnect.) , dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '         %do; ';
            put '             /* Drop all the uploaded tables in case error */ ';
            put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                       tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '          %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' %else ';
            put ' %do; ';
            put '     %macdichi(action= ' "&action." ', chtable= ' chtable ', data_tbl= &bc_data. , dbengine= &dbengine.,' ;
            put '               libname= &libname., libstmt= ' %nrbquote(libstmt) ', sub_id= ' subject_id ', sub_key=%str(' subject_key') );' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         /* Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                    tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            /* S0583077 - call the machcnt for each BC and each subject in BC */
            put ' ';
            put ' /***** Contacts Report Count at Cell_Package level *****/';
            put ' ';
            put ' %if &perfflag eq D or &perfflag eq T %then';
            put ' %do; ';
            put '   %machcnt(action= ' "&action." ',cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
            put '            data_tbl= &unqintbl.' subject_id ' , dbengine=&dbengine. , f_con_cd= ' "&f_con_cd." ',libnm= &libname.,';
            put '            newid= &newid.,popexists=&popexists , tmplibnm=&templibname. , ';
            put '            dbconnect=  %nrbquote(&dbconnect.) , ';
            put '            targetschema= &targetschema. , ';
            put '            tempschema= &tempschema. ';
            put '           ); ';
            put ' ';
            put '   %if &MAError. ne %str() %then';
            put '   %do; ';
            put '        %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then';
            put '        %do; ';
            put '           /* Drop all the uploaded tables in case error */ ';
            put '           %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                      tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '        %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '   %end; /* End for Error check */ ';
            put ' %end; /* End for perfflag check */ ';

            /* S0970573 - delete the uploaded table when perfflag= N */
            put ' %else %if &perfflag eq N %then';
            put ' %do; ';
            put '       %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '        tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put ' %end; /* End for perfflag = N check */ ';

            put ' ';
            put ' /* End of Subject: ' subject ' in BC: ' %NRBQUOTE(bcname) ' Processing */ ';
            put ' %ExitBC_' bcid +(-1) '_SubjectId_' subject_id +(-1) ': ' ; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between subject_id and colon */
            put ' ';
        end;
        else if FIRST.BCID=0 then /* Except the first BCID in the group and includes the last BCID */
        do;
            put ' ';
            put ' ';
            put ' /***** Macro Calls for Business Context ID: ' bcid ' and Subject ID: ' subject_id '*****/;';
            put ' ';
            put ' %macdibl(action= ' "&action." ', bcid= ' bcid ', data_tbl=&bc_data, dbdsoptions=' %nrbquote(dbdsoptions)', ';
            put '          stpcall=CH, sub_id= ' subject_id ', sub_key=%str(' subject_key'), sub_key_fmt= %str(' subject_key_fmt ') ); ';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '     %do; ';
            put '          /* Drop all the uploaded tables in case error */ ';
            put '          %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                    tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '     %end; ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '     %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put ' %end; ';
            put ' ';
            /* S0663006 */
            put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or  &dbengine. eq GREENPLM or';
            put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
            put ' %do; ';
            put '     %macdiche(action= ' "&action." ',chschema= &targetschema., chtable= ' chtable ', data_tbl= &bc_data. ,';
            put '               dbconnect=  %nrbquote(&dbconnect.) , dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '         %do; ';
            put '             /* Drop all the uploaded tables in case error */ ';
            put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                       tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '         %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' %else ';
            put ' %do; ';
            put '     %macdichi(action= ' "&action." ', chtable= ' chtable ', data_tbl= &bc_data. , dbengine= &dbengine.,' ;
            put '               libname= &libname., libstmt= ' %nrbquote(libstmt) ', sub_id= ' subject_id ', sub_key=%str(' subject_key') );' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         /* Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            /* S0583077 call machcnt for each BC and each subject in BC */
            put ' ';
            put ' /***** Contacts Report Count at Cell_Package level *****/';
            put ' ';
            put ' %if &perfflag eq D or &perfflag eq T %then';
            put ' %do; ';
            put '   %machcnt(action= ' "&action." ',cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
            put '             data_tbl= &unqintbl.' subject_id ' , dbengine=&dbengine. , f_con_cd= ' "&f_con_cd." ',libnm= &libname.,';
            put '             newid= &newid.,popexists=&popexists , tmplibnm=&templibname. , ';
            put '             dbconnect=  %nrbquote(&dbconnect.) , ';
            put '             targetschema= &targetschema. , ';
            put '             tempschema= &tempschema. ';
            put '           ); ';
            put ' ';
            put '   %if &MAError. ne %str() %then';
            put '   %do; ';
            put '       %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then';
            put '       %do; ';
            put '           /* Drop all the uploaded tables in case error */ ';
            put '           %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                     tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '        %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=CH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '    %end; /* End for Error check */ ';
            put ' %end; /* End for perfflag check */ ';

            /* S0970573 - delete the uploaded table when perfflag= N */
            put ' %else %if &perfflag eq N %then';
            put ' %do; ';
            put '       %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '        tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put ' %end; /* End for perfflag = N check */ ';

            put ' ';
            put ' /* End of Subject: ' subject ' in BC: ' %NRBQUOTE(bcname) ' Processing */ ';
            put ' %ExitBC_' bcid +(-1) '_SubjectId_' subject_id +(-1) ': ' ; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between subject_id and colon */
            put ' ';
        end;
        if LAST.BCID=1 then /* Only for last BCID */
        do;
            put ' /* End of BC: ' bcname ' Processing */ ';
            put ' %ExitBCID_' bcid +(-1) ': '; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between bcid and colon */
            put ' ';
        end;
    run;

    data _null_;
        file &cdisrc. flowover lrecl=32767 mod;
        put ' ';
        put '%mend;';
        put ' ';
        put ' %'"&cdisrc."';'; /* call to execute the dynamic macro */
    run;

    /* Exit process if any error */
    %if &SYSERR. > 4 %then
    %do;
        %let MAError = 721;
        %let MAMsg = Macro[&sysmacroname.]. Error in Data Step while attempting to create dynamic source code file. Check the log for details.;
        %return;
    %end;
%end;
/* Step 3: MA Specific code block */
/* S0591527 is not applicble for MA as it deals with only one BC and Subject at any given time */
%else %if &action. eq USTA or &action. eq UPOP %then
%do;

    /* S0529138 - Convert the space delimiter to comma delimiter for subjectid variables */
    data _null_;
        call symput('subjectidcol', tranwrd("&subjectidcol."," ",","));
        call symput('chsubjectidcol', tranwrd("&chsubjectidcol."," ",","));
    run;

    /* Check the existance of the input table and subject_id */
    %let tablenotexists=0;
    %let subidnotexists=0;

    /* Exit if the input table doesn't exists */
    /* S0584561 Refer the correct inputtable name */
    %if %sysfunc(exist(&inputtable.))= 0 %then
    %do;
        %let tablenotexists=1;
    %end;
    %else
    %do;
        %let dsid=%sysfunc(open(&inputtable.));

        %let keynum=0;
        %let done=0;

        %do %until(&done=1);
            %let keynum = %eval(&keynum. + 1);
            %let subkey&keynum = %qscan(%bquote(&subjectidcol.),&keynum.,%str(,)%str( ));
            %let chsubkey&keynum = %qscan(%bquote(&chsubjectidcol.),&keynum.,%str(,)%str( ));
            /* Exit if the number of subject columns not match between input and target tables */
            %if ( &&subkey&keynum eq %str() and &&chsubkey&keynum ne %str() ) or
                    ( &&subkey&keynum ne %str() and &&chsubkey&keynum eq %str() ) %then
            %do;
                %let subidnotexists=1;
                %let done=1;
            %end;
            /* Exit if the subject column is not part of the input table */
            %else %if &&subkey&keynum ne %str() %then
            %do;
                %let rc=%sysfunc(varnum(&dsid,&&subkey&keynum.));

                %if &rc = 0 %then
                %do;
                    %let subidnotexists=1;
                    %let done=1;
                %end;
            %end;
            %else %if &&subkey&keynum. eq %str() %then
            %do;
                %let done=1;
            %end;
        %end;

        %let dummy=%sysfunc(close(&dsid.));
    %end;

    /* If table or subject id doesn't exists print the message in the log and silently fail. That means
       this error message will not be displayed back to the user in the GUI defect: S0470452*/
    %if &tablenotexists.=1 or &subidnotexists.=1 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 733;
        %let MAMsg = Macro[&sysmacroname.]. Input table is empty and could not continue the process. Check the log for details.;
        %return;
    %end;

    /* S0634713: This defect fix overrides the defect S0558672 push. So removing the mabcmeta macro call from here */
    /* Taskid-57851 - Read the ch_status table from input xml - Read the ch_status table from input xml */

    %mabcmeta(Inputstream=&inputstream., stpcall=CH);
    %if (&MAError. ne %str()) %then
    %do;
        %return;
    %end;

    /* Though the BC_ID and SUBJECT_ID metata data information not required for the
       MA CH processing, The RESPTRACKING_CD (max 14 digit) will be always passed, as
       this is the same code goes around all the clients.Strip last 4 digits
       from RESPTRACKING_CD to create cell_package_sk to use in MA*/
    /* S0584561 Create inputset in the work location */
    data &data_tbl._tmp(drop= _RTCLN);
        length CELL_PACKAGE_SK 8;
        format CELL_PACKAGE_SK f10.;
        set &inputtable.;
            _RTCLN=lengthn(left(put(RESPTRACKING_CD,14.)));
            CELL_PACKAGE_SK=substrn(left(put(RESPTRACKING_CD,14.)),1,(_RTCLN-4));
    run;

    /* Taskid-57851 - add the status code in the &data_tble. */
    proc sql;
        create table    &data_tbl. as
            select      inp.*,sts.CONTACT_HISTORY_STATUS_CD as CHSTATUS_CD
            from        &data_tbl._tmp inp
            inner join  &ch_status_tbl. sts
            on          sts.CELL_PACKAGE_SK = inp.CELL_PACKAGE_SK
    ;quit;

    /* S0634713: This defect fix overrides the defect S0558672 fix. So the processing block removed from here */

    /* Assign white space to avoid source file creation error */
    %if %length(&chschema.) = 0 %then %let chschema = %str( );
    %if %length(&dbconnect.) = 0 %then %let dbconnect = %str( );
    %if %length(&dbdsoptions.) = 0 %then %let dbdsoptions = %str( );
    %if %length(&dbmstemp.) = 0 %then %let dbmstemp = %str( );
    %if %length(&dbtempschema.) = 0 %then %let dbtempschema = %str( );
    %if %length(&xmlflag.) = 0 %then %let xmlflag = %str( );
    %if %length(&updatemode.) = 0 %then %let updatemode = %str( );
    %if %length(&matables.) = 0 %then %let matables = %str( );

    data _null_;
        file &cdisrc. flowover lrecl=32767;
        put ' ';
        put ' %macro ' "&cdisrc." ';'; /* The macro header */
    run;

    /* S0591527: The macdists macro processing is not applicable for MA campaigns */
    data _null_;
        file &cdisrc. flowover lrecl=32767 mod;
        put ' ';
        put ' ';
        put ' ';
        put ' %macdiupl(action= ' "&action." ', chtable= ' "&chtable." ' , dbconstmt= ' "%nrbquote(&dbconnect.)" ', dbdsoptions= ' "%nrbquote(&dbdsoptions.)" ','  ;
        put '           dbmstemp= ' "&dbmstemp." ', dbtempschema= ' "&dbtempschema." ', data_tbl=%str(' "&data_tbl." ') ,';
        put '           execdt= ' "&execdt." ' , libstmt= ' "%nrbquote(&libstmt.)" ', matables=' "%nrbquote(&matables.)" ' ,' ;
        put '           stpcall=CH, sub_key= %str(' "&chsubjectidcol." ') , subjectidcol= %str(' "&subjectidcol." ') );';
        put ' ';
        put ' %macdibl(action= ' "&action." ', data_tbl=&bc_data., dbdsoptions=' "%nrbquote(&dbdsoptions.)" ', ';
        put '          stpcall=CH, sub_key= %str(' "&chsubjectidcol." ') ); '; /* S0528358 Added sub_key parameter here */
        put ' ';
        put ' %if &MAError. ne %str() %then';
        put ' %do; ';
        put '     %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or';
        put '         &dbengine. eq DB2 or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
        put '     %do; ';
        put '        /* Drop all the uploaded tables in case error */ ';
        put '        %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
        put '                  tblnms=%str(&unqintbl. &unqintbl._CPK), templibname= &templibname.); ';
        put '     %end; ';
        put ' ';
        put '     %return; ';
        put ' %end; ';
        put ' ';
        put ' ';
        /* S0663006 */
        put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or &dbengine. eq GREENPLM or';
        put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
        put ' %do; ';
        put '     %macdiche(action= ' "&action." ',chschema=  &chschema ,chtable= ' "&chtable." ', data_tbl= &bc_data. ,';
        put '              dbconnect= %nrbquote(&dbconnect.), dbengine= &dbengine. , libname= &libname. ,';
        put '              sub_key= %str(' "&chsubjectidcol." ') , subjectidcol= %str(' "&subjectidcol." ') , tempschema= &tempschema. );' ;
        put ' ';
        put '     %if  &MAError.  ne %str() %then';
        put '     %do; ';
        put '         %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
        put '         %do; ';
        put '             /* Drop all the uploaded tables in case error */ ';
        put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
        put '                       tblnms=%str(&unqintbl. &unqintbl._CPK), templibname= &templibname.); ';
        put '         %end; ';
        put ' ';
        put '         %return; ';
        put '     %end; ';
        put ' %end; ';
        put ' %else ';
        put ' %do; ';
        put '     %macdichi(action= ' "&action." ', chtable= ' "&chtable." ', data_tbl= &bc_data. , dbengine= &dbengine. ,' ;
        put '               libname= &libname. , libstmt= ' "%nrbquote(&libstmt.)" ', subjectidcol= %str(' "&subjectidcol." '),' ;
        put '               sub_key= %str(' "&chsubjectidcol." ') );' ;
        put ' ';
        put '     %if  &MAError.  ne %str() %then';
        put '     %do; ';
        put '         /* Drop all the uploaded tables in case error */ ';
        put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
        put '                   tblnms=%str(&unqintbl. &unqintbl._CPK), templibname= &templibname.); ';
        put ' ';
        put '         %return; ';
        put '     %end; ';
        put ' %end; ';
        put ' ';
        put ' ';
        /* S0583077 call machcnt for each BC */
        put ' ';
        put ' /***** Contacts Report Count at Cell_Package level *****/';
        put ' ';
        put ' %machcnt(action= ' "&action." ',cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
        put '           data_tbl= &unqintbl. , dbengine=&dbengine. , f_con_cd= ' "&f_con_cd." ',libnm= &libname.,';
        put '           newid= &newid.,popexists=&popexists , tmplibnm=&templibname. , ';
        put '           dbconnect=  %nrbquote(&dbconnect.) , ';
        put '           targetschema= &chschema. , ';
        put '           tempschema= &tempschema. ';
        put '         ); ';

        put ' ';
        put ' ';
        /* S0515661 */
        put ' /***** Delete Uploaded Temp tables *****/;';
        put '             %macdidel(libexist=Y, tblnms=%str(&unqintbl. &unqintbl._CPK), templibname= &templibname.); ';
    run;

    data _null_;
        file &cdisrc. flowover lrecl=32767 mod;
        put ' ';
        put '%mend;';
        put ' ';
        put ' %'"&cdisrc."';'; /* call to execute the dynamic macro */
    run;

    /* Exit process if any error */
    %if &SYSERR. > 4 %then
    %do;
        %let MAError = 721;
        %let MAMsg = Macro[&sysmacroname.]. Error in Data Step while attempting to create dynamic source code file. Check the log for details.;
        %return;
    %end;

%end;

/* Exit process if any error */
%if &SYSERR. > 4 %then
%do;
    %let MAError = 721;
    %let MAMsg = Macro[&sysmacroname.]. Error in Data Step while attempting to create dynamic source code file. Check the log for details.;
    %return;
%end;

/* Include the Dynamic Source code file */
%include &cdisrc.;

/* S0799554: Reset the MAError code to MAError_old which is set from the macdists macro. */
/* Assign the error codes back */
%if &syscc_old. ne %str() %then
%do;
    %let SYSCC= &syscc_old.;
    %let MAError= &maerror_old.;
    %let MAMsg= Error processing History tables.  Check log and MAMISC.CI_HISTORY_ERROR_REPORT for details;
%end;

%if &MADebug. eq Y %then
%do;
    %put MACDICHW macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
