/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: macdiptw.sas                                            */
/* VERSION: 5.4                                                */
/* DESCRIPTION: Common Data Interface wrapper macro for PT       */
/*              process to create dynamic source code file       */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - UPOPDTTM                                         */
/*     xmlflag - Y to indicate that Stp receives input XML Stream*/
/*     updatemode - T (Trickling)                                */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macdiptw                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* S0479890 - added new macro call macdibl to upload the subject level records for SDM   */
/* and RDM client.  Also added new param bcid to maconcnt macro call                     */
/*                                                                                       */
/* S0486525 - Modifications to support NETEZZA                                           */
/*                                                                                       */
/* S0486300 - Modifications to support SPDS                                              */
/*                                                                                       */
/* S0499640 - Fix to handle the DBMSTEMP=YES option (global temporary schema to upload   */
/* table) correct                                                                        */
/*                                                                                       */
/* S0513401 - Added engine check for NEOVIEW to call macdipte                            */
/*                                                                                       */
/* S0513401 - passed targetschema as macro parameters to macdipte because for Neoview    */
/* the resource template is not available and we assign this parameter in the madbconn   */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 19Sep2008  S0529138   Changes to handle the comma separated multipart-key correct in  */
/*                       SDM/RDM RH processing                                           */
/*                                                                                       */
/* 03Oct2008  S0536278   Assign correct formats to all key columns in the input table    */
/*                       to avoid mismatch in data type after bulkload                   */
/*                                                                                       */
/* 19Nov2008  S0504678   Dynamic Treatment Changes for CI5.3 development                 */
/*                                                                                       */
/* 01Dec2008  S0504678   Changes to support PT update from both 5.1 and 5.3 campaigns    */
/*                                                                                       */
/* 30Dec2008  S0472135   Code added to remove the temp tables in case of any error       */
/*                                                                                       */
/* 04Jun2009  S0583077   added new macro parameter(perfflag)                             */
/*                       using new counts process maptcnt in place of maconcnt           */
/*                       calling maptcnt for each subject in each BC                     */
/*                                                                                       */
/* 02Jul2009  S0591527   a) Subset input table to have only a particular BC specific recs*/
/*                       when multiple BC records are part of the single input set       */
/*                       b) Process every BC/Subject records part of the input set even  */
/*                       one BC/Subject fails from the single batch file and wirte the   */
/*                       execution status to a error report file                         */
/*                                                                                       */
/* 06May2010 S0663006   Add ASTER db support                                             */
/*                                                                                       */
/* 14Dec2011  S0799554   Reset the MAERROR code to the original one, so to handle the    */
/*                       SDM special handling correctly                                  */
/* 15Jul2013  S0970573 Delete temp tables when perfflg=N                                 */
/*****************************************************************************************/

%macro macdiptw(action=, perfflag=, updatemode=, xmlflag=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MACDIPTW - Current Version of the code: 541MA09;
    %put Last Modified on 15Jun2013;
    %put MACDIPTW macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Definitions*/

/* Global variable to keep track of the error codes */
%global syscc_old maerror_old mamsg_old;

/* Define Stream Name */
%let Inputstream=PTStream;

/* Define Business Context Metadata table name */
%let bc_meta_tbl=%str(WORK.CCSConnect);

/* Define Data Model Metadata table name */
%let dm_meta_tbl=%str(WORK.CCSDM);

/* Define Data table name */
%let data_tbl=%str(WORK.PTUpdate);

/* Define XML Payload fileref */
%let xml_pl_ref=PTUpdate;

/* Define XML Payload Mapfile */
%let xml_pl_map=plmap;

/* Define Combined Meta table */
%let cdi_meta_tbl=%str(WORK.CDIMetatbl);

/* Define Dynamic Source fileref */
%let cdisrc=CDIPTSrc;

/* Define the Report Counts table name*/
%let cp_tbl_nm=CI_CELL_PACKAGE;
%let cpt_tbl_nm=CI_CELL_PACKAGE_TREATMENT_CNT;
%let ct_tbl_nm=CI_TREATMENT;
%let cdta_tbl_nm=CI_DYNAMIC_TREATMENT_ATTRIBUTE;
%let f_con_cd=_30;

/* S0583077 - if perfflag is not set then set it to default(D)*/
%if &perfflag. eq %str() %then %let perfflag =D;

/* Fileref to create dynamic source code file */
filename &cdisrc. catalog "work.CICDI.&cdisrc..Source";

/* Exit process if any error */
%if &SYSERR. > 4 %then
%do;
    %let MAError =720;
    %let MAMsg=Macro[&sysmacroname.]. Error while attempting to define variables. Check the log for details.;
    %return;
%end;

/* Step 2: Code Block for Trickling and Bulk update */
/* Macro to read the BC Connect  & Subject meta table to create master table for dynamic
       source code generation */
    %mabcmeta(bc_meta_tbl=&bc_meta_tbl., cdi_meta_tbl=&cdi_meta_tbl., Inputstream=&Inputstream.,
              dm_meta_tbl=&dm_meta_tbl., data_tbl=&data_tbl., stpcall=PT, xml_pl_map=&xml_pl_map.,
              xml_pl_ref=&xml_pl_ref., xmlflag=&xmlflag., updatemode=&updatemode.);

    /* Return to the calling macro if MAError is already set to a non-zero value */
    %if (&MAError. ne %str()) %then
    %do;
        %return;
    %end;

    /* Convert the Numeric column type to Character if need be and create other
       additional coloumns which is missing in the input table */

    /* S0536278 apply formats to all key columns in input table */
    /* S0504678 a) CI5.3 Dynamic Treatment changes to add Treatment_Hash_Val
              b) Add TMTTracking_Cd which will have either one of the following Treatment_Tracking_Cd
                     Treatment_hash_Val varchar(32)- for 5.3 Campaign with Dynamic Treatments
                     Treatment_Sk number(10)- for 5.3 campaign with Static Treatments
                     Treatment_Cd varchar(32)- for 5.1 campaigns. */
     /* S0583077 Apply format to contact_dttm, modified column name PRESENTED_TREATMENT_HIST_DTTM*/
     data &data_tbl. (rename=(TMT_CDS=TMTTRACKING_CD) );
         attrib CELL_PACKAGE_SK length=8 format=f10.
                TREATMENT_SK length=8 format=f10.
                TMT_CDS length=$32 format=$32.
                TREATMENT_HASH_VAL length=$32 format=$32.
                PRESENTED_TREATMENT_HIST_DTTM length=8 format=DATETIME24.3;
         set &data_tbl.;
             TMT_CDS=strip(TMTTRACKING_CD);
             PRESENTED_TREATMENT_HIST_DTTM=PRESENTED_TREATMENT_DTTM;
         drop TMTTRACKING_CD PRESENTED_TREATMENT_DTTM ;
      run;

    /* Exit the process if there is an error */
    %if &SYSERR. > 4 %then
    %do;
        %let MAError = 725;
        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create input table. Check the log for details.;
        %return;
    %end;

   /*Source Code creation Data block and might be the last block in this script and
     do any necessary processing above this block or inside this itself*/
    data _null_;
        file &cdisrc. flowover lrecl=32767;
        put ' ';
        put ' %macro ' "&cdisrc." ';'; /* The macro header */
    run;

    /* S0591527: a) Apply the bc_data where ever needed in the dynamic source code
                 b) New macdists macro included to print the error report */
    data _null_;
        file &cdisrc. flowover lrecl=32767 mod;
        set &cdi_meta_tbl. end=last;
        by bcid;
        if FIRST.BCID=1 then /* Only for the first BCID in the group */
        do;
            put ' ';
            put ' %let targetschema =' targetschema';';
            put ' /***** Macro Calls for Business Context ID: ' bcid ' and Subject ID: ' subject_id '*****/;';
            put ' ';
            put ' ';
            put ' %macdiupl(action= ' "&action." ', bcid= ' bcid ', ct_tbl_nm= ' "&ct_tbl_nm." ',  dbconstmt= ' %nrbquote(dbconnect) ' ,';
            put '           dbdsoptions='%nrbquote(dbdsoptions)' ,dbmstemp= ' dbmstemp ', dbtempschema= ' dbtempschema ', data_tbl=%str(' "&data_tbl." ') ,';
            put '           libstmt=' %nrbquote(libstmt)', stpcall= PT, sub_id= ' subject_id ', sub_key=%str(' subject_key '), xmlflag= ' "&xmlflag" ');' ;
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', inputnm= %str(' "&data_tbl." '), fl_loc=MAMISC, stpcall=PT);';
            put ' ';
            put '     /* Go To end of the BC processing */ ';
            put '     %goto ExitBCID_' bcid  '; ';
            put ' %end; ';
            put ' ';
            put ' %macdibl(action= ' "&action." ', bcid= ' bcid ', data_tbl=&bc_data., dbdsoptions=' %nrbquote(dbdsoptions)',';
            put '          stpcall=PT, sub_id= ' subject_id ', sub_key=%str(' subject_key'), sub_key_fmt= %str(' subject_key_fmt') ); ';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '     %end; ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0663006 - Use Explicit Passthrough INSERT for Aster */
            put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or ';
            put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
            put ' %do; ';
            put '     %macdipte(action= ' "&action." ',ptschema= &targetschema., pttable= ' pttable ', data_tbl= &bc_data. ,';
            put '               dbconnect=  %nrbquote(&dbconnect.)  , dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            put '     %if  &MAError.  ne %str() %then';
            put '     %do; ';
            put '         %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '         %do; ';
            put '             /* S0472135: Drop all the uploaded tables in case error */ ';
            put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                       tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '         %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' %else ';
            put ' %do; ';
            put '     %macdipti(action= ' "&action." ', pttable= ' pttable ', data_tbl= &bc_data. , dbengine= &dbengine.,' ;
            put '               libname= &libname., libstmt= ' %nrbquote(libstmt) ', sub_id= ' subject_id ', sub_key=%str(' subject_key ') );' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '     %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0583077 - call the maptcnt for each BC and each subject in BC */
            put ' ';
            put ' /***** Contacts Report Count at Cell_Package level *****/';
            put ' %if &perfflag eq D or &perfflag eq T %then';
            put ' %do; ';
            put '   %maptcnt(ch_tbl=' chtable ' ,cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
            put '             data_tbl=  &unqintbl.' subject_id ' ,dbengine=&dbengine. , libnm= &libname.,';
            put '             newid= &newid. , perfflag= ' "&perfflag." ' ,pth_tbl= ' pttable ',sub_key_sp= &sub_key_sp, tmplibnm=&templibname.); ';
            put ' ';
            put '   %if &MAError. ne %str() %then';
            put '   %do; ';
            put '       %if %length(&dbconnect.) > 0 and (&dbengine. eq SASSPDS) %then ';
            put '       %do; ';
            put '           /* S0472135: Drop all the uploaded tables in case error */ ';
            put '           %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                     tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '       %end; ';
            put ' ';
            put '       /* Print the Error report and reset the error code and messages */ ';
            put '       %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '       /* Go To end of the Subject processing */ ';
            put '       %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '   %end; /* End for Error check */ ';
            put ' %end; /* End for perfflag check */ ';

            /* S0970573 - delete the uploaded table when perfflag= N */
            put ' %else %if &perfflag eq N %then';
            put ' %do; ';
            put '       %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '        tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put ' %end; /* End for perfflag = N check */ ';

            put ' ';
            put ' /* End of Subject: ' subject ' in BC: ' %NRBQUOTE(bcname) ' Processing */ ';
            put ' %ExitBC_' bcid +(-1) '_SubjectId_' subject_id +(-1) ': ' ; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between subject_id and colon */
            put ' ';

        end;
        else if FIRST.BCID=0 then /* Except the first BCID in the group and includes the last BCID */
        do;
            put ' ';
            put ' %let targetschema =' targetschema';';
            put ' /***** Macro Calls for Business Context ID: ' bcid ' and Subject ID: ' subject_id '*****/;';
            put ' ';
            put ' %macdibl(action= ' "&action." ', bcid= ' bcid ', data_tbl=&bc_data., dbdsoptions=' %nrbquote(dbdsoptions)',';
            put '          stpcall=PT, sub_id= ' subject_id ', sub_key=%str(' subject_key'), sub_key_fmt= %str(' subject_key_fmt') ); ';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '     %end; ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '       %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put ' %end; ';
            put ' ';
            /* S0663006 - Use Explicit Passthrough INSERT for Aster */
            put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or ';
            put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
            put ' %do; ';
            put '     %macdipte(action= ' "&action." ',ptschema= &targetschema, pttable= ' pttable ', data_tbl= &bc_data. ,';
            put '               dbconnect=  %nrbquote(&dbconnect.) , dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         %if %length(&dbconnect.) > 0 and (&dbengine. eq SASSPDS) %then ';
            put '         %do; ';
            put '             /* S0472135: Drop all the uploaded tables in case error */ ';
            put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                       tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '         %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' %else ';
            put ' %do; ';
            put '     %macdipti(action= ' "&action." ', pttable= ' pttable ', data_tbl= &bc_data. , dbengine= &dbengine.,' ;
            put '               libname= &libname., libstmt= ' %nrbquote(libstmt) ', sub_id= ' subject_id ', sub_key=%str(' subject_key ') );' ;
            put ' ';
            put '     %if &MAError. ne %str() %then';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0583077 - call the maptcnt for each BC and each subject in BC */
            put ' ';
            put ' /***** Contacts Report Count at Cell_Package level *****/';
            put ' %if &perfflag eq D or &perfflag eq T %then';
            put ' %do; ';
            put '    %maptcnt(ch_tbl=' chtable ' ,cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
            put '              data_tbl=  &unqintbl.' subject_id ' ,dbengine=&dbengine. , libnm= &libname.,';
            put '              newid= &newid. , perfflag= ' "&perfflag." ' ,pth_tbl= ' pttable ',sub_key_sp= &sub_key_sp, tmplibnm=&templibname.); ';
            put ' ';
            put '    %if &MAError. ne %str() %then';
            put '    %do; ';
            put '        %if %length(&dbconnect.) > 0 and (&dbengine. eq SASSPDS) %then ';
            put '        %do; ';
            put '            /* S0472135: Drop all the uploaded tables in case error */ ';
            put '            %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                      tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '        %end; ';
            put ' ';
            put '       /* Print the Error report and reset the error code and messages */ ';
            put '       %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=PT, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '       /* Go To end of the Subject processing */ ';
            put '       %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '    %end; /* End for Error check */';
            put ' %end; /* End for perfflag check */ ';

            /* S0970573 - delete the uploaded table when perfflag= N */
            put ' %else %if &perfflag eq N %then';
            put ' %do; ';
            put '       %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '        tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put ' %end; /* End for perfflag = N check */ ';

            put ' ';
            put ' /* End of Subject: ' subject ' in BC: ' %NRBQUOTE(bcname) ' Processing */ ';
            put ' %ExitBC_' bcid +(-1) '_SubjectId_' subject_id +(-1) ': ' ; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between subject_id and colon */
            put ' ';
        end;
        if LAST.BCID=1 then /* Only for last BCID */
        do;
            put ' /* End of BC: ' bcname ' Processing */ ';
            put ' %ExitBCID_' bcid +(-1) ': '; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between bcid and colon */
            put ' ';
        end;
run;

data _null_;
file &cdisrc. flowover lrecl=32767 mod;
    put ' ';
    put '%mend;';
    put ' ';
    put ' %'"&cdisrc."';'; /* call to execute the dynamic macro */
run;

/* Exit process if any error */
%if &SYSERR. > 4 %then
%do;
     %let MAError = 721;
     %let MAMsg = Macro[&sysmacroname.]. Error in Data Step while attempting to create dynamic source code file. Check the log for details.;
     %return;
%end;

/* Include the Dynamic Source code file */
%include &cdisrc.;

/* S0799554: Reset the MAError code to MAError_old which is set from the macdists macro. */
/* Assign the error codes back */
%if &syscc_old. ne %str() %then
%do;
    %let SYSCC= &syscc_old.;
    %let MAError= &maerror_old.;
    %let MAMsg= Error processing History tables.  Check log and MAMISC.CI_HISTORY_ERROR_REPORT for details;
%end;

%mend;
