/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: macdiupl.sas                                            */
/* VERSION: 5.4                                                  */
/* DESCRIPTION: Common Data Interface macro to bulkload/Upload   */
/*              inputtable to data base schema for further       */
/*              processing                                       */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - USTA/USTADTTM/UPOP/UPOPDTTM                      */
/*     bcid - 2 digit BC id                                      */
/*     chtable - CH Physical table name                          */
/*     crcr_tbl_nm - Table name for RESPONSE_CHANNEL_RESPONSE    */
/*     ct_tbl_nm - Table name for CI_TREATMENT                   */
/*     data_tbl - Actual data table name                         */
/*     dbconstmt - Data base explicit pass through connect stmt  */
/*     dbdsoptions - Data set options for bulk loading           */
/*     dbmstemp - Y or Null                                      */
/*     dbtempschema - Data base temporary schema                 */
/*     execdt - Execute Datetime                                 */
/*     libstmt - Libname for CH database table                   */
/*     matables - Input table library                            */
/*     stpcall - CH/RH/PT Storeprocess which invoked this macro  */
/*     sub_id - 2 digit subject id                               */
/*     sub_key - Comma separated subject_key for other than MA   */
/*     subjectidcol - Comma separated subject_key for MA         */
/*     updatemode - B/T Bulk or Trickling                        */
/*     xmlflag - Y/N to indicate whether to use XML Stream       */
/*                                                               */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macdiupl                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* S0472916 - Added contact_dttm part of the nodupkey to allow multiple contacts to the  */
/* same subject this is possible from RDM client                                         */
/*                                                                                       */
/* S0479722 - Upload two tables for MA campaign when action=UPOP a) The entire input set */
/* b) The tiny subset (only cell_package_sk) from input set. Upload only the tiny subset */
/* if action=USTA to avoid sub-query performance issue                                   */
/*                                                                                       */
/* S0486525 - Modifications to support NETEZZA                                           */
/* a)Added code to get the templibname from magentmp macro                               */
/* b) Modified code to create the tables with upcase, as this is                         */
/* needed because of preserve_tab_names=yes in libstmt                                   */
/*                                                                                       */
/* S0486300 - Modifications to support SPDS                                              */
/*                                                                                       */
/* S0476735 - treatment_sk is populated with -1 in case of null/missing                  */
/* treatment_cd                                                                          */
/*                                                                                       */
/* S0499388 - Response_Channel_Response_cd conversion is included for SDM client as well */
/* to track and count the SDM responses correct                                          */
/*                                                                                       */
/* S0499640 - Fix to handle the DBMSTEMP=YES option (global temporary schema to          */
/* upload table) correct                                                                 */
/*                                                                                       */
/* DATE      Defect No  Comments                                                         */
/* 04Sep2008 S0530203   The Contact_Dttm column was renamed to Cdttm for SPDS to be in   */
/*                      consistent with other data bases. But this new name was not      */
/*                      referred for SDM and RDM processes                               */
/*                                                                                       */
/* 17Sep2008 S0521156   Keep the DTTM values as string for SASSPDS,ODBC and apply the    */
/*                      required function to convert that to numeric when updating the   */
/*                      target table                                                     */
/*                                                                                       */
/* 19Nov2008 S0504678   Dynamic Treatment Changes for CI5.3 development                  */
/*                                                                                       */
/* 01Dec2008  S0504678  Changes to support PT & RH update from both 5.1 & 5.3 campaigns  */
/*                                                                                       */
/* 23Feb2009  S0558672  Sort and dedup the list of cell_package_sk from new XML file     */
/*                                                                                       */
/* 04Jun2009  S0583077 Modified the column name presented_treatment_dttm to use          */
/*                     presented_treatment_hist_dttm                                     */
/*                     Added unique table name for SPDS                                  */
/*                                                                                       */
/* 22Jun2009 S0583163   Reset the SYSCC value to 100000 in case of error                 */
/*                                                                                       */
/* 02Jul2009 S0591527   Subset input table to have only a particular BC specific records */
/*                      when multiple BC records are part of the single input set        */
/*                                                                                       */
/* 10Jul2009 S0516132   Change for SPDS uniqtbl name                                     */
/*                                                                                       */
/* 15Jul2009 S0515661   SPDS Temp Libname changes                                        */
/*                                                                                       */
/* 20Jan2010 S0637761   Modified if condition to use the length funciton                 */
/*                                                                                       */
/* 21Jan2010 S0634713   Refresh the Contact Records for the removed cell                 */
/*                                                                                       */
/* 19May2010 Taskid-57851 MA5.4Iteration5-Update Stored Process for CH "exclude" status  */
/*                                                                                       */
/* 04Jun2010 S0603360   Remove Netezza and Neoview manual dbconnect stmt construction    */
/*                                                                                       */
/* 08Jun2010 S0666172   Add DBMSTEMP support for Netezza and OLEDB connection            */
/*                                                                                       */
/* 21Sep2010 S0693256   new column support CONTACT_DT                                    */
/*                                                                                       */
/* 12Nov2010 S0715116   Add 2 new columns External_Presented_Info_ID1 & ID2              */
/*                                                                                       */
/* 28Jun2011 S0744863   Added support for GREENPLUM                                      */
/*                                                                                       */
/* 09Dec2011 S0791301  create / refer all the table/column names in upper case           */
/* 14Aug2013 S0993481  add dsreadopts                                                    */
/*****************************************************************************************/

%macro macdiupl(action=, bcid=, chtable=, crcr_tbl_nm=, ct_tbl_nm=, dbconstmt=, dbdsoptions=,
                dbmstemp=, dbtempschema=, data_tbl=, execdt=, libstmt=, matables=,
                stpcall=, sub_id=, sub_key=, subjectidcol=,  updatemode=, xmlflag=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MACDIUPL - Current Version of the code: 541MA09;
    %put Last Modified on 14Aug2013;
    %put MACDIUPL macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Variable Inialisation and other initial processing */
%global newid worklib bc_data ctdate;

%let dbengine = %upcase(%qscan(&libstmt.,3,%str( )));

/* S0591527: Subset the input dataset to have only BC specific records. This will avoid
   unwanted processing the complete dataset when multiple BC records are part of the
   single batch. */
%if %upcase(&action.) eq UPOPDTTM or %upcase(&action.) eq USTADTTM %then
%do;
    proc sql noprint;
        select count(distinct(BCID)) into :dist_bc from &data_tbl.;
    quit;

    %if &dist_bc. > 1 %then
    %do;
        %let bc_data = &data_tbl._&bcid.; /* Suffix the data_tbl with the _<bcid> to store only bc subset */
    %end;
    %else
    %do;
        %let bc_data = &data_tbl.;
    %end;
%end;
%else /* MA campaign will have only one BC and one Subject level records in the input set */
%do;
    %let bc_data = &data_tbl.;
%end;

/* Target Table library */
%if &dbmstemp.=%str() or &dbengine. eq SASSPDS %then
%do;
    %macrtlib(%unquote(&libstmt.),MACDIUPL);
%end;
%else %if %upcase(&dbmstemp.) eq YES and &dbengine. ne SASSPDS %then
%do; /* S0499640 When DBMSTEMP=YES then the library statement should have
        CONNECTION=GLOBAL set in order to access any global temporary table */

    %if &dbengine. eq OLEDB %then
    %do;
        %macrtlib(%unquote(&libstmt.),MACDIUPL); /* S0666172: Connection=Global not required for OLEDB connection */
    %end;
    %else
    %do;
        %let libstmt= %sysfunc(strip(&libstmt.)); /* Remove spaces after semicolon */

        %let semicolpos= %index(&libstmt.,%str(;));
        %let liblength= %length(&libstmt.);

        %if &semicolpos. > 0 %then
        %do;
            /* Remove the semicolon if it part of the libname */
            %if %substr(%superq(libstmt),&liblength.) eq %str(;) %then
                %let libstmt=%substr(%superq(libstmt),1,%eval(&liblength.-1));
        %end;

        %macrtlib(%unquote(&libstmt. CONNECTION=GLOBAL),MACDIUPL);
    %end;
%end;


%if (&MAError. ne %str()) %then
%do;
        %return;
%end;
/* Extract library name for later use */
%let libname = %upcase(%qscan(&libstmt.,2));

/* Input Table library */
/* S0637761 */
/*%if &matables ne %str() %then*/
%if %length(&matables) > 0 %then
%do;
    %macrtlib(%unquote(&matables.),MACDIUPL);

    %if (&MAError. ne %str()) %then
    %do;
        %return;
    %end;
    /* Extract library name for later use */
    %let matblslib= %upcase(%qscan(&matables.,2));
%end;

/* S0744863: Special handling for GREENPLM DB */
%if %upcase(&dbengine.) eq GREENPLM %then
%do;
    %let dbconnect = %machkdbc(dbconnect=&dbconstmt.,libstmt=&libstmt.);
%end;
%else %if %length(&dbconstmt.) > 0 %then
%do;
    %let dbconnect = &dbconstmt.;
%end;

/* Step 2: Convert Contact DTTM in to a standard format */
/* S0521156: SASSPDS and ODBC connection needs the DTTM value as a number.  For MA Contact History
   updates the execdt macro variable is used.  Since the single batch of input recrods from SDM and RDM
   can have records from multiple business contexts pointing to different DB, the standard string format
   needs to be converted to number during the insert/update process in the respective code base.

   SPDS specific processing to created CDTTM/RDTTM/PTDTTM is removed below to avoid the possible issues
   when records from different db comes in a single batch*/
/* S0693256 - add new column info CONTACT_DT*/
%if (%upcase(&stpcall.) eq CH) and (&action. eq USTADTTM or &action. eq UPOPDTTM) %then
%do;

    data &bc_data.; /* S0591527 - Subset only the bc data */
        set &data_tbl.;
        where BCID="&bcid.";
        CDTTM=put(datepart(contact_dttm),yymmdd10.)||' '||put(timepart(contact_dttm),tod12.3);
        CTDATE=put(datepart(contact_dttm),yymmdd10.);
    run;

%end;
/* Create Contact DTTM if not in input table this will be same for the given comm. execution */
%else %if (%upcase(&stpcall.) eq CH) and (&action. eq USTA or &action. eq UPOP) %then
%do;
   data _null_;
        call symput('cdttm',put(datepart(&execdt.),yymmdd10.)||' '||put(timepart(&execdt.),tod12.3));
        call symput('ctdate',put(datepart(&execdt.),yymmdd10.));
    run;
%end;
/* Response and Processed DTTM */
%else %if (%upcase(&stpcall.) eq RH) %then
%do;
        data &bc_data.; /* S0591527 - Subset only the bc data */
            set &data_tbl.;
            where BCID="&bcid.";
            RDTTM=put(datepart(RESPONSE_DTTM),yymmdd10.)||' '||put(timepart(RESPONSE_DTTM),tod12.3);
            call symput('pdttm',put(datepart(datetime()),yymmdd10.)||' '||put(timepart(datetime()),tod8.));
        run;
%end;
/* S0583077 */
/* Presented Treatment DTTM value */
%else %if (%upcase(&stpcall.) eq PT) %then
%do;
        data &bc_data.; /* S0591527 - Subset only the bc data */
            set &data_tbl.;
            where BCID="&bcid.";
            PTDTTM=put(datepart(presented_treatment_hist_dttm),yymmdd10.)||' '||put(timepart(presented_treatment_hist_dttm),tod12.3);
        run;
%end;

%if &SYSERR. > 4 %then
%do;
    %let MAError = 726;
    %let MAMsg = Macro[&sysmacroname.]. Error in DATA STEP while formating DTTM attribute. Check the log for details.;
    %return;
%end;


/* Step 3: Create temp libname to upload the input table */
/* Upload the input table only for other than SAS and SASPDS tables */

/* S0515661 */
/* SPDS database specific processing */
%if &dbengine. eq SASSPDS  %then
%do;
    /* if the dbtempschema is not specified and dbmstemp is also not specified then */
    /* use the target schema to do the temporary processing & just set the  templibname to Target libref*/
    %if &dbtempschema.=%str() AND &dbmstemp. = %str() %then
    %do;
        %let templibname=%upcase(%qscan(&libstmt.,2));
    %end;
    /* if the dbtempschema is provided then use dbtempschema to create a temp library */
    %else %if &dbtempschema. ne %str() %then
    %do;
        %magentmp(dbmstemp=&dbmstemp.,dbtempschema="&dbtempschema.",libstmt=&libstmt.);
        %macrtlib(%unquote(&dbtmplib.),MACDIUPL); /* Execute DBTEMPLIB library */
        %if (&MAError. ne %str()) %then
        %do;
            %return;
        %end;
        %let templibname=%upcase(%qscan(&dbtmplib.,2));
    %end;
    /* create a Tempoarry library with TEMP=YES , this is when DBTEMP= YES */
    %else
    %do;
        %let libstmt= %sysfunc(strip(&libstmt.)); /* Remove spaces after semicolom */
        %let semicolpos= %index(&libstmt.,%str(;));
        %let liblength= %length(&libstmt.);
        %let enginepos= %index(%upcase(&libstmt.),%quote(&dbengine.));
        %if &semicolpos. > 0 %then
        %do;
            /* Remove the semicolon if it part of the libname before concatenate reread..*/
            %if %substr(%superq(libstmt),&liblength.) eq %str(;) %then %let libstmt=%substr(%superq(libstmt),1,%eval(&liblength.-1));
        %end;

        %let spdtemp= LIBNAME SPDTEMP %qsubstr(%superq(libstmt),&enginepos) TEMP=YES;

        %macrtlib(%unquote(&spdtemp.), MACDIUPL);
        %if &MAError. ne %str() %then
        %do;
            %return;
        %end;
        %let templibname=%upcase(%qscan(&spdtemp.,2));
        /*%let unqintbl=in_tbl_;*//* S0583077 */
    %end;
    %let worklib=&templibname.;
%end;
/* for NON SPDS database processing */
%else
%do;
    %magentmp(dbmstemp=&dbmstemp.,dbtempschema=&dbtempschema.,libstmt=&libstmt.);
    %macrtlib(%unquote(&dbtmplib.),MACDIUPL); /* Execute DBTEMPLIB library */

    %if (&MAError. ne %str()) %then
    %do;
        %return;
    %end;

    /* Pull out libref from temporary libname statement */
    %let templibname=%upcase(%qscan(&dbtmplib,2));

    /* Worklib variable is introduced to handle the temp lib for all the db */
    %let worklib=WORK;

    /* Libname statement being passed should look like one of the next 3: */
    /* i) For temp. tables recognized as temporary by database, must have dbmstemp=yes,connection=global */
    /*    and schema will be ignored. */
    /* ii)For temp. tables created in a permanent schema, use supplied schema name for EP */
    /* iii)If no schema name is supplied, then the user default schema will be used */

    %if &dbtempschema. ne %str() %then
    %do;
       /* Netezza - 3 dots has to be applied as the table name is represented as
       database-name.implicit-schema name.object-name but there is one schema
       per database in Netezza , so 2 dots are required between database and table */
       %if &dbengine. eq NETEZZA %then
            %let tempschema=&dbtempschema...;
       %else /* The two dots required here to prevent the error situation in case of tempschema is blank */
            %let tempschema=&dbtempschema..;
    %end;
    %else
    %do;
        %let tempschema=%str();
    %end;

    /* S0666172: DBMSTEMP support for Netezza and OLEDB */
    %If %index(%upcase(&dbtmplib.),DBMSTEMP=Y)  >= 1 %then
    %do;
        %if %index(%upcase(&dbtmplib),CONNECTION=GLOBAL) = 0 and &dbengine. ne OLEDB %then
        %do;
                %let MAError=727;
                %let syscc=100000;
                %let MAMsg=Macro[&sysmacroname.] Incompatible option for DBMSTEMP=Y. missing CONNECTION= option.;
                %return;
        %end;
        %else %if &dbengine. eq TERADATA %then
        %do;
            %if %index(%upcase(&dbtmplib),DATABASE=) >= 1 or
                %index(%upcase(&dbtmplib),FASTLOAD=Y)>= 1  %then
            %do;
                %let MAError=727;
                %let syscc=100000;
                %let MAMsg=Macro[&sysmacroname.] Incompatible option for DBMSTEMP=Y. missing CONNECTION= option.;
                %return;
            %end;
        %end;
        %else %if &dbengine. eq DB2 %then
            %let tempschema=SESSION.;
        %else %if &dbengine. eq OLEDB %then
            %let tempschema=##; /* S0666172: Two # is used to refer the Global temp schema in SQL server */
        %else
            %let tempschema=%str( );
    %end;
%end; /* end for NON SPDS database processing */


/* Generate the Unique Number for all database */

    /* Step 3: Get an unique sequence number to create input table name
     Note: ranuni(0) was not producing unique number when parallel communications are started executing
     almost at the same time defect S0472080. Now using sequence generator to avoid this duplicate
     name issue.  The first time insert of CIintbl to sequence data set should either manual or
     by executing single communication to avoid any table lock issue. Once this sequence record
     is inserted the lock issue will be handled with in the sequence generator macro maseq.sas */

    %maseq(MAMISC.Sequence, CIintbl, 1);

    %if (&SYSCC. >=2 and &SYSCC. ne 4) and &newid. > 0 %then
    %do;
        %let SYSCC=0;
    %end;
    %else %if &SYSERR. > 4 or (&SYSCC. >=2 and &SYSCC. ne 4) %then
    %do;
        %let MAError = 728;
        %let MAMsg = Macro[&sysmacroname]. Error generating sequence number for CI_Input table name. Check the log for details.;
        %return;
    %end;

    /* Create unique Input table name */
    %let unqintbl=CI_INTABLE&newid.;
    %put UNQINTBL= &unqintbl.;


/* Step 5: Remove duplicate records from the input data table */

/* 5a: Find out the number subject_id to be included in the by statement */
%let maxnumkey=0;
%let done=0;

/* Read comma separated subject_key in to individual macro variables */
%do %until (&done=1);
   %let maxnumkey = %eval(&maxnumkey+1);
   %let chsubjectidcol&maxnumkey.=%qscan(%bquote(&sub_key.),&maxnumkey.,',');
   %if &subjectidcol. ne %str() %then
       %let subjectidcol&maxnumkey.=%qscan(%bquote(&subjectidcol.),&maxnumkey.,',');

       %if &&chsubjectidcol&maxnumkey.= %then
       %do;
           %let done=1;
           %let maxnumkey=%eval(&maxnumkey-1);
       %end;
%end;

/* S0504678  TMTTracking_Cd will have either one of the following Treatment_Tracking_Cds
             Treatment_hash_Val varchar(32)- for 5.3 Campaign with Dynamic Treatments
             Treatment_Sk number(10)- for 5.3 campaign with Static Treatments
             Treatment_Cd varchar(32)- for 5.1 campaigns.

             This bellow special processing block is used to find out the correct Treatment_Sk
             from CI_Dynamic_Treatment_Attribute table incase of 5.3 Dynamic Treatments Campaign
             or from CI_Treatment Table in case of 5.3 Static Tretment campaing or 5.1 campaign

             The Empty/Null TMTTracking_Cd gets eliminated from CICommon webservice.  '-1' gets
             populated in case of any non-matched Treatment_Cd */

%if %upcase(&stpcall.) eq PT or %upcase(&stpcall.) eq RH %then
%do;
    /* Create a comma separated subject_key for this special processing */
    %let _sub_list_space=;
    %let _sub_list_comma=;
    %do i=1 %to &maxnumkey.;
        %if &i. = 1 %then
        %do;
            %let _sub_list_space=SUBJECT_ID&i.;
            %let _sub_list_comma=SUBJECT_ID&i.;
        %end;
        %if &i. > 1 %then
        %do;
            %let _sub_list_space= &_sub_list_space. SUBJECT_ID&i.;
            %let _sub_list_comma= &_sub_list_comma.,SUBJECT_ID&i.;
        %end;
    %end;

    data _TMT_INPUT(keep=CELL_PACKAGE_SK &_sub_list_space.
                         %if %upcase(&stpcall.) eq RH %then
                         %do;
                              RESPONSE_CD
                         %end;
                         TREATMENT_SK TMTTRACKING_CD);
        set &bc_data.; /* S0591527 */
        if verify(trim(TMTTRACKING_CD),'01234546789')=0 then
            TREATMENT_SK=input(TMTTRACKING_CD, Best.);
    run;

    proc sql;
        /* a) Join the Treatment_tracking_cd with the Treatment_Hash_Val in
              Dynamic Treatment table to get the corresponding treatment_sk for 5.3 Campaigns
              which has dynamic treatments */
        create table work._TREATMENT_SET as
            select inp.CELL_PACKAGE_SK,
                   &_sub_list_comma.,
                   %if %upcase(&stpcall.) eq RH %then
                   %do;
                       RESPONSE_CD,
                   %end;
                   dtmt.TREATMENT_SK,
                   inp.TMTTRACKING_CD as TREATMENT_HASH_VAL,
                   inp.TMTTRACKING_CD
            from _TMT_INPUT inp inner join
                 &libname..&cdta_tbl_nm. dtmt
                 on ( inp.CELL_PACKAGE_SK = dtmt.CELL_PACKAGE_SK and
                      inp.TMTTRACKING_CD = dtmt.TREATMENT_HASH_VAL);
    quit;

    /* b) Remove the absolute duplicate records from the above step. It is quite possible that the same treatment
          can be part of different package_hash.  This will result in duplicate records when the join is
          performed using only cell_package_sk and treatment_hash_val. */
    proc sort data=work._TREATMENT_SET nodupkey;
        by CELL_PACKAGE_SK &_sub_list_space.
           %if %upcase(&stpcall.) eq RH %then
           %do;
               RESPONSE_CD
           %end;
           TREATMENT_SK TREATMENT_HASH_VAL;
    run;

    proc sql;
        /* c) Join the input table and CI_Tratment table to get the correct and existing
              Treatment_sk for 5.3 campaigns which has static treatments */
        insert into work._TREATMENT_SET
                ( CELL_PACKAGE_SK,
                  &_sub_list_comma.,
                  %if %upcase(&stpcall.) eq RH %then
                  %do;
                      RESPONSE_CD,
                  %end;
                  TREATMENT_SK,
                  TREATMENT_HASH_VAL,
                  TMTTRACKING_CD  )
            select
                  inp.CELL_PACKAGE_SK,
                  &_sub_list_comma.,
                  %if %upcase(&stpcall.) eq RH %then
                  %do;
                      RESPONSE_CD,
                  %end;
                  ct.TREATMENT_SK,
                  '-1',
                  inp.TMTTRACKING_CD
            from _TMT_INPUT inp inner join
                 &libname..&ct_tbl_nm. ct
                 on ( inp.TREATMENT_SK = ct.TREATMENT_SK );

        /* d) Join the input table and CI_Tratment table by treatment_cd to get the correct
              and existing Treatment_sk for 5.1 campaigns */
        insert into work._TREATMENT_SET
                ( CELL_PACKAGE_SK,
                  &_sub_list_comma.,
                  %if %upcase(&stpcall.) eq RH %then
                  %do;
                      RESPONSE_CD,
                  %end;
                  TREATMENT_SK,
                  TREATMENT_HASH_VAL,
                  TMTTRACKING_CD)
            select
                  inp.CELL_PACKAGE_SK,
                  &_sub_list_comma.,
                  %if %upcase(&stpcall.) eq RH %then
                  %do;
                      RESPONSE_CD,
                  %end;
                  ct.TREATMENT_SK,
                  '-1',
                  TMTTRACKING_CD
            from _TMT_INPUT inp inner join
                 &libname..&ct_tbl_nm. ct
                 on ( inp.TMTTRACKING_CD = ct.TREATMENT_CD );
    quit;
%end; /* End of Special Processing block to accomadate both 5.1 and 5.3 campaigns*/

/* For Contact History Input Table */
%if %upcase(&stpcall.) eq CH %then
%do;

    /* S0472916 - contact_dttm included part of nodupkey to allow
       multiple contact records getting inserted in to CH table */

    /* Create the MATABLES in SPDTEMP domain to be in consitant with other process
       while trying to insert/update records to the target history tables */
    %if (&dbengine. eq SASSPDS) and (&action. eq USTA or &action. eq UPOP) %then
    %do;
           /* S0591527 */ /* S0516132 */ /* S0515661 */
           proc sort data=&data_tbl. out=&templibname..&unqintbl. nodupkey;
    %end;
    %else
    %do;
        proc sort data=&bc_data. nodupkey ;
    %end;
            by CELL_PACKAGE_SK
            %do i=1 %to &maxnumkey.;
                 %if (&action. eq UPOPDTTM) or (&action. eq USTADTTM) %then
                 %do;
                     SUBJECT_ID&i.
                 %end;
                 %else
                 %do;
                     &&CHSUBJECTIDCOL&i.
                 %end;
             %end;
             %if (&action. eq UPOPDTTM) %then
             %do;
                 CDTTM /* S0530203 - contact_dttm referred as cdttm as per the new name */
             %end;
             %else %if (&action. eq USTADTTM) %then
             %do;
                 descending CDTTM /* S0530203 - contact_dttm referred as cdttm as per the new name */
             %end;
            ; /* Termination for by statement */
        run;

    /* Additional step to remove the duplicate records if at all any from SDM client:
       The input table is already sorted by cell_package_sk/subject_id and in descending order of
       contact_dttm.  Now remove the duplicates by cell_package_sk and subject_id */
    %if &action. eq USTADTTM %then
    %do;
        proc sort data=&bc_data. nodupkey ; /* S0591527 */
            by CELL_PACKAGE_SK
            %do i=1 %to &maxnumkey.;
                SUBJECT_ID&i.
            %end;
            ; /* Termination for by statement */
        run;
    %end;

    /* S0479722 - Create a tiny table in work area with only the distinct cell_package_sk */
    /* S0558672 - Use the list of resptracking_cd from MA Input XML to create the tiny table*/
    /* S0515661 - SPDS Temp Libname changes */
    /* S0634713 - Since MACore doesn't store the deleted cell detials use the Communication_SK
                  the new macro variable to get list of cell_package_sk to be refreshed in the
                  contact history table */
    /* Taskid-57851 - add the ch_status along with the cell_package_sk in the tiny table to use in the update block */
    /* S0993481 */
    %if (&action. eq UPOP) or (&action. eq USTA) %then
    %do;
       proc sql;
            create table &worklib..&unqintbl._CPK_TMP as
                select CELL_PACKAGE_SK
                    from &libname..&cp_tbl_nm. ( &dsreadopts.)
                    where COMMUNICATION_SK=&communication_sk.
                order by CELL_PACKAGE_SK;
       quit;
       /* Taskid-57851 - add the status code in the tiny table */
       proc sql;
            create table    &worklib..&unqintbl._CPK as
                select      tmp.CELL_PACKAGE_SK , sts.CONTACT_HISTORY_STATUS_CD as CHSTATUS_CD
                from        &worklib..&unqintbl._CPK_TMP tmp
                left join   &ch_status_tbl. sts
                on          sts.CELL_PACKAGE_SK = tmp.CELL_PACKAGE_SK
       ;quit;
    %end;

    /* Exit the process if there is any error */
    %if &SYSERR. > 4 or (&SYSCC. >=2 and &SYSCC. ne 4) %then
    %do;
        %let MAError = 724;
        %let MAMsg = Macro[&sysmacroname.]. Error processing input table. Check the log for details.;
        %return;
    %end;
%end; /* End of stpcall=CH */

/* For Presented Treatment Input Table */
%else %if %upcase(&stpcall.) eq PT %then
%do;

    /* S0504678 New Treatment_Hash_Val column added and the Treatment_SK lookup is changed to
       CI_DYNAMCI_TREATMENT_ATTRIBUTE table based on Treatment_Hash_Val.

       Join the _treatment_set table with the data_tbl to prepare the final input set
       with all the records which has the treatment_sk/treatment_hash_val populated. */

    proc sql;
        create table GET_PT_TREATMENT_SK as
            select
                inp.CELL_PACKAGE_SK,
                %do i=1 %to &maxnumkey.;
                  inp.SUBJECT_ID&i. ,
                %end;
                CASE WHEN ts.TREATMENT_SK > 0 THEN ts.TREATMENT_SK
                ELSE -1
                END as TREATMENT_SK,
                CASE WHEN lengthn(ts.TREATMENT_HASH_VAL) > 0 THEN ts.TREATMENT_HASH_VAL
                ELSE '-1'
                END as TREATMENT_HASH_VAL,
                BCID,
                SUBJECT_ID,
                PTDTTM,
                PRESENTED_TREATMENT_HIST_DTTM,
                EXTERNAL_PRESENTED_INFO_ID1, /* S0715116 */
                EXTERNAL_PRESENTED_INFO_ID2
            from &bc_data. inp left join /* S0591527 */
                 _TREATMENT_SET ts
                 on (inp.CELL_PACKAGE_SK = ts.CELL_PACKAGE_SK and
                     inp.TMTTRACKING_CD = ts.TMTTRACKING_CD and
                      %do i=1 %to &maxnumkey.;
                          inp.SUBJECT_ID&i. = ts.SUBJECT_ID&i.
                          %if &i. < &maxnumkey. %then
                          %do;
                                and
                          %end;
                      %end;
                    ) ; /* Termination for the sql script */
    quit;

    /* Sort the input table and remove duplicate records. Mulitple entries for the same
       cell_package_sk, treatment_sk, treatment_hash_Val and subjectid
       differeing only by pt_dttm is retained */
    proc sort data=GET_PT_TREATMENT_SK out=&bc_data. nodupkey ; /* S0591527 */
        by CELL_PACKAGE_SK
        %do i=1 %to &maxnumkey.;
            SUBJECT_ID&i.
        %end;
        TREATMENT_SK TREATMENT_HASH_VAL PTDTTM;
    run;
%end; /* End of stpcall=PT */

/* For Response History Input Table */
%else %if %upcase(&stpcall.) eq RH %then
%do;

    /* S0504678 New Treatment_Hash_Val column added and the Treatment_SK lookup is changed to
       CI_DYNAMCI_TREATMENT_ATTRIBUTE table based on Treatment_Hash_Val. Populate -1 incase
       the Treatment_Hash_Val is empty or Null

       Join the _treatment_set table with the data_tbl to prepare the final input set
       with all the records which has the treatment_sk/treatment_hash_val populated.

       For Response History the Treatment_Sk and Treatment_Hash_Val may or may not have
       value populated. So if the value is null then replace it with -1 as these 2 columns
       are part of the composite primary key */

    proc sql;
        create table GET_RH_TREATMENT_SK as
            select
                inp.CELL_PACKAGE_SK,
                %do i=1 %to &maxnumkey.;
                  inp.SUBJECT_ID&i. ,
                %end;
                inp.RESPONSE_CD,
                CASE WHEN ts.TREATMENT_SK > 0 THEN ts.TREATMENT_SK
                ELSE -1
                END as TREATMENT_SK,
                CASE WHEN lengthn(ts.TREATMENT_HASH_VAL) > 0 THEN ts.TREATMENT_HASH_VAL
                ELSE '-1'
                END as TREATMENT_HASH_VAL,
                INFERRED_RESPONSE_FLG,
                RESPONSE_CHANNEL_CD,
                EXTERNAL_RESPONSE_INFO_ID1,
                EXTERNAL_RESPONSE_INFO_ID2,
                BCID,
                SUBJECT_ID,
                RDTTM,
                RESPONSE_DTTM
            from &bc_data. inp left join /* S0591527 */
                  _TREATMENT_SET ts
                  on ( inp.CELL_PACKAGE_SK = ts.CELL_PACKAGE_SK and
                       inp.RESPONSE_CD = ts.RESPONSE_CD and
                       inp.TMTTRACKING_CD = ts.TMTTRACKING_CD and
                         %do i=1 %to &maxnumkey.;
                             inp.SUBJECT_ID&i. = ts.SUBJECT_ID&i.
                             %if &i. < &maxnumkey. %then
                             %do;
                                 and
                             %end;
                          %end;
                        ) ; /* Termination for the Sql statement */
    quit;

    /* Assign the Response_Cd configured by Response_Channel if any */
    /* This section is included for the SDM client as well per defect S0499388 */
    proc sql;
        create table &bc_data. as /* S0591527 */
            select
            %do i=1 %to &maxnumkey.;
                  SUBJECT_ID&i. ,
            %end;
                  CELL_PACKAGE_SK,
                  TREATMENT_SK,
                  TREATMENT_HASH_VAL,
                  INFERRED_RESPONSE_FLG,
                  RHI.RESPONSE_CHANNEL_CD,
                  EXTERNAL_RESPONSE_INFO_ID1,
                  EXTERNAL_RESPONSE_INFO_ID2,
                  BCID,
                  SUBJECT_ID,
                  RDTTM,
                  RESPONSE_DTTM,
            CASE WHEN CRCR.RESPONSE_CD='' then RHI.RESPONSE_CD
                 ELSE CRCR.RESPONSE_CD
                 END AS RESPONSE_CD
            FROM GET_RH_TREATMENT_SK RHI LEFT JOIN
                 &libname..&crcr_tbl_nm. CRCR
                 ON(RHI.RESPONSE_CHANNEL_CD=CRCR.RESPONSE_CHANNEL_CD and
                    UPCASE(RHI.RESPONSE_CD)=UPCASE(CRCR.RESPONSE_CHANNEL_RESPONSE_CD)
                    );
    quit;

    /* Error handling is required here */
    %if &SYSDBRC. ne 0 %then
    %do;
        %let MAError = 781;
        %let MAMsg = Macro[&sysmacroname]. Error processing Response_Channel_Response_Cd. SYSDBRC=[&sysdbrc.] and SYSDBMSG=[&sysdbmsg.]. Check the log for details.;
        %return;
    %end;

    /* Remove the duplicate records by all the below mentioned by variables */
    proc sort data=&bc_data. nodupkey ; /* S0591527 */
        by CELL_PACKAGE_SK TREATMENT_SK TREATMENT_HASH_VAL
        %do i=1 %to &maxnumkey.;
             SUBJECT_ID&i.
        %end;
        RESPONSE_CD RDTTM;
    run;
%end; /* End of stpcall=RH */

%mend;
