/****************************************************************************/
/* Copyright (c) 2007 - 2013 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME: machcnt.sas                                                        */
/* VERSION: 5.41                                                            */
/* DESCRIPTION: Common Data Interface macro to compute contact counts       */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %machcnt                                                          */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                    Value                 */
/* action     UPOP/USTA/UPOPDTTM/USTADTTM                                   */
/* cp_tbl     Cell_Package table name                                       */
/* cpt_tbl    Cell_Package_Treatment_Cnt table name                         */
/* data_tbl   Input Data table                                              */
/* dbengine   Name of the database Engine,Update tables Statements are      */
/*            Engine Dependent                                              */
/* f_con_cd   Code for Failed Contacts                                      */
/* libnm      Library Name for CI Common Data Model                         */
/* newid      Unique new id for creating data processing tables in templib  */
/* popexists  Population in case of action=USTA                             */
/* tmplibnm   Temporary Library Name for data processing                    */
/****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.2.7.2.5.4.2.2.7 $ ;
/* Revision Date: $Date: 2013/10/31 12:22:59 $ ;
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 04Jun09 S0583077  New Version                                            */
/*                                                                          */
/* 30Jun09 S0558672 Join the count table with cell_package_sk input set to  */
/*                   get all possible cell_package_sk (0 count cell)        */
/*                   for the communication                                  */
/*                                                                          */
/* 02Sep09 S0605419 create a new table to hold the data instead of having   */
/*                  single proc sql statement to select & insert in the     */
/*                  same table as this does not work with SPDS              */
/*                                                                          */
/* 24Mar10 S0651614 Aster Partition Key Change                              */
/*                                                                          */
/* 24Mar11 S0744501 modified "where not exists" query to use "EXCEPT" clause*/
/*                                                                          */
/* 28Jun11 S0744863   Added support for GREENPLUM                           */
/*                                                                          */
/* 09Dec11 S0791301  create/refer all the table/col names in upper case     */
/*                                                                          */
/* 15Jun12 S0782605   use proc sql to get counts instead of proc summary    */
/* 06Dec12 S0779594   Modified Treat_cnt table update for UPOPDTTM          */
/* 19Apr13 S0930917   Special handling for Netezza counts                   */
/* 30Oct13 S1015033 Correcting bad locking                                  */
/****************************************************************************/
%macro machcnt(action=,cp_tbl=,cpt_tbl=,data_tbl=,dbengine=,f_con_cd=,libnm=,newid=,
       popexists=,tmplibnm=,dbconnect=,targetschema=,tempschema=);

    /* machcnt steps                                            */
    /* Step1- Initialisation                                    */
    /* Step2- Generate Summary table based on input parameters  */
    /* Step3- Update Target tables based on input paramters     */
    /* Step4- drop work tables                                  */

    %if  &MAError.  ne %str() %then  %return;

    %if &MADebug. eq Y %then
    %do;
        %put MACHCNT - Current Version of the code: 541MA11;
        %put MACHCNT macro Start Time - %sysfunc(datetime(), datetime24.2);
    %end;

/* Step1 Initialization */

    /* set options DBIDIRECTEXEC & SYS_SQL_IP_SPEEDO to enable sas directly send the proc sql statements to DBMS*/
    /* SPEEDO = YES not supported for following db's*/
    %if &dbengine. ne OLEDB  and &dbengine. ne ODBC %then
    %do;
        options DBIDIRECTEXEC;
        %let SYS_SQL_IP_SPEEDO=YES;
        option dsnferr;
    %end;

    /* Defining lock control variable for Netezza (can be overridden globally) */
    %if %symexist(ci_sys_nza_pg_locks) eq 0 %then
    %do;
        %let ci_sys_nza_pg_locks = 1;
    %end;
    /*Create a variable to hold temp table names to drop at the end of process*/
    %let TblNames=%sysfunc(strip(&data_tbl.)) ;

    %if &MADebug. eq Y %then
    %do;
        %let st=%sysfunc(datetime()) ;
        %put Parameters passed to machcnt;
        %put Action: &action;
        %put ci_cell_package table name: &cp_tbl;
        %put ci_cell_package_treatment_cnt table name: &cpt_tbl;
        %put Input Data table name: &data_tbl;
        %put Code for Failed Contacts:&f_con_cd;
        %put Population in case of action=USTA :&popexists;
        %put Library for CI Common Data Model: &libnm;
        %put NewId a unique number for creating intermediate data processing table: &newid;
        %put Temporary Library for CI Common Data Model: &tmplibnm;
    %end;

    /* Updating status for the existing MA campaign doesn't affect the contacts count.  Do nothing in   this macro */
    %if &action. =USTA and &popexists. > 0 %then %return;

/*Step2 Create Summary*/
    /* S0651614 */
    %if &dbengine=ASTER or &dbengine=GREENPLM %then
    %do;
        %machast(action=&action ,cp_tbl=&cp_tbl ,cpt_tbl=&cpt_tbl ,data_tbl=&data_tbl ,dbengine=&dbengine ,
                f_con_cd=&f_con_cd ,libnm=&libnm ,newid=&newid ,popexists=&popexists ,tmplibnm=&tmplibnm );
        %if  &MAError. ne %str() %then %goto exit;
    %end;
    %else
    %do;
        /*Step2a generate summary when action = UPOPDTTM or UPOP or USTA*/
        %if &action. =UPOPDTTM or &action. = UPOP or  &action. = USTA %then
        %do;
            /*Step2a1 generate summary from input table*/
            %if &MADebug = Y %then  %put Step2a1 Start Time:%sysfunc(datetime(),datetime20);
            /*Append the tmp table to TblNames variable*/
            %let TblNames=&TblNames. %sysfunc(strip(CH_CP_COUNTS&newid.));
            %* S0782605 - use proc sql to get counts instead of proc summary ;
            proc sql;
                create table &tmplibnm..CH_CP_COUNTS&newid. as
                    select  CELL_PACKAGE_SK, PACKAGE_HASH_VAL , count(*) as ATTEMPTED_CNT
                    from    &tmplibnm..&data_tbl.
                    group by CELL_PACKAGE_SK, PACKAGE_HASH_VAL;
            quit;

            %if &MADebug = Y %then  %put Step2a1 End Time:%sysfunc(datetime(),datetime20);

            /* S0558672 */
            /* if the input table does not have any of the cell_package_sk in dist_cpk  */
            /* table then add them in the ch_cp_counts with Attempted_cnt as 0 */

            %if &action. = UPOP or &action. = USTA %then
            %do;
                /* S0605419 S0744501 */
                %let TblNames=&TblNames. %sysfunc(strip(CH_CP_DELETED&newid.));
                proc sql;
                    create table    &tmplibnm..CH_CP_DELETED&newid. as
                        select      t.CELL_PACKAGE_SK, 0 as ATTEMPTED_CNT
                        from
                        (select     CELL_PACKAGE_SK from &tmplibnm..&unqintbl._CPK
                        except
                        select      CELL_PACKAGE_SK from &tmplibnm..CH_CP_COUNTS&newid.)t
                ;quit;

                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                proc sql;
                    insert into     &tmplibnm..CH_CP_COUNTS&newid.(CELL_PACKAGE_SK,ATTEMPTED_CNT)
                        select      CELL_PACKAGE_SK,ATTEMPTED_CNT
                        from        &tmplibnm..CH_CP_DELETED&newid.
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =739; %goto exit; %end;

            %end;


            /*Step2a2 Get the Track Presented Treatment flag for the input cell packages this is only for Action=UPOPDTTM*/
            %if &action. =UPOPDTTM %then
            %do;
                %if &MADebug = Y %then  %put Step2a2 Start Time:%sysfunc(datetime(),datetime20);
                %let TblNames=&TblNames. %sysfunc(strip(CH_CP_TMT_CNT&newid.));
                proc sql;
                    create table    &tmplibnm..CH_CP_TMT_CNT&newid. as
                        select      t.*,
                                    cp.TRACK_PRESENTED_TREATMENTS_FLG as TPT_FLG
                        from        &tmplibnm..CH_CP_COUNTS&newid. t
                        inner join  &libnm..&cp_tbl. cp
                        on          t.CELL_PACKAGE_SK = cp.CELL_PACKAGE_SK
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                %let pxt_tbl = CI_PACKAGE_X_TREATMENT;
                %* S0779594 - modify ch_cpt_tmt_cnt table creation
                For offer arbitration campaign - all the treatments may not be offered, the offered treatment need to be figured out
                using the package_hash_val from the contact history input table
                1.if the package_hash_val is NULL then get the for the cell_package_sk get the list of treatments from pkg_x_treat table for matching contributing_cell_package_sk
                2.if the package_hash_val is not null then get the list of treatments for the matching package_hash_val from pkg_x_treat
                NOTE - following quert needs to be tested for different input data conditions ;
                %let TblNames=&TblNames. %sysfunc(strip(CH_CPT_TMT_CNT&newid.));
                proc sql;
                        create table    &tmplibnm..CH_CPT_TMT_CNT&newid. as
                            /* for matching package_hash_val get the treatment_sk from pkg_x_treatment*/
                            select      t.*,pxt.TREATMENT_SK
                            from        &tmplibnm..CH_CP_TMT_CNT&newid. t
                            inner join  &libnm..&pxt_tbl. pxt
                            on          t.PACKAGE_HASH_VAL= pxt.PACKAGE_HASH_VAL
                            /* if there are no treatments added in the campaign then its published with NOT_TREATMENT and
                            package_hash_val for such cell is null hence , for null package_hash_val get the treatment_sk from pkg_x_treatment for matching contributing_cell_package_sk */
                            union
                            select      t.*,pxt.TREATMENT_SK
                            from        &tmplibnm..CH_CP_TMT_CNT&newid. t
                            inner join  &libnm..&pxt_tbl. pxt
                            on          t.CELL_PACKAGE_SK= pxt.CONTRIBUTING_CELL_PACKAGE_SK
                            where       t.PACKAGE_HASH_VAL is null
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
                %if &MADebug = Y %then  %put Step2a2 End Time:%sysfunc(datetime(),datetime20);
            %end;
            %else
            %do;
                /*For Generic Update Get the Treatment_sk for each cell_package*/
                %if &dbengine. ne ORACLE AND &dbengine. ne DB2 AND &dbengine. ne TERADATA AND &dbengine. ne NETEZZA %then
                %do;
                    %let TblNames=&TblNames. %sysfunc(strip(CH_CPT_COUNTS&newid.));
                    proc sql;
                        create table    &tmplibnm..CH_CPT_COUNTS&newid. as
                            select      t.*,cpt.TREATMENT_SK
                            from        &tmplibnm..CH_CP_COUNTS&newid. t
                            inner join  &libnm..&cpt_tbl. cpt
                            on          t.CELL_PACKAGE_SK = cpt.CELL_PACKAGE_SK
                    ;quit;
                    %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
                %end;
            %end;
        %end;


        /*Step2b generate summary when action = USTADTTM*/
        %if &action. =USTADTTM %then
        %do;
            %if &MADebug = Y %then  %put Step2b Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(CH_CP_FAILED_CNT&newid.));
            %* S0782605 - use proc sql to get counts instead of proc summary ;
            proc sql;
                create table    &tmplibnm..CH_CP_FAILED_CNT&newid.  as
                    select      CELL_PACKAGE_SK , count(*) as FAILED_CNT
                    from        &tmplibnm..&data_tbl.
                    where       CHSTATUS_CD="&f_con_cd."
                    group  by   CELL_PACKAGE_SK ;
            quit;

            /*For Generic Update Get the Treatment_sk for each cell_package*/
            %if &dbengine. ne ORACLE AND  &dbengine. ne DB2 AND &dbengine. ne TERADATA AND &dbengine. ne NETEZZA %then
            %do;
                %let TblNames=&TblNames. %sysfunc(strip(CH_CPT_FAILED_CNT&newid.));
                proc sql;
                    create table    &tmplibnm..CH_CPT_FAILED_CNT&newid. as
                        select      t.*,cpt.TREATMENT_SK
                        from        &tmplibnm..CH_CP_FAILED_CNT&newid. t
                        inner join  &libnm..&cpt_tbl. cpt
                        on          t.CELL_PACKAGE_SK = cpt.CELL_PACKAGE_SK
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
            %end;
            %if &MADebug = Y %then  %put Step2b End Time:%sysfunc(datetime(),datetime20);
        %end;
    %end;

    /* S0951678 - for Netezza db do not remove the ".." (two dots) from the tempschema */
    /** Special handling of explicit passthrough table names (where necessary) */
    data _null_;
        tempschema = "&tempschema";
        %if &dbengine. = NETEZZA %then
        %do;
            call symput("cnt_temp", tempschema);
        %end;
        %else
        %do;
            call symput("cnt_temp", compress(tempschema, '.', 's'));
        %end;
        targetschema = "&targetschema";
        call symput("cnt_target", compress(targetschema, '.', 's'));
    run;

    %if %length(&cnt_temp) ne 0 %then
    %do;
        /* S0951678 - for Netezza, tempschema already contains two dots*/
        %if %upcase(&dbengine.) eq NETEZZA %then
        %do;
            %let tmp_cp_tmt_tbl = &cnt_temp.CH_CP_TMT_CNT&newid.;
            %let tmp_cpt_tmt_tbl = &cnt_temp.CH_CPT_TMT_CNT&newid.;
            %let tmp_cp_cnt_tbl = &cnt_temp.CH_CP_COUNTS&newid.;
            %let tmp_cp_fail_tbl  = &cnt_temp.CH_CP_FAILED_CNT&newid.;

        %end;
        %else
        %do;
            %let tmp_cp_tmt_tbl = &cnt_temp..CH_CP_TMT_CNT&newid.;
            %let tmp_cpt_tmt_tbl = &cnt_temp..CH_CPT_TMT_CNT&newid.;
            %let tmp_cp_cnt_tbl = &cnt_temp..CH_CP_COUNTS&newid.;
            %let tmp_cp_fail_tbl  = &cnt_temp..CH_CP_FAILED_CNT&newid.;
        %end;

    %end;
    %else
    %do;
        %let tmp_cp_tmt_tbl = CH_CP_TMT_CNT&newid.;
        %let tmp_cpt_tmt_tbl = CH_CPT_TMT_CNT&newid.;
        %let tmp_cp_cnt_tbl = CH_CP_COUNTS&newid.;
        %let tmp_cp_fail_tbl  = CH_CP_FAILED_CNT&newid.;
    %end;
    %if %length(&cnt_target) ne 0 %then
    %do;
        %let tgt_cp_tbl = &cnt_target..&cp_tbl.;
        %let tgt_cpt_tbl = &cnt_target..&cpt_tbl.;
    %end;
    %else
    %do;
        %let tgt_cp_tbl = &cp_tbl.;
        %let tgt_cpt_tbl = &cpt_tbl.;
    %end;

/*Step3 Update Target tables */
    /*Step3a.Update Target table when action =UPOPDTTM */
    %if &action. = UPOPDTTM %then
    %do;
        %if &dbengine. = NETEZZA %then
        %do;
            proc sql;
            &dbconnect;
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=753;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            /* Step3a1 Update Cell Package Target table */
            %if &MADebug = Y %then  %put Step3a1 Start Time:%sysfunc(datetime(),datetime20);
            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cp_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cp_tbl tt
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM
                    &tmp_cp_tmt_tbl st
                WHERE
                    tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cp_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cp_tbl tt
                SET
                    TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM
                    &tmp_cp_tmt_tbl st
                WHERE
                    tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
                    AND st.TPT_FLG='N'
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &MADebug = Y %then  %put Step3a1 End Time:%sysfunc(datetime(),datetime20);

            /*Step3a2 Update Cell package Treatment target table*/
            %if &MADebug = Y %then  %put Step3a2 Start Time:%sysfunc(datetime(),datetime20);

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cpt_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cpt_tbl tt
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM
                    &tmp_cpt_tmt_tbl st
                WHERE
                    st.CELL_PACKAGE_SK = tt.CELL_PACKAGE_SK
                    AND st.TREATMENT_SK = tt.TREATMENT_SK
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cpt_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cpt_tbl tt
                SET
                    TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM
                    &tmp_cpt_tmt_tbl st
                WHERE
                    st.CELL_PACKAGE_SK = tt.CELL_PACKAGE_SK
                    AND st.TREATMENT_SK = tt.TREATMENT_SK
                    AND st.TPT_FLG='N'
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &MADebug = Y %then  %put Step3a2 End Time:%sysfunc(datetime(),datetime20);

            quit; /* PROC SQL */

        %end; /* of IF NETEZZA */

        %else %if &dbengine. = TERADATA or &dbengine. = DB2 or &dbengine. = ORACLE %then
        %do;

           proc sql;
           &dbconnect.
           %if &SQLXRC. ne 0 %then
           %do;
              %let MAError=753;
              %let SYSCC=100000;
              %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
              %return;
           %end;

        /*Step3a1 Update Cell Package Target table */
        %if &MADebug = Y %then  %put Step3a1 Start Time:%sysfunc(datetime(),datetime20);
        execute (
            MERGE INTO &tgt_cp_tbl tt
            USING &tmp_cp_tmt_tbl st
            ON (
                tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            )
            WHEN MATCHED THEN UPDATE
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

        execute (
            MERGE INTO &tgt_cp_tbl tt
            USING &tmp_cp_tmt_tbl st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            AND st.TPT_FLG='N'
            )
            WHEN MATCHED THEN UPDATE
                SET
                    TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %if &MADebug = Y %then  %put Step3a1 End Time:%sysfunc(datetime(),datetime20);

        /*Step3a2 Update Cell package Treatment target table*/
        %if &MADebug = Y %then  %put Step3a2 Start Time:%sysfunc(datetime(),datetime20);
        execute (
            MERGE INTO &tgt_cpt_tbl tt
            USING &tmp_cpt_tmt_tbl st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            AND tt.TREATMENT_SK = st.TREATMENT_SK
            )
            WHEN MATCHED THEN UPDATE
            SET
                TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT + st.ATTEMPTED_CNT,
                PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

        execute (
            MERGE INTO &tgt_cpt_tbl tt
            USING &tmp_cpt_tmt_tbl st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            AND tt.TREATMENT_SK = st.TREATMENT_SK
            AND st.TPT_FLG='N'
            )
            WHEN MATCHED THEN UPDATE
                SET
                    TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT + st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %if &MADebug = Y %then  %put Step3a2 End Time:%sysfunc(datetime(),datetime20);

        quit;

        %end;
        /*Else use generic update using Data Modify*/
        %else
        %do;
            %let pdttm=%sysfunc(datetime(),datetime20);
            /* Update Cell Package Target table */
            data &libnm..&cp_tbl.;
            modify &libnm..&cp_tbl.(&dswriteopts.) &tmplibnm..CH_CP_TMT_CNT&newid.;
            by CELL_PACKAGE_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                TOTAL_ATTEMPTED_CONTACT_CNT=SUM(TOTAL_ATTEMPTED_CONTACT_CNT, ATTEMPTED_CNT);
                if upcase(TPT_FLG)='N' then
                do;
                    TOTAL_MARKETING_CONTACT_CNT=SUM(TOTAL_MARKETING_CONTACT_CNT, ATTEMPTED_CNT);
                end;
                PROCESSED_DTTM="&pdttm"dt;
                replace;
                end;
                otherwise do;stop;end;
            end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
            %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
            %end;
            %end;
            %else %if &SYSERR.> 4  %then %do;%let MAError =737;%goto exit; %end;

            %let pdttm=%sysfunc(datetime(),datetime20);
            /*Update Cell_Package_Treatment_Cnt Table*/
            data &libnm..&cpt_tbl.;
            modify &libnm..&cpt_tbl.(&dswriteopts.) &tmplibnm..CH_CPT_TMT_CNT&newid.;
            by CELL_PACKAGE_SK TREATMENT_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                TOTAL_ATTEMPTED_CONTACT_CNT=SUM(TOTAL_ATTEMPTED_CONTACT_CNT, ATTEMPTED_CNT);
                if upcase(TPT_FLG)='N' then
                do;
                    TOTAL_MARKETING_CONTACT_CNT=SUM(TOTAL_MARKETING_CONTACT_CNT, ATTEMPTED_CNT);
                end;
                PROCESSED_DTTM="&pdttm"dt;
                replace;
                end;
                otherwise do;stop;end;
            end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
            %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
            %end;
            %end;
            %else %if &SYSERR.> 4  %then %do;%let MAError =737;%goto exit; %end;
        %end;
        %end;
    /*Step3b.Update Target table when action =UPOP or USTA */
    %if &action. = UPOP or  &action. = USTA %then
    %do;
        %if &dbengine. = NETEZZA %then
        %do;
            proc sql;
            &dbconnect;
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=753;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            /*Step3b1 Update Cell Package Target table */
            %if &MADebug = Y %then  %put Step3b1 Start Time:%sysfunc(datetime(),datetime20);
            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cp_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cp_tbl tt
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = st.ATTEMPTED_CNT,
                    TOTAL_MARKETING_CONTACT_CNT = st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM
                    &tmp_cp_cnt_tbl st
                WHERE
                    tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &MADebug = Y %then  %put Step3b1 End Time:%sysfunc(datetime(),datetime20);

            /*Step3b2 Update Cell package Treatment target table*/
            %if &MADebug = Y %then  %put Step3b2 Start Time:%sysfunc(datetime(),datetime20);
            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cpt_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cpt_tbl tt
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = st.ATTEMPTED_CNT,
                    TOTAL_MARKETING_CONTACT_CNT = st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM (
                    select
                        ATTEMPTED_CNT,
                        c.CELL_PACKAGE_SK,
                        TREATMENT_SK
                    from
                        &tmp_cp_cnt_tbl c
                    inner join
                        &tgt_cpt_tbl t ON c.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
                ) st
                WHERE
                    tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
                    AND tt.TREATMENT_SK = st.TREATMENT_SK
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &MADebug = Y %then  %put Step3b2 End Time:%sysfunc(datetime(),datetime20);

            quit;
        %end; /* of %if NETEZZA */

        %else %if &dbengine. = TERADATA or &dbengine. eq DB2 or &dbengine. eq ORACLE %then
        %do;

           proc sql;
        &dbconnect.
        %if &SQLXRC. ne 0 %then
        %do;
            %let MAError=753;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /*Step3b1 Update Cell Package Target table */
        %if &MADebug = Y %then  %put Step3b1 Start Time:%sysfunc(datetime(),datetime20);

        execute (
            MERGE INTO &tgt_cp_tbl tt
            USING &tmp_cp_cnt_tbl st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            )
            WHEN MATCHED THEN UPDATE
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = st.ATTEMPTED_CNT,
                    TOTAL_MARKETING_CONTACT_CNT = st.ATTEMPTED_CNT,
                    PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %if &MADebug = Y %then  %put Step3b1 End Time:%sysfunc(datetime(),datetime20);

        /*Step3b2 Update Cell package Treatment target table*/
        %if &MADebug = Y %then  %put Step3b2 Start Time:%sysfunc(datetime(),datetime20);

        execute (
            MERGE INTO &tgt_cpt_tbl tt
            USING (
            select
                ATTEMPTED_CNT,
                c.CELL_PACKAGE_SK,
                TREATMENT_SK
            from
                    &tmp_cp_cnt_tbl c
            inner join
                    &tgt_cpt_tbl t ON c.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
            ) st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            AND tt.TREATMENT_SK = st.TREATMENT_SK
            )
            WHEN MATCHED THEN UPDATE
            SET
                TOTAL_ATTEMPTED_CONTACT_CNT = st.ATTEMPTED_CNT,
                TOTAL_MARKETING_CONTACT_CNT = st.ATTEMPTED_CNT,
                PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %if &MADebug = Y %then  %put Step3b2 End Time:%sysfunc(datetime(),datetime20);

        quit;

        %end;
        /*Else use generic update using Data Modify*/
        %else
        %do;
            %let pdttm=%sysfunc(datetime(),datetime20);
            /* Update Cell Package Target table */
            data &libnm..&cp_tbl.;
            modify &libnm..&cp_tbl.(&dswriteopts.) &tmplibnm..CH_CP_COUNTS&newid.;
            by CELL_PACKAGE_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                TOTAL_ATTEMPTED_CONTACT_CNT = ATTEMPTED_CNT;
                TOTAL_MARKETING_CONTACT_CNT = ATTEMPTED_CNT;
                PROCESSED_DTTM="&pdttm"dt;
                replace;
                end;
                otherwise do;stop;end;
            end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
            %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
            %end;
            %end;
            %else %if &SYSERR.> 4  %then %do;%let MAError =737;%goto exit; %end;

            %let pdttm=%sysfunc(datetime(),datetime20);
            /*Update Cell_Package_Treatment_Cnt Table*/
            data &libnm..&cpt_tbl.;
            modify &libnm..&cpt_tbl.(&dswriteopts.) &tmplibnm..CH_CPT_COUNTS&newid.;
            by CELL_PACKAGE_SK TREATMENT_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                TOTAL_ATTEMPTED_CONTACT_CNT=ATTEMPTED_CNT;
                TOTAL_MARKETING_CONTACT_CNT=ATTEMPTED_CNT;
                PROCESSED_DTTM="&pdttm"dt;
                replace;
                end;
                otherwise do;stop;end;
            end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
            %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
            %end;
            %end;
            %else %if &SYSERR.> 4  %then %do;%let MAError =737;%goto exit; %end;
        %end;
        %end;
    /*Step3c.Update Target table when action =USTADTTM */
    %if &action. = USTADTTM %then
    %do;
        %if &dbengine. = NETEZZA %then
        %do;
            proc sql;
            &dbconnect;
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=753;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            /*Step3c1 Update Cell Package Target table */
            %if &MADebug = Y %then  %put Step3c1 Start Time:%sysfunc(datetime(),datetime20);
            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cp_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cp_tbl
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = tt.TOTAL_ATTEMPTED_CONTACT_CNT - st.FAILED_CNT,
                    TOTAL_MARKETING_CONTACT_CNT = tt.TOTAL_MARKETING_CONTACT_CNT - st.FAILED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM
                    &tmp_cp_fail_tbl st
                WHERE
                    tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &MADebug = Y %then  %put Step3c1 End Time:%sysfunc(datetime(),datetime20);

            /*Step3c2 Update Cell package Treatment target table*/
            %if &MADebug = Y %then  %put Step3c2 Start Time:%sysfunc(datetime(),datetime20);
            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (BEGIN LOCAL TRANSACTION) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
                EXECUTE (LOCK TABLE &tgt_cpt_tbl) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            EXECUTE (
                UPDATE
                    &tgt_cpt_tbl tt
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = tt.TOTAL_ATTEMPTED_CONTACT_CNT - st.FAILED_CNT,
                    TOTAL_MARKETING_CONTACT_CNT = tt.TOTAL_MARKETING_CONTACT_CNT - st.FAILED_CNT,
                    PROCESSED_DTTM = NOW()
                FROM (
                    select
                        FAILED_CNT,
                        c.CELL_PACKAGE_SK,
                        TREATMENT_SK
                    from
                        &tmp_cp_fail_tbl. c
                    inner join
                        &tgt_cpt_tbl. t ON c.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
                ) st
                WHERE
                    tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
                    AND tt.TREATMENT_SK = st.TREATMENT_SK
            ) by &dbengine;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            %if &dbengine. eq NETEZZA and &ci_sys_nza_pg_locks. eq 1 %then
            %do;
                EXECUTE (COMMIT) by &dbengine;
                %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
            %end;

            %if &MADebug = Y %then  %put Step3c2 End Time:%sysfunc(datetime(),datetime20);

            quit;

        %end; /* of %if NETEZZA */

        %else %if &dbengine. = TERADATA or &dbengine. eq DB2 or &dbengine. eq ORACLE %then
        %do;

           proc sql;
        &dbconnect.
        %if &SQLXRC. ne 0 %then
        %do;
            %let MAError=753;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /*Step3c1 Update Cell Package Target table */
        %if &MADebug = Y %then  %put Step3c1 Start Time:%sysfunc(datetime(),datetime20);

        execute (
            MERGE INTO &tgt_cp_tbl tt
            USING &tmp_cp_fail_tbl st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            )
            WHEN MATCHED THEN UPDATE
                SET
                    TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT - st.FAILED_CNT,
                    TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT - st.FAILED_CNT,
                    PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %if &MADebug = Y %then  %put Step3c1 End Time:%sysfunc(datetime(),datetime20);

        /*Step3c2 Update Cell package Treatment target table*/
        %if &MADebug = Y %then  %put Step3c2 Start Time:%sysfunc(datetime(),datetime20);
        execute (
            MERGE INTO &tgt_cpt_tbl tt
            USING (
            select
                    FAILED_CNT,
                c.CELL_PACKAGE_SK,
                TREATMENT_SK
            from
                    &tmp_cp_fail_tbl c
            inner join
                    &tgt_cpt_tbl t ON c.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
            ) st
            ON (
            tt.CELL_PACKAGE_SK = st.CELL_PACKAGE_SK
            AND tt.TREATMENT_SK = st.TREATMENT_SK
            )
            WHEN MATCHED THEN UPDATE
            SET
                TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT - st.FAILED_CNT,
                TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT - st.FAILED_CNT,
                PROCESSED_DTTM = CURRENT_TIMESTAMP
        ) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        execute (COMMIT) by &dbengine.;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %if &MADebug = Y %then  %put Step3c2 End Time:%sysfunc(datetime(),datetime20);

        quit;

        %end;
        /*Else use generic update using Data Modify*/
        %else
        %do;
            %let pdttm=%sysfunc(datetime(),datetime20);
            /* Update Cell Package Target table */
            data &libnm..&cp_tbl.;
            modify &libnm..&cp_tbl.(&dswriteopts.) &tmplibnm..CH_CP_FAILED_CNT&newid.;
            by CELL_PACKAGE_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT - FAILED_CNT;
                TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT - FAILED_CNT;
                PROCESSED_DTTM="&pdttm"dt;
                replace;
                end;
                otherwise do;stop;end;
            end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
            %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
            %end;
            %end;
            %else %if &SYSERR.> 4  %then %do;%let MAError =737;%goto exit; %end;

            %let pdttm=%sysfunc(datetime(),datetime20);
            /*Update Cell_Package_Treatment_Cnt Table*/
            data &libnm..&cpt_tbl.;
            modify &libnm..&cpt_tbl.(&dswriteopts.) &tmplibnm..CH_CPT_FAILED_CNT&newid.;
            by CELL_PACKAGE_SK TREATMENT_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                TOTAL_ATTEMPTED_CONTACT_CNT = TOTAL_ATTEMPTED_CONTACT_CNT - FAILED_CNT;
                TOTAL_MARKETING_CONTACT_CNT = TOTAL_MARKETING_CONTACT_CNT - FAILED_CNT;
                PROCESSED_DTTM="&pdttm"dt;
                replace;
                end;
                otherwise do;stop;end;
            end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
            %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
            %end;
            %end;
            %else %if &SYSERR.> 4  %then %do;%let MAError =737;%goto exit; %end;
        %end;
        %end;
/*Step4 Drop Work tables*/
    %exit:
        /*if MAError is set then set the MAMsg*/
        %if  &MAError.  ne %str() %then
        %do;
            %if &MAError = 736 %then %let MAMsg = Macro[&sysmacroname.]. Error in Create table Statement SQLRC[&SQLRC]. Check the log for details;
            %if &MAError = 737 %then %let MAMsg = Macro[&sysmacroname.]. Error in Update table Statement SQLRC[&SQLRC]. Check the log for details;
            %if &MAError = 739 %then %let MAMsg = Macro[&sysmacroname.]. Error in Insert into table Statement SQLRC[&SQLRC]. Check the log for details;
            %if &MAError = 766 %then %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
        /*Delete the temporary tables*/
        %macdidel(libexist=Y, tblnms=&TblNames., templibname=&tmplibnm.);

        %if &MADebug. eq Y %then
        %do;
            %put Step4 End Time:%sysfunc(datetime(),datetime20);
            %let et=%sysfunc(datetime());
            /*print the time taken for execution;*/
            %mautlpt(start_tm=&st., end_tm=&et., proc_nm='machcnt');
            %put machcnt macro End Time - %sysfunc(datetime(), datetime20);
        %end;

%mend;
