/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/* NAME: madtsql.sas                                                                     */
/* VERSION: 5.4                                                                        */
/* DESCRIPTION: This script populates the Dynamic Treatments records to SQL Server tables*/
/*                                                                                       */
/* Parameters: Macro variables                                                           */
/*    Macro variables:                                                                   */
/*        dbconnect - Explicit Pass-through connect statement                            */
/*        targetschema - Target DB location                                              */
/*        templibname - Libref points to the temporary DB library                        */
/*        tempschema - Temporary location to upload the input tables                     */
/*                                                                                       */
/*                                                                                       */
/* PRODUCT: MA                                                                           */
/* USAGE: %madtsql                                                                       */
/*                                                                                       */
/*****************************************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/* DATE       Defect No  Comments                                                        */
/* 16Jan2009  S0504678   First version of the code                                       */
/*                                                                                       */
/* 03Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 06May2010  S0647068   Change the insert logic to prevent remote constraint error      */
/*                                                                                       */
/* 06May2010  CI5.4      Populate CI_Package_X_Treatment for offer arbitration campaign  */
/*                                                                                       */
/* 09Dec2011 S0791301    create/refer all the table/col names in upper case              */
/*****************************************************************************************/

%macro madtsql(dbconnect=, targetschema=, templibname=, tempschema=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MADTSQL Current Version of the code: 541MA02;
    %put Last Modified on 09Dec2011;
    %put MADTSQL macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Created Process DTTM variable */
data _null_;
    call symput('pdttm',put(datepart(datetime()),yymmdd10.)||' '||put(timepart(datetime()),tod8.));
run;

/* Insert the new records to the respective Dynamic Treatment tables */
proc sql noerrorstop;

&dbconnect;;

    /* Exit the process if the pass-through connect statement fails */
    %if &SQLXRC. ne 0 %then
    %do;
        %let MAError=753;
        %let SYSCC=100000;
        %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
        %return;
    %end;

    /* S0701120 arbitration flag check is removed */
    %if &_g_cpxt_tbl_cnt. > 0 %then
    %do;
        /* Step 2: Populate CI_PACKAGE_X_TREATMENT with Package_hash_val */
        /* This table has to be populated for both Static and dynamic treatments so always expect to have some records in it */

        /* 2a): Create Index */
        %if %index(%upcase(&dbtmplib.),DBMSTEMP=Y) < 1 %then
        %do;
            %if &MADebug. = Y %then
            %do;
                %put Start Time for CI_PACKAGE_X_TREATMENT Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;

            /* Tempschema is assigned with 2 dots in %madtbl macro incase it is null/empty. There use only
            single dot for tempschema the macro varibale resolution which will take care of constructing the table name
            as either schema.tablename or tablename (incase tempschema is empty) */
            execute(create unique index I&_g_cpxt_unq.
                     on &tempschema.&_g_cpxt_unq.
                     (
                      PACKAGE_SK,
                      PACKAGE_HASH_VAL,
                      TREATMENT_SK,
                      CONTRIBUTING_CELL_PACKAGE_SK,
                      SEQUENCE_NO
                     )
                    ) by OLEDB;


            /* Exit if there is any error */
            /* SQLXRC gets set to 0 every time a commit transaction executed.  So to capture the correct
               error code and return the process, do the check immediately after a transaction finished
               execution */
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=755;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error creating Index on input table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            %if &MADebug. = Y %then
            %do;
                %put End Time for CI_PACKAGE_X_TREATMENT Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;

        %end; /* End of Index Creation */

        /* 2b): Insert nonexisting records into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_PACKAGE_X_TREATMENT Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (insert into &targetschema..&_g_cpxt_tbl.
                       ( PACKAGE_SK,
                         PACKAGE_HASH_VAL,
                         TREATMENT_SK,
                         CONTRIBUTING_CELL_PACKAGE_SK,
                         SEQUENCE_NO
                        )
                 select
                         PACKAGE_SK,
                         PACKAGE_HASH_VAL,
                         TREATMENT_SK,
                         CONTRIBUTING_CELL_PACKAGE_SK,
                         SEQUENCE_NO
                 from    &tempschema.&_g_cpxt_unq. st
                 where not exists
                      (
                        select 1
                        from   &targetschema..&_g_cpxt_tbl. tt
                        where  tt.PACKAGE_SK                   = st.PACKAGE_SK
                        and    tt.PACKAGE_HASH_VAL             = st.PACKAGE_HASH_VAL
                        and    tt.TREATMENT_SK                 = st.TREATMENT_SK
                        and    tt.CONTRIBUTING_CELL_PACKAGE_SK = st.CONTRIBUTING_CELL_PACKAGE_SK
                        and    tt.SEQUENCE_NO                  = st.SEQUENCE_NO
                      )
                )by OLEDB;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_cpxt_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /* Drop the temp table */
        execute ( drop table &tempschema.&_g_cpxt_unq. )by OLEDB;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_PACKAGE_X_TREATMENT Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of Step 2 */

    /* Step 3: Populate CI_DYNAMIC_TREATMENT_ATTRIBUTE table */
    %if &_g_dt_tbl_cnt. > 0 %then
    %do;
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Index Creation: %sysfunc(datetime(), datetime24.2);
        %end;

        /* 3a): Create index */

        /* S0546929 Treatment_SK is changed to Treatment_Hash_Val to allow duplicate treatment_sk */
        %if %index(%upcase(&dbtmplib.),DBMSTEMP=Y) < 1 %then
        %do;
            execute(create unique index I&_g_da_unq
                     on &tempschema.&_g_da_unq.
                     (
                      CELL_PACKAGE_SK,
                      TREATMENT_HASH_VAL,
                      PACKAGE_HASH_VAL
                     )
                    ) by OLEDB;

            /* Exit if there is any error */
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=755;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error creating Index on input table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            %if &MADebug. = Y %then
            %do;
                %put End Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;

        %end;

        /* 3b): Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (insert into &targetschema..&_g_dt_tbl.
                     ( CELL_PACKAGE_SK,
                       TREATMENT_SK,
                       PACKAGE_HASH_VAL,
                       TREATMENT_HASH_VAL
                      )
                 select
                       CELL_PACKAGE_SK,
                       TREATMENT_SK,
                       PACKAGE_HASH_VAL,
                       TREATMENT_HASH_VAL
                 from &tempschema.&_g_da_unq. st
                 where not exists
                 ( select 1
                   from   &targetschema..&_g_dt_tbl. tt
                   where  tt.CELL_PACKAGE_SK    = st.CELL_PACKAGE_SK
                   and    tt.TREATMENT_SK       = st.TREATMENT_SK
                   and    tt.PACKAGE_HASH_VAL   = st.PACKAGE_HASH_VAL
                   and    tt.TREATMENT_HASH_VAL = st.TREATMENT_HASH_VAL
                   )
                )by OLEDB;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_dt_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /* Drop the temp table */
        execute ( drop table &tempschema.&_g_da_unq. )by OLEDB;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_DYNAMIC_TREATMENT_ATTRIBUTE Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of Step 3 */

    /* Step 4: Populate the CI_TREATMENT_CHAR_UDF table after removing the records already exists */
    %if &_g_char_udf_cnt. > 0 %then
    %do;
        /* 4a): Create index */
        %if %index(%upcase(&dbtmplib.),DBMSTEMP=Y) < 1 %then
        %do;
            %if &MADebug. = Y %then
            %do;
                %put Start Time for CI_TREATMENT_CHAR_UDF Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;

            execute(create unique index I&_g_cu_unq
                     on &tempschema.&_g_cu_unq.
                     (
                      TREATMENT_SK,
                      CHAR_UDF_NM,
                      TREATMENT_HASH_VAL
                     )
                    ) by OLEDB;

            /* Exit if there is any error */
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=755;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error creating Index on input table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            %if &MADebug. = Y %then
            %do;
                %put End Time for CI_TREATMENT_CHAR_UDF Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;
        %end; /* End of Index creation */

        /* 4b): Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_TREATMENT_CHAR_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (insert into &targetschema..&_g_char_udf_tbl.
                     ( TREATMENT_SK,
                       CHAR_UDF_NM,
                       TREATMENT_HASH_VAL,
                       CHAR_UDF_VAL,
                       CHAR_EXT_COLUMN_NM,
                       PROCESSED_DTTM
                      )
                 select
                       TREATMENT_SK,
                       CHAR_UDF_NM,
                       TREATMENT_HASH_VAL,
                       CHAR_UDF_VAL,
                       CHAR_EXT_COLUMN_NM,
                       CONVERT(DATETIME,%str(%')&pdttm.%str(%'),121)
                 from &tempschema.&_g_cu_unq. st
                 where not exists
                      (
                        select 1
                        from   &targetschema..&_g_char_udf_tbl. tt
                        where  tt.TREATMENT_SK       = st.TREATMENT_SK
                        and    tt.TREATMENT_HASH_VAL = st.TREATMENT_HASH_VAL
                        and    tt.CHAR_UDF_NM        = st.CHAR_UDF_NM
                      )
                )by OLEDB;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_char_udf_tbl.].  SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /* Drop the temp table */
        execute ( drop table &tempschema.&_g_cu_unq. )by OLEDB;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_TREATMENT_CHAR_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of Step 4 */

    /* Step 5: Populate the CI_TREATMENT_NUM_UDF table after removing the records already exists */
    %if &_g_num_udf_cnt. > 0 %then
    %do;
        /* 5a): Create index */
        %if %index(%upcase(&dbtmplib.),DBMSTEMP=Y) < 1 %then
        %do;
            %if &MADebug. = Y %then
            %do;
                %put Start Time for CI_TREATMENT_NUM_UDF Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;

            execute(create unique index I&_g_nu_unq
                     on &tempschema.&_g_nu_unq.
                     (
                      TREATMENT_SK,
                      NUM_UDF_NM,
                      TREATMENT_HASH_VAL
                     )
                    ) by OLEDB;

            /* Exit if there is any error */
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=755;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error creating Index on input table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            %if &MADebug. = Y %then
            %do;
                %put End Time for CI_TREATMENT_NUM_UDF Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;
        %end; /* End of DBMSTEMP=Y loop */

        /* 5b): Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_TREATMENT_NUM_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (insert into &targetschema..&_g_num_udf_tbl.
                     ( TREATMENT_SK,
                       NUM_UDF_NM,
                       TREATMENT_HASH_VAL,
                       NUM_UDF_VAL,
                       NUM_EXT_COLUMN_NM,
                       PROCESSED_DTTM
                      )
                 select
                       TREATMENT_SK,
                       NUM_UDF_NM,
                       TREATMENT_HASH_VAL,
                       NUM_UDF_VAL,
                       NUM_EXT_COLUMN_NM,
                       CONVERT(DATETIME,%str(%')&pdttm.%str(%'),121)
                 from &tempschema.&_g_nu_unq. st
                 where not exists
                      (
                        select 1
                        from   &targetschema..&_g_num_udf_tbl. tt
                        where  tt.TREATMENT_SK       = st.TREATMENT_SK
                        and    tt.TREATMENT_HASH_VAL = st.TREATMENT_HASH_VAL
                        and    tt.NUM_UDF_NM         = st.NUM_UDF_NM
                      )
                )by OLEDB;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_num_udf_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /* Drop the temp table */
        execute ( drop table &tempschema.&_g_nu_unq. )by OLEDB;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_TREATMENT_NUM_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of Step 5 */

    /* Step 6: Populate the CI_TREATMENT_DATE_UDF table after removing the records already exists */
    %if &_g_dt_udf_cnt. > 0 %then
    %do;
        /* 6a): Create index */
        %if %index(%upcase(&dbtmplib.),DBMSTEMP=Y) < 1 %then
        %do;
            %if &MADebug. = Y %then
            %do;
                %put Start Time for CI_TREATMENT_DATE_UDF Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;

            execute(create unique index I&_g_du_unq
                     on &tempschema.&_g_du_unq.
                     (
                      TREATMENT_SK,
                      DATE_UDF_NM,
                      TREATMENT_HASH_VAL
                     )
                    ) by OLEDB;

            /* Exit if there is any error */
            %if &SQLXRC. ne 0 %then
            %do;
                %let MAError=755;
                %let SYSCC=100000;
                %let MAMsg=Macro[&sysmacroname.] Error creating Index on input table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
                %return;
            %end;

            %if &MADebug. = Y %then
            %do;
                %put End Time for CI_TREATMENT_DATE_UDF Index Creation: %sysfunc(datetime(), datetime24.2);
            %end;
        %end; /* End of DBMSTEMP=Y loop */

        /* 6b): Insert into target table */
        %if &MADebug. = Y %then
        %do;
            %put Start Time for CI_TREATMENT_DATE_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;

        execute (insert into &targetschema..&_g_dt_udf_tbl.
                     ( TREATMENT_SK,
                       DATE_UDF_NM,
                       TREATMENT_HASH_VAL,
                       DATE_EXT_COLUMN_NM,
                       DATE_UDF_VAL,
                       PROCESSED_DTTM
                      )
                 select
                       TREATMENT_SK,
                       DATE_UDF_NM,
                       TREATMENT_HASH_VAL,
                       DATE_EXT_COLUMN_NM,
                       CONVERT(DATETIME,DATE_UDF_VAL,121),
                       CONVERT(DATETIME,%str(%')&pdttm.%str(%'),121)
                 from &tempschema.&_g_du_unq. st
                 where not exists
                    ( select 1
                      from &targetschema..&_g_dt_udf_tbl. tt
                      where  tt.TREATMENT_SK       = st.TREATMENT_SK
                      and    tt.TREATMENT_HASH_VAL = st.TREATMENT_HASH_VAL
                      and    tt.DATE_UDF_NM        = st.DATE_UDF_NM
                    )
                )by OLEDB;

        /* Exit if there is any error */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=783;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error inserting Treatment records to [&_g_dt_udf_tbl.]. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;

        /* Drop the temp table */
        execute ( drop table &tempschema.&_g_du_unq. )by OLEDB;

        %if &MADebug. = Y %then
        %do;
            %put End Time for CI_TREATMENT_DATE_UDF Insert process: %sysfunc(datetime(), datetime24.2);
        %end;
    %end; /* End of CI_TREATMENT_DATE_UDF table processing */

disconnect from OLEDB;

quit;

%if &MADebug. eq Y %then
%do;
    %put MADTSQL macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
