/****************************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME:maptcnt.sas                                                         */
/* VERSION:5.41                                                             */
/* DESCRIPTION: Common Data Interface macro to compute Marketing Contact    */
/* Counts                                                                   */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %maptcnt                                                          */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                          Value           */
/* ch_tbl     ci_contact_history table name                                 */
/* cp_tbl     ci_cell_package table name                                    */
/* cpt_tbl    ci_cell_package_treatment_cnt table name                      */
/* data_tbl   Input data table                                              */
/* dbengine   Name of the database Engine,Update tables Statments are       */
/*            Engine Dependent                                              */
/* libnm      Library Name for CI Common Data Model                         */
/* newid      Unique new id for creating data processing tables in templib  */
/* PerfFlag   D or T,to decide whether to generate summary by looking at    */
/*            history or just work with the input data set                  */
/*            D- Default, T-True Counts                                     */
/* pth_tbl    ci_presented_treatment_history table name                     */
/* sub_key_sp Space Separated Subject Key Column  List                      */
/* tmplibnm   Temporary Library Name for data processing                    */
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 04Jun09 S0583077  New Version                                            */
/* 24Mar10 S0651614  Aster Partition Key Change                             */
/* 25Jun10 S0594364  Use a common macro for getting the row count of tables */
/* 09Dec11 S0791301  create/refer all the table/col names in upper case     */
/* 15Jun12 S0782605  use proc sql to get counts instead of proc summary     */
/****************************************************************************/
%macro maptcnt(ch_tbl=,cp_tbl=,cpt_tbl=,data_tbl=,dbengine=,libnm=,newid=,
                perfflag=,pth_tbl=,sub_key_sp=,tmplibnm=,);

    /* maptcnt Steps                                                */
    /* Step1- Initialisation                                        */
    /* Step2- Generate intermediate history tables to form bucket   */
    /* Step3- Generate Summary tables                               */
    /* Step4- Update Target tables                                  */
    /* Step5- Drop Work tables                                      */

    %if  &MAError.  ne %str() %then  %return;

    %if &MADebug. eq Y %then
    %do;
        %put MAPTCNT - Current Version of the code: 541MA03;
        %put Last Modified on 15Jun2012;
        %put MAPTCNT macro Start Time - %sysfunc(datetime(), datetime24.2);
    %end;

    /* Verify input parameters */
    %let dbengine=%upcase(&dbengine.);
    %let perfflag=%upcase(&perfflag.);
    %if &perfflag. ne D and &perfflag. ne T %then
    %do;
        %put Performance flag value [&perfflag.] is not supported; %return;
    %end;

/* Step1 Initialisation                                              */

    /* set options DBIDIRECTEXEC & SYS_SQL_IP_SPEEDO to enable sas directly send the proc sql statements to DBMS*/
    /* SPEEDO = YES not supported for following db's*/
    %if &dbengine. ne OLEDB  and &dbengine. ne ODBC %then
    %do;
        options DBIDIRECTEXEC;
        %let SYS_SQL_IP_SPEEDO=YES;
        option dsnferr;
    %end;

    %if &MADebug = Y %then
    %do;
        %put maptcnt Execution Start Time:%sysfunc(datetime(),datetime20);
        %let st=%sysfunc(datetime()) ;
        %put Parameters passed to maptcnt;
        %put ci_contact_history table name: &ch_tbl;
        %put ci_cell_package table name: &cp_tbl;
        %put ci_cell_package_treatment_cnt table name: &cpt_tbl;
        %put Input Data table name: &data_tbl;
        %put Library for CI Common Data Model: &libnm;
        %put NewId a unique number for creating intermediate data processing table: &newid;
        %put ci_presented_treatment_history table name: &pth_tbl;
        %put Subject Key: &sub_key_sp;
        %put Temporary Library for CI Common Data Model: &tmplibnm;
    %end;
    /*Create a variable to hold temp table names to drop at the end of process*/
    %let TblNames=%sysfunc(strip(&data_tbl.)) ;
    /* Remove the leading & trailing spaces if any in the sub key */
    %let sub_key_sp= %sysfunc(strip(&sub_key_sp.));
    /* Convert the space separated db keys in to comma separated db keys */
    %let sub_key_cs=%sysfunc(translate(&sub_key_sp.,',',' '));
    /* Extract the Subject Key Columns and create new subject variables */
    /* e.g. &sub_key_sp = CLIENT_ID ACC_NO convert this  subject variables as dbSubCol1 = CLIENT_ID & dbsubCol2 = ACC_NO */
    %let i = 1;
    %let dbSubCol=%scan(&sub_key_sp,&i,%str( ));
    %do %while( &dbSubCol ne );
        %let dbSubCol&i=&dbsubCol;
        %let i = %eval(&i+1);
        %let subCnt=&i;
        %let dbSubCol=%scan(&sub_key_sp,&i,%str( ));
    %end;

/* Step2 Generate Summary Tables                                    */
    /* S0651614 */
    %if &dbengine=ASTER %then
    %do;
        %maptast(ch_tbl=&ch_tbl ,cp_tbl=&cp_tbl ,cpt_tbl=&cpt_tbl ,data_tbl=&data_tbl ,dbengine=&dbengine ,libnm=&libnm ,newid=&newid ,
                perfflag=&perfflag ,pth_tbl=&pth_tbl ,sub_key_sp=&sub_key_sp ,tmplibnm=&tmplibnm );
        %if  &MAError. ne %str() %then %goto exit;
    %end;
    %else
    %do;
        /*if PerfFlag = T then create summary by joining the history tables*/
        %if &perfflag. eq T %then
        %do;
            /* Step2a Create a worskset with distinct cell_package_sk & subject_id's from the input set */
            %if &MADebug = Y %then  %put Step2a Start Time:%sysfunc(datetime(),datetime20);
            /*Append the tmp table to TblNames variable*/
            %let TblNames=&TblNames. %sysfunc(strip(PT_DCPK&newid.));
            proc sql;
                create table &tmplibnm..PT_DCPK&newid. as
                    select distinct CELL_PACKAGE_SK,&sub_key_cs from &tmplibnm..&data_tbl.
            ;quit;
            %if &MADebug = Y %then  %put Step2a End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2b: Get presented treatment history records for input cell_package and subject_id */
            /* add column counted_flag = Y for all records */
            %if &MADebug = Y %then  %put Step2b Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(PT_HIST&newid.));

            proc sql;
                create table    &tmplibnm..PT_HIST&newid. as
                    select      pth.CELL_PACKAGE_SK ,
                                pth.TREATMENT_SK ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    pth.&&dbsubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                pth.PRESENTED_TREATMENT_HIST_DTTM,
                                'Y' as COUNTED_FLG
                    from        &tmplibnm..PT_DCPK&newid. t
                    inner join  &libnm..&pth_tbl. pth
                    on          t.CELL_PACKAGE_SK = pth.CELL_PACKAGE_SK
                    %let i = 1;
                    %do %while (&i. ne &subCnt);
                        and           t.&&dbSubCol&i=pth.&&dbSubCol&i
                        %let i = %eval(&i+1);
                    %end;
                    where not exists (
                                select  1 from    &tmplibnm..&data_tbl. i
                                where   i.CELL_PACKAGE_SK=pth.CELL_PACKAGE_SK
                                and     i.TREATMENT_SK=pth.TREATMENT_SK
                                %let i = 1;
                                %do %while (&i. ne &subCnt);
                                and     i.&&dbSubCol&i=pth.&&dbSubCol&i
                                    %let i = %eval(&i+1);
                                %end;
                                and     i.PRESENTED_TREATMENT_HIST_DTTM =pth.PRESENTED_TREATMENT_HIST_DTTM )
                    union all
                    select      CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , PRESENTED_TREATMENT_HIST_DTTM ,
                                'N' as COUNTED_FLG
                    from        &tmplibnm..&data_tbl.

            ;quit;

            %if &MADebug = Y %then  %put Step2b End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

             /* Step2d: Get latest contact date time */
            %if &MADebug = Y %then  %put Step2d Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(PT_CH_HIST&newid.));
            proc sql;
                create table    &tmplibnm..PT_CH_HIST&newid. as
                    select      ipt.CELL_PACKAGE_SK ,
                                ipt.TREATMENT_SK ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    ipt.&&dbsubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                ipt.PRESENTED_TREATMENT_HIST_DTTM,
                                ipt.COUNTED_FLG,
                                max(ch.CONTACT_DTTM) as CH_DTTM
                    from        &tmplibnm..PT_HIST&newid. ipt
                    inner join  &libnm..&ch_tbl ch
                    on          ipt.CELL_PACKAGE_SK = ch.CELL_PACKAGE_SK
                    %let i = 1;
                    %do %while (&i. ne &subCnt);
                        and           ipt.&&dbSubCol&i=ch.&&dbSubCol&i
                        %let i = %eval(&i+1);
                    %end;
                    and         ch.CONTACT_DTTM <= ipt.PRESENTED_TREATMENT_HIST_DTTM
                    group by    ipt.CELL_PACKAGE_SK,ipt.TREATMENT_SK,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    ipt.&&dbsubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                ipt.PRESENTED_TREATMENT_HIST_DTTM,ipt.COUNTED_FLG
            ;quit;
            %if &MADebug = Y %then  %put Step2d End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2e: Separate counted & uncounted records and create two copies of uncounted one for cell_package level & other for treatment level */
            %if &MADebug = Y %then  %put Step2e Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(PT_CP_UNCOUNTED&newid.));
            proc sql;
                Create table    &tmplibnm..PT_CP_UNCOUNTED&newid. as
                    select      t1.* from &tmplibnm..PT_CH_HIST&newid. t1
                    where       t1.COUNTED_FLG = 'N' and
                    not exists (select  1  from  &tmplibnm..PT_CH_HIST&newid. t2
                                where   t2.CELL_PACKAGE_SK=t1.CELL_PACKAGE_SK
                                %let    i = 1;
                                %do %while (&i. ne &subCnt);
                                    and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                    %let i = %eval(&i+1);
                                %end;
                                and     t2.CH_DTTM=t1.CH_DTTM
                                and     t2.COUNTED_FLG = 'Y')
            ;quit;

            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            %let TblNames=&TblNames. %sysfunc(strip(PT_CPT_UNCOUNTED&newid.));
            proc sql;
                Create table    &tmplibnm..PT_CPT_UNCOUNTED&newid. as
                    select      t1.* from &tmplibnm..PT_CH_HIST&newid. t1
                    where       t1.COUNTED_FLG = 'N' and
                    not exists (  select  1  from &tmplibnm..PT_CH_HIST&newid. t2
                                where   t2.CELL_PACKAGE_SK=t1.CELL_PACKAGE_SK
                                and     t2.TREATMENT_SK=t1.TREATMENT_SK
                                %let i = 1;
                                %do %while (&i. ne &subCnt);
                                and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                    %let i = %eval(&i+1);
                                %end;
                                and     t2.CH_DTTM=t1.CH_DTTM
                                and     t2.COUNTED_FLG = 'Y')
            ;quit;

            %if &MADebug = Y %then  %put Step2e End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

    /* Step3- Generate Summary tables                           */

            /* Step3a: Generate Cell_Package level summary */
            /* Step3a1: Sort data for Cell_Package level summary */
            %if &MADebug = Y %then  %put Step3a1 Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(PT_CP_B4SUMMARY&newid.));
            /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
            proc sql;
                create table    &tmplibnm..PT_CP_B4SUMMARY&newid. as
                    select      CELL_PACKAGE_SK , &sub_key_cs. , CH_DTTM , count(*) as DUMMYCNT
                    from        &tmplibnm..PT_CP_UNCOUNTED&newid.
                    group by    CELL_PACKAGE_SK , &sub_key_cs. , CH_DTTM
            ;quit;
            %if &MADebug = Y %then  %put Step3a1 End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step3b: Generate Cell_Package treatment level summary */
            /* Step3b1: Sort data for Cell_Package treatment level summary */
            %if &MADebug = Y %then  %put Step3b1 Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(PT_CPT_B4SUMMARY&newid.));
            /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
            proc sql;
                create table    &tmplibnm..PT_CPT_B4SUMMARY&newid. as
                    select      CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , CH_DTTM , count(*) as DUMMYCNT
                    from        &tmplibnm..PT_CPT_UNCOUNTED&newid.
                    group by    CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , CH_DTTM
            ;quit;
            %if &MADebug = Y %then  %put Step3b1 End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        %end;/*End of PerfFlag= T*/

        /*if PerfFlag = D then create the summary just based on the input data table*/
        %if &perfflag. eq D %then
        %do;
            %let TblNames=&TblNames. %sysfunc(strip(PT_CP_B4SUMMARY&newid.));
            /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
            proc sql;
                create table    &tmplibnm..PT_CP_B4SUMMARY&newid. as
                    select      CELL_PACKAGE_SK , &sub_key_cs. ,count(*) as DUMMYCNT
                    from        &tmplibnm..&data_tbl.
                    group by    CELL_PACKAGE_SK , &sub_key_cs.
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            %let TblNames=&TblNames. %sysfunc(strip(PT_CPT_B4SUMMARY&newid.));
            /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
            proc sql;
                create table    &tmplibnm..PT_CPT_B4SUMMARY&newid. as
                    select      CELL_PACKAGE_SK, TREATMENT_SK ,&sub_key_cs.,count(*) as DUMMYCNT
                    from        &tmplibnm..&data_tbl.
                    group by    CELL_PACKAGE_SK, TREATMENT_SK ,&sub_key_cs.
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        %end;/*End of PerfFlag= D*/

        /* Step3a2: Summaries data for Cell_Package level summary */
        %if &MADebug = Y %then  %put Step3a2 Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(PT_CP_SUMMARY&newid.));
        %* S0782605 - use proc sql to get counts instead of proc summary;
        proc sql;
            create table    &tmplibnm..PT_CP_SUMMARY&newid. as
                select      CELL_PACKAGE_SK , count(*) as MARKETINGCONTACTCNT
                from        &tmplibnm..PT_CP_B4SUMMARY&newid.
                group by    CELL_PACKAGE_SK ;
        quit;

        %if &MADebug = Y %then  %put Step3a2 End Time:%sysfunc(datetime(),datetime20);

        /* Step3b2: Summaries data for Cell_Package treatment level summary */
        %if &MADebug = Y %then  %put Step3b2 Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(PT_CPT_SUMMARY&newid.));
        %* S0782605 -  use proc sql to get counts instead of proc summary  ;
        proc sql;
            create table &tmplibnm..PT_CPT_SUMMARY&newid. as
                select   CELL_PACKAGE_SK , TREATMENT_SK , count(*)  as MARKETINGCONTACTCNT
                from     &tmplibnm..PT_CPT_B4SUMMARY&newid.
                group by CELL_PACKAGE_SK , TREATMENT_SK ;
        quit;

        %if &MADebug = Y %then  %put Step3b2 End Time:%sysfunc(datetime(),datetime20);

    %end;
/* Step4- Update Target tables                                      */

    /* Step4a- Update TOTAL_MARKETING_CONTACT_CNT Cell_Pacakge table */
    /* Step4a1- Check if pt_cp_summary contains any record then only update the target table*/
    /* to avoid executing update statement in case there is nothing to update */
    /* S0594364 */
    %marowcnt( &tmplibnm..PT_CP_SUMMARY&newid. , cnt_cp_tmts);
    %if &cnt_cp_tmts. > 0 %then
    %do;
        %let pdttm=%sysfunc(datetime(),datetime20);
        %if &MADebug = Y %then  %put Step4a2 Start Time:%sysfunc(datetime(),datetime20);
        /* Step4a2- Update Cell_Package Target Table */
        /*The SQL_IP_SPEEDO can convert update statments for following database so use PROC SQL Update*/
        %if &dbengine.= ORACLE or  &dbengine.= DB2 or &dbengine.= TERADATA or &dbengine.= SASSPDS %then
        %do;
            proc sql;
                update &libnm..&cp_tbl.
                    set    TOTAL_MARKETING_CONTACT_CNT=
                                    (select TOTAL_MARKETING_CONTACT_CNT + MARKETINGCONTACTCNT
                                     from &tmplibnm..PT_CP_SUMMARY&newid. cps
                                     where &cp_tbl..CELL_PACKAGE_SK = cps.CELL_PACKAGE_SK),
                            PROCESSED_DTTM="&pdttm"dt
                    where  exists   (select 1 from &tmplibnm..PT_CP_SUMMARY&newid.  ptu
                                 where ptu.CELL_PACKAGE_SK =&cp_tbl..CELL_PACKAGE_SK)
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %end;
        /*Else use generic update using Data Modify*/
        %else
        %do;
            data &libnm..&cp_tbl.;
                modify &libnm..&cp_tbl. &tmplibnm..PT_CP_SUMMARY&newid.;
                by CELL_PACKAGE_SK;
                select (_IORC_);
                    when (%SYSRC(_SOK))
                    do;
                        TOTAL_MARKETING_CONTACT_CNT=SUM(TOTAL_MARKETING_CONTACT_CNT, MARKETINGCONTACTCNT);
                        PROCESSED_DTTM="&pdttm"dt;
                        replace;
                    end;
                    otherwise do;stop;end;
                end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
                %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
                %end;
            %end;
        %end;
        %if &MADebug = Y %then  %put Step4a2 End Time:%sysfunc(datetime(),datetime20);
    %end;


    /* Step4b- Update TOTAL_MARKETING_CONTACT_CNT Cell_Pacakge_treatment_cnt table */
    /* Step4b1: Check if pt_cpt_summary contains any record then only update the target table */
    /* to avoid executing update statement in case there is nothing to update */
    /* S0594364 */
    %marowcnt( &tmplibnm..PT_CPT_SUMMARY&newid. , cnt_cpt_tmts);
    %if &cnt_cpt_tmts. > 0 %then
    %do;
        /* Step4b2: Update Cell_Package_Treatment Target Table */
        %if &MADebug = Y %then  %put Step4b2 Start Time:%sysfunc(datetime(),datetime20);
        %let pdttm=%sysfunc(datetime(),datetime20);
        /*The SQL_IP_SPEEDO can convert update statments for following database so use PROC SQL Update*/
        %if &dbengine.= ORACLE or  &dbengine.= DB2 or &dbengine.= TERADATA or &dbengine.= SASSPDS %then
        %do;
            proc sql;
                update &libnm..&cpt_tbl.
                set    TOTAL_MARKETING_CONTACT_CNT=
                                (select TOTAL_MARKETING_CONTACT_CNT + MARKETINGCONTACTCNT from &tmplibnm..PT_CPT_SUMMARY&newid. cpts
                                 where &cpt_tbl..CELL_PACKAGE_SK = cpts.CELL_PACKAGE_SK
                                 and   &cpt_tbl..TREATMENT_SK = cpts.TREATMENT_SK),
                        PROCESSED_DTTM="&pdttm"dt
                where  exists   (select 1 from &tmplibnm..PT_CPT_SUMMARY&newid. ptu
                                 where ptu.CELL_PACKAGE_SK =&cpt_tbl..CELL_PACKAGE_SK
                                 and ptu.TREATMENT_SK= &cpt_tbl..TREATMENT_SK )
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %end;
        /*Else use generic update using Data Modify*/
        %else
        %do;
            data &libnm..&cpt_tbl.;
                modify &libnm..&cpt_tbl. &tmplibnm..PT_CPT_SUMMARY&newid.;
                by CELL_PACKAGE_SK TREATMENT_SK;
                select (_IORC_);
                    when (%SYSRC(_SOK))
                    do;
                        TOTAL_MARKETING_CONTACT_CNT=SUM(TOTAL_MARKETING_CONTACT_CNT, MARKETINGCONTACTCNT);
                        PROCESSED_DTTM="&pdttm"dt;
                        replace;
                    end;
                    otherwise do;stop;end;
                end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
                %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
                %end;
            %end;
        %end;
        %if &MADebug = Y %then  %put Step4b2 End Time:%sysfunc(datetime(),datetime20);
    %end;

/* Step5- Drop Work tables                                      */
%exit:
    /*if MAError is set then set the MAMsg*/
    %if  &MAError.  ne %str() %then
    %do;
        %if &MAError = 736 %then %let MAMsg = Macro[&sysmacroname.]. Error in Create table Statement SQLRC[&SQLRC]. Check the log for details;
        %if &MAError = 737 %then %let MAMsg = Macro[&sysmacroname.]. Error in Update table Statement SQLRC[&SQLRC]. Check the log for details;
        %if &MAError = 766 %then %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
    /*Delete the temporary tables*/
    %macdidel(libexist=Y, tblnms=&TblNames., templibname=&tmplibnm.);

    %if &MADebug = Y %then
    %do;
        %let et=%sysfunc(datetime());
        /*print the time taken for execution;*/
        %mautlpt(start_tm=&st., end_tm=&et., proc_nm='maptcnt');
        %put maptcnt Execution End Time:%sysfunc(datetime(),datetime20);
    %end;
%mend;
