/****************************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME:marhcnt.sas                                                         */
/* VERSION:5.41                                                             */
/* DESCRIPTION: Common Data Interface macro to compute response counts      */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %marhcnt                                                          */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                    Value                 */
/* ch_tbl     ci_contact_history table name                                 */
/* cp_tbl     ci_cell_package table name                                    */
/* cpt_tbl    ci_cell_package_treatment_cnt table name                      */
/* crxcp_tbl  ci_response_x_cell_package table name                         */
/* data_tbl   Input Data table                                              */
/* dbengine   Name of the database Engine,Update tables Statments are       */
/*            Engine Dependent                                              */
/* libnm      Library Name for CI Common Data Model                         */
/* newid      Unique new id for creating data processing tables in templib  */
/* PerfFlag   D or T,to decide whether to generate summary by looking at    */
/*            history or just work with the input data set                  */
/*            D- Default, T-True Counts                                     */
/* pth_tbl    ci_presented_treatment_history table name                     */
/* rh_tbl     ci_response_history table Name                                */
/* sub_key_sp Space Separated Subject Key Column  List                      */
/* s_rsp_cd   Successfull response Type Code                                */
/* tmplibnm   Temporary Library Name for data processing                    */
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 04Jun09 S0583077  New Version                                            */
/* 07Jul09 S0591527  Exit the count process if the input table is empty     */
/* 24Mar10 S0651614  Aster Partition Key Change                             */
/* 25Jun10 S0594364  Use a common macro for getting the row count of tables */
/* 09Dec11 S0791301  create/refer all the table/col names in upper case     */
/* 15Jun11 S0782605  use proc sql to get counts instead of proc summary     */
/****************************************************************************/
%macro marhcnt(ch_tbl=,cp_tbl=,cpt_tbl=,crxcp_tbl=,data_tbl=,dbengine=,libnm=,
        newid=,perfflag=,pth_tbl=,rh_tbl=,sub_key_sp=,s_rsp_cd=,tmplibnm=,);

    /* marhcnt Steps                                                */
    /* Step1- Initialisation                                        */
    /* Step2- Generate intermediate history tables to form bucket   */
    /*        for Cell Packages With Treatment Flag = Y             */
    /* Step3- Generate Summary tables for Cell Packages             */
    /*        With Treatment Flag = Y                               */
    /* Step4- Generate Summary tables for Cell Packages             */
    /*        With Treatment Flag = N                               */
    /* Step5- Generate Summary tables for Cell Packages             */
    /*        With Treatment Flag = N                               */
    /* Step6- Merge the Summary tables from Step3 & Step5           */
    /* Step7: Update Target tables                                  */
    /* Step8- Drop Work tables                                      */

    %if &MADebug. eq Y %then
    %do;
        %put MARHCNT - Current Version of the code: 541MA03;
        %put Last Modified on 15Jun2012;
        %put MARHCNT macro Start Time - %sysfunc(datetime(), datetime24.2);
    %end;

    %if  &MAError.  ne %str() %then  %return;

    /* S0591527 : Exit if the input table is empty */
    /* S0594364 */
    %marowcnt( &tmplibnm..&data_tbl. , inptbl_cnt);
    %if &inptbl_cnt. = 0 %then
    %do;
        %put No records to process for Business Context;
        %return;
    %end;

    /* Verify input parameters */
    %let dbengine=%upcase(&dbengine.);
    %let perfflag=%upcase(&perfflag.);
    %if &perfflag. ne D and &perfflag. ne T %then
    %do;
        %put Performance flag value [&perfflag.] is not supported; %return;
    %end;

/*  Step1 Initialisation                                             */

    /* set options DBIDIRECTEXEC & SYS_SQL_IP_SPEEDO to enable sas directly send the proc sql statements to DBMS*/
    /* SPEEDO = YES not supported for following db's*/
    %if &dbengine. ne OLEDB  and &dbengine. ne ODBC %then
    %do;
        options DBIDIRECTEXEC;
        %let SYS_SQL_IP_SPEEDO=YES;
        option dsnferr;
    %end;

    %if &MADebug = Y %then
    %do;
        %let st=%sysfunc(datetime()) ;
        %put Parameters passed to marhcnt;
        %put ci_contact_history table name: &ch_tbl;
        %put ci_cell_package table name: &cp_tbl;
        %put ci_cell_package_treatment_cnt table name: &cpt_tbl;
        %put ci_response_x_cell_package table name : &crxcp_tbl;
        %put Input Data table name: &data_tbl;
        %put Library for CI Common Data Model: &libnm;
        %put NewId a unique number for creating intermediate data processing table: &newid;
        %put Performance Flag: &perfflag;
        %put ci_presented_treatment_history table name: &pth_tbl;
        %put ci_response_history table Name : &rh_tbl;
        %put Subject Key: &sub_key_sp;
        %put Successful Response Type Cd: &s_rsp_cd;
        %put Temporary Library for CI Common Data Model: &tmplibnm;
    %end;

    /*Create a variable to hold temp table names to drop at the end of process*/
    %let TblNames=%sysfunc(strip(&data_tbl.)) ;
    /* Remove the leading & trailing spaces if any in the sub key */
    %let sub_key_sp= %sysfunc(strip(&sub_key_sp.));
    /* Convert the space separated db keys to comma separated db keys */
    %let sub_key_cs=%sysfunc(translate(&sub_key_sp.,',',' '));
    /* Extract the Subject Key Columns and create new subject variables */
    /* e.g. &sub_key_sp = CLIENT_ID ACC_NO convert this  subject variables as dbsubCol1 = CLIENT_ID & dbsubCol2 = ACC_NO */
    %let i = 1;
    %let dbSubCol=%scan(&sub_key_sp,&i,%str( ));
    %do %while( &dbSubCol ne );
        %let dbSubCol&i=&dbSubCol;
        %let i = %eval(&i+1);
        %let subCnt=&i;
        %let dbSubCol=%scan(&sub_key_sp,&i,%str( ));
    %end;

/* Step2 Generate intermediate history tables to form bucket for Cell Packages With Treatment Flag = Y */
    /* S0651614 */
    %if &dbengine=ASTER %then
    %do;
        %marhast(ch_tbl=&ch_tbl ,cp_tbl=&ch_tbl ,cpt_tbl=&cpt_tbl ,crxcp_tbl=&crxcp_tbl ,data_tbl=&data_tbl, dbengine=&dbengine ,libnm=&libnm,
        newid=&newid ,perfflag=&perfflag ,pth_tbl=&pth_tbl ,rh_tbl=&rh_tbl ,sub_key_sp=&sub_key_sp ,s_rsp_cd=&s_rsp_cd ,tmplibnm=&tmplibnm);
        %if  &MAError. ne %str() %then %goto exit;
    %end;
    %else
    %do;
        /* Create workset using input table to generate the summary    */
        /* if Perf flag = Y then it create two worksets one for tpt_flg= Y and other for tpt_flg= N  */
        /* if Perf flag = N then it will have one workset for all cell packages irrespective of tpt_flg */

        /*if PerfFlag = T (True Counts)then create summary by joining the history tables*/
        %if &perfflag. eq T %then
        %do;
            /* Step2a:Create WorkSet joining input with cell_package to get track_presented_treatments_flg */
            %if &MADebug = Y %then  %put Step1 Start Time:%sysfunc(datetime(),datetime20);
            /*Append the tmp table to TblNames variable*/
            %let TblNames=&TblNames. %sysfunc(strip(RHWORKSET&newid.));
            proc sql;
                create table    &tmplibnm..RHWORKSET&newid. as
                    select      t.CELL_PACKAGE_SK ,
                                t.TREATMENT_SK,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    t.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                t.RESPONSE_CD,
                                t.RESPONSE_DTTM,
                                cp.TRACK_PRESENTED_TREATMENTS_FLG as TPT_FLG
                    from        &tmplibnm..&data_tbl. t
                    inner join  &libnm..&cp_tbl. cp
                    on          t.CELL_PACKAGE_SK = cp.CELL_PACKAGE_SK
            ;quit;
            %if &MADebug = Y %then  %put Step1 End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /*Step2b Create separate copies of dataset for Treatment_flag Y and N*/
            %let TblNames=&TblNames. %sysfunc(strip(RHWORKY&newid.));
            proc sql;
                create table &tmplibnm..RHWORKY&newid. as
                    select * from &tmplibnm..RHWORKSET&newid. where TPT_FLG = 'Y';
            quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            %let TblNames=&TblNames. %sysfunc(strip(RHWORKN&newid.));
            proc sql;
                create table &tmplibnm..RHWORKN&newid. as
                    select * from &tmplibnm..RHWORKSET&newid. where TPT_FLG = 'N';
            quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2c: Check if there are records in rhWorkSet_y */
            /* S0594364 */
            %marowcnt( &tmplibnm..RHWORKY&newid. , cnt_TPT_FLG_y);
            %if &cnt_TPT_FLG_y. > 0 %then
            %do;
                /*Step2d Create a worskset with distinct cell_package_sk & subject_id's from rhWorkSet_y ;*/
                %let TblNames=&TblNames. %sysfunc(strip(RH_DICPSK&newid.));
                proc sql;
                    create table &tmplibnm..RH_DICPSK&newid. as
                        select distinct CELL_PACKAGE_SK , &sub_key_cs from &tmplibnm..RHWORKY&newid. ;
                quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step2e: Get the response history for the input_cell_package_sk having Tpt_flag= Y */
                /* add column counted_flag = Y for all records; */
                %if &MADebug = Y %then  %put Step 2e Start Time:%sysfunc(datetime(),datetime20);
                %let TblNames=&TblNames. %sysfunc(strip(RH_HISTORY&newid.));
                proc sql;
                    create table    &tmplibnm..RH_HISTORY&newid. as
                        select      rh.CELL_PACKAGE_SK ,
                                    rh.TREATMENT_SK ,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        rh.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    rh.RESPONSE_CD,
                                    rh.RESPONSE_DTTM,
                                    'Y' as COUNTED_FLG
                        from        &tmplibnm..RH_DICPSK&newid. rhw
                        inner join  &libnm..&rh_tbl. rh
                        on          rh.CELL_PACKAGE_SK =rhw.CELL_PACKAGE_SK
                        %let i = 1;
                        %do %while (&i. ne &subCnt);
                            and           rh.&&dbSubCol&i=rhw.&&dbSubCol&i
                            %let i = %eval(&i+1);
                        %end;
                        where not exists (  select  1
                                    from    &tmplibnm..RHWORKY&newid. i
                                    where   i.CELL_PACKAGE_SK=rh.CELL_PACKAGE_SK
                                    and     i.TREATMENT_SK=rh.TREATMENT_SK
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt);
                                    and     i.&&dbSubCol&i=rh.&&dbSubCol&i
                                        %let i = %eval(&i+1);
                                    %end;
                                    and     i.RESPONSE_DTTM =rh.RESPONSE_DTTM
                                    and     i.RESPONSE_CD = rh.RESPONSE_CD)
                        union all
                        select      CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. ,
                                    RESPONSE_CD , RESPONSE_DTTM, 'N' as COUNTED_FLG
                        from        &tmplibnm..RHWORKY&newid.
                ;quit;
                %if &MADebug = Y %then  %put Step 2e End Time:%sysfunc(datetime(),datetime20);
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step2g: Check if there are records in rh_history for Treatment_sk = -1 ;*/
                %let cnt_trt_sk_1=;
                proc sql noprint;
                    select count(*) into :cnt_trt_sk_1 from &tmplibnm..RH_HISTORY&newid. where TREATMENT_SK = -1
                ;quit;

                %if &cnt_trt_sk_1. > 0 %then
                %do;
                    /* Step2h: Convert (-1) Treatment_sk into corrosponding Treatment_sk for that cell_package ;*/
                    %if &MADebug = Y %then  %put Step2e Start Time:%sysfunc(datetime(),datetime20);
                    %let TblNames=&TblNames. %sysfunc(strip(RH_HISTORY_MAPTRTSK&newid.));
                    proc sql;
                        create table    &tmplibnm..RH_HISTORY_MAPTRTSK&newid. as
                            select      t.CELL_PACKAGE_SK ,
                                        cpt.TREATMENT_SK as TREATMENT_SK,
                                        %let i = 1;
                                        %do %while (&i. ne &subCnt.);
                                            t.&&dbSubCol&i ,
                                            %let i = %eval(&i+1);
                                        %end;
                                        t.RESPONSE_CD,
                                        t.RESPONSE_DTTM,
                                        COUNTED_FLG
                            from        &tmplibnm..RH_HISTORY&newid. t
                            inner join  &libnm..&cpt_tbl. cpt
                            on          cpt.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
                            and         t.TREATMENT_SK =-1
                    ;quit;
                    %if &MADebug = Y %then  %put Step2e End Time:%sysfunc(datetime(),datetime20);
                    %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                    /* Step2i: Convert (-1) Treatment_sk into corrosponding Treatment_sk for that cell_package ;*/
                    /*remove the treatment_sk = -1 records*/
                    proc sql;
                        delete from &tmplibnm..RH_HISTORY&newid. where TREATMENT_SK =-1;
                    quit;
                    %if &SQLRC ne 0  %then %do;%let MAError =738; %goto exit; %end;

                    /*add the mapped treatment records*/
                    proc sql;
                        insert into &tmplibnm..RH_HISTORY&newid.
                            select * from &tmplibnm..RH_HISTORY_MAPTRTSK&newid.
                    quit;
                    %if &SQLRC ne 0  %then %do;%let MAError =739; %goto exit; %end;

                %end;
                /* Step2j: Get Response Type Codes */
                %if &MADebug = Y %then  %put Step2j Start Time:%sysfunc(datetime(),datetime20);
                %let TblNames=&TblNames. %sysfunc(strip(RH_XR_HISTORY&newid.));
                proc sql;
                    create table    &tmplibnm..RH_XR_HISTORY&newid. as
                        select      rh.*,
                                    xr.RESPONSE_TYPE_CD
                        from        &tmplibnm..RH_HISTORY&newid. rh
                        inner join  &libnm..&crxcp_tbl. xr
                        on          xr.CELL_PACKAGE_SK = rh.CELL_PACKAGE_SK
                        and         xr.RESPONSE_CD = rh.RESPONSE_CD
                ;quit;
                %if &MADebug = Y %then  %put Step2j End Time:%sysfunc(datetime(),datetime20);
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step2k: Get latest Presented Treatment Date Time ;*/
                %if &MADebug = Y %then  %put Step2k Start Time:%sysfunc(datetime(),datetime20);
                %let TblNames=&TblNames. %sysfunc(strip(RH_XR_PT_HIST&newid.));
                proc sql;
                    create table    &tmplibnm..RH_XR_PT_HIST&newid. as
                        select      rx.CELL_PACKAGE_SK ,
                                    rx.TREATMENT_SK ,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        rx.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    rx.RESPONSE_CD ,
                                    rx.RESPONSE_TYPE_CD,
                                    rx.RESPONSE_DTTM ,
                                    rx.COUNTED_FLG,
                                    max(pt.PRESENTED_TREATMENT_HIST_DTTM) as PT_DTTM
                        from        &tmplibnm..RH_XR_HISTORY&newid. rx
                        inner join  &libnm..&pth_tbl. pt
                        on          pt.CELL_PACKAGE_SK = rx.CELL_PACKAGE_SK
                        and         pt.TREATMENT_SK=rx.TREATMENT_SK
                        %let i = 1;
                        %do %while (&i. ne &subCnt);
                            and           pt.&&dbSubCol&i=rx.&&dbSubCol&i
                            %let i = %eval(&i+1);
                        %end;
                        and         pt.PRESENTED_TREATMENT_HIST_DTTM  <= rx.RESPONSE_DTTM
                        group by    rx.CELL_PACKAGE_SK , rx.TREATMENT_SK ,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        rx.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    rx.RESPONSE_CD, rx.RESPONSE_TYPE_CD, rx.RESPONSE_DTTM, rx.COUNTED_FLG
                ;quit;
                %if &MADebug = Y %then  %put Step2k End Time:%sysfunc(datetime(),datetime20);
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step2l: Get latest Contact Date Time ;*/
                %if &MADebug = Y %then  %put Step2l Start Time:%sysfunc(datetime(),datetime20);
                %let TblNames=&TblNames. %sysfunc(strip(RH_XR_PT_CH_HIST&newid.));
                proc sql;
                    create table    &tmplibnm..RH_XR_PT_CH_HIST&newid. as
                        select      rxp.CELL_PACKAGE_SK ,
                                    rxp.TREATMENT_SK ,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        rxp.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    rxp.RESPONSE_CD,
                                    rxp.RESPONSE_TYPE_CD,
                                    rxp.RESPONSE_DTTM,
                                    rxp.PT_DTTM,
                                    rxp.COUNTED_FLG,
                                    max(ch.CONTACT_DTTM) as CH_DTTM
                        from        &tmplibnm..RH_XR_PT_HIST&newid. rxp
                        left join   &libnm..&ch_tbl. ch
                        on          ch.CELL_PACKAGE_SK=rxp.CELL_PACKAGE_SK
                        %let i = 1;
                        %do %while (&i. ne &subCnt);
                            and           ch.&&dbSubCol&i=rxp.&&dbSubCol&i
                            %let i = %eval(&i+1);
                        %end;
                        and         ch.CONTACT_DTTM <= rxp.PT_DTTM
                        group by    rxp.CELL_PACKAGE_SK , rxp.TREATMENT_SK ,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        rxp.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    rxp.RESPONSE_CD , rxp.RESPONSE_TYPE_CD , rxp.RESPONSE_DTTM , rxp.PT_DTTM , rxp.COUNTED_FLG
                ;quit;
                %if &MADebug = Y %then  %put Step2l End Time:%sysfunc(datetime(),datetime20);
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step2m: Separate counted & uncounted records and create two copies of uncounted one for cell_package level & other for treatment level;*/
                %if &MADebug = Y %then  %put Step2m Start Time:%sysfunc(datetime(),datetime20);
                /*data &tmplibnm..rh_counted&newid. &tmplibnm..rh_cp_uncounted&newid. &tmplibnm..rh_cpt_uncounted&newid. ; */
                %let TblNames=&TblNames. %sysfunc(strip(RH_CP_UNCOUNTED&newid.));
                proc sql;
                    create table    &tmplibnm..RH_CP_UNCOUNTED&newid. as
                        select      t1.* from &tmplibnm..RH_XR_PT_CH_HIST&newid. t1
                        where       COUNTED_FLG = 'N' and
                        not exists (  select  1
                                    from    &tmplibnm..RH_XR_PT_CH_HIST&newid. t2
                                    where   t2.CELL_PACKAGE_SK=t1.CELL_PACKAGE_SK
                                    %let    i = 1;
                                    %do %while (&i. ne &subCnt);
                                        and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                        %let i = %eval(&i+1);
                                    %end;
                                    and     t2.RESPONSE_CD=t1.RESPONSE_CD
                                    and     t2.CH_DTTM=t1.CH_DTTM
                                    and     t2.COUNTED_FLG = 'Y')
                ;quit;

                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                %let TblNames=&TblNames. %sysfunc(strip(RH_CPT_UNCOUNTED&newid.));
                proc sql;
                    create table    &tmplibnm..RH_CPT_UNCOUNTED&newid. as
                        select      t1.* from &tmplibnm..RH_XR_PT_CH_HIST&newid. t1
                        where       t1.COUNTED_FLG = 'N' and
                        not exists (  select  1
                                    from    &tmplibnm..RH_XR_PT_CH_HIST&newid. t2
                                    where   t2.CELL_PACKAGE_SK=t1.CELL_PACKAGE_SK
                                    and     t2.TREATMENT_SK = t1.TREATMENT_SK
                                    %let    i = 1;
                                    %do %while (&i. ne &subCnt);
                                        and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                        %let i = %eval(&i+1);
                                    %end;
                                    and     t2.RESPONSE_CD=t1.RESPONSE_CD
                                    and     t2.CH_DTTM=t1.CH_DTTM
                                    and     t2.COUNTED_FLG = 'Y')
                ;quit;

                %if &MADebug = Y %then  %put Step2m End Time:%sysfunc(datetime(),datetime20);
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

    /* Step3- Generate Summary tables for Cell Packages With Treatment Flag = Y */

                /* Step3a:  Generate cell Package level summary for cell_package_sk having tpt_flg = Y when PerfFlag = T*/

                /* Step3a1: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Cell  Package Level */
                %let TblNames=&TblNames. %sysfunc(strip(RH_CP_B4SUMMARY&newid.));

                /*Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition*/
                proc sql;
                    create table    &tmplibnm..RH_CP_B4SUMMARY&newid. as
                        select      CELL_PACKAGE_SK , &sub_key_cs. , RESPONSE_CD, RESPONSE_TYPE_CD , CH_DTTM , count(*) as DUMMYCNT
                        from        &tmplibnm..RH_CP_UNCOUNTED&newid.
                        group by    CELL_PACKAGE_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD , CH_DTTM
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step3a2: count TOTAL_RESPONSE_CNT  at cell package level */
                %let TblNames=&TblNames. %sysfunc(strip(CP_TOTAL_RESP&newid.));
                %* S0782605 - use proc sql to get counts instead of proc summary ;
                proc sql;
                    create table    &tmplibnm..CP_TOTAL_RESP&newid. as
                        select      CELL_PACKAGE_SK , count(*) as T_RESP_CNT
                        from        &tmplibnm..RH_CP_B4SUMMARY&newid.
                        group by    CELL_PACKAGE_SK ;
                quit;

                /* Step3a3: count TOTAL_SUCCESSFUL_RESPONSE_CNT at cell package level */
                %let TblNames=&TblNames. %sysfunc(strip(CP_SUCCESS_RESP&newid.));
                proc sql;
                    create table    &tmplibnm..CP_SUCCESS_RESP&newid.   as
                        select      CELL_PACKAGE_SK , count(*) as S_RESP_CNT
                        from        &tmplibnm..RH_CP_B4SUMMARY&newid.
                        where       RESPONSE_TYPE_CD = "&s_rsp_cd"
                        group by    CELL_PACKAGE_SK ;
                quit;

                /* Step3b: Generate Cell Package Treatment Level Summary for cell_package_sk having tpt_flg = Y when PerfFlag = T*/

                /* Step3b1: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment Level */
                %let TblNames=&TblNames. %sysfunc(strip(RH_CPT_B4SUMMARY&newid.));

                /*Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition*/
                proc sql;
                    create table    &tmplibnm..RH_CPT_B4SUMMARY&newid. as
                        select      CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD , CH_DTTM , count(*) as DUMMYCNT
                        from        &tmplibnm..RH_CPT_UNCOUNTED&newid.
                        group by    CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD , CH_DTTM
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step3b2: count TOTAL_RESPONSE_CNT  at Treatment level */
                %let TblNames=&TblNames. %sysfunc(strip(CPT_TOTAL_RESP&newid.));
                %* S0782605 - use proc sql to get counts instead of proc summary;
                proc sql;
                    create table    &tmplibnm..CPT_TOTAL_RESP&newid. as
                        select      CELL_PACKAGE_SK , TREATMENT_SK , count(*) as T_RESP_CNT
                        from        &tmplibnm..RH_CPT_B4SUMMARY&newid.
                        group by    CELL_PACKAGE_SK , TREATMENT_SK  ;
                quit;

                /* Step3b3: TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment level */
                %let TblNames=&TblNames. %sysfunc(strip(CPT_SUCCESS_RESP&newid.));
                proc sql;
                    create table    &tmplibnm..CPT_SUCCESS_RESP&newid. as
                        select      CELL_PACKAGE_SK , TREATMENT_SK , count(*) as S_RESP_CNT
                        from        &tmplibnm..RH_CPT_B4SUMMARY&newid.
                        where       RESPONSE_TYPE_CD = "&s_rsp_cd"
                        group by    CELL_PACKAGE_SK , TREATMENT_SK ;
                quit;

                /* Step3c: Merge the summary tables                 */

                /* Step3c1: Merge the cell package level summary    */
                %let TblNames=&TblNames. %sysfunc(strip(CP_RESP_COUNT&newid.));
                data &tmplibnm..CP_RESP_COUNT&newid.;
                    merge &tmplibnm..CP_TOTAL_RESP&newid.(in=tr) &tmplibnm..CP_SUCCESS_RESP&newid. (in=sr);
                    by CELL_PACKAGE_SK;
                run;

                /* Step3c2: Merge the treatment level summary tables */
                %let TblNames=&TblNames. %sysfunc(strip(CPT_RESP_COUNT&newid.));
                data &tmplibnm..CPT_RESP_COUNT&newid.;
                    merge &tmplibnm..CPT_TOTAL_RESP&newid.(in=tr) &tmplibnm..CPT_SUCCESS_RESP&newid. (in=sr);
                    by CELL_PACKAGE_SK TREATMENT_SK;
                run;
            %end;
        %end;

        %if &perfflag. eq D %then
        %do;
           /*Create work dataset from input dataset;*/
            %let TblNames=&TblNames. %sysfunc(strip(RHWORKN&newid.));
            proc sql;
                create table &tmplibnm..RHWORKN&newid. as
                    select  CELL_PACKAGE_SK,TREATMENT_SK, &sub_key_cs.,RESPONSE_CD,RESPONSE_DTTM,'N' as TPT_FLG
                    from        &tmplibnm..&data_tbl.
            quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
        %end;

    /* Step4- Generate Summary tables for Cell Packages With Treatment Flag = N (or when Perfflag=D)*/

        /* Step4a: Check if there are records in rhWorkSet_n */
        /* S0594364 */
        %marowcnt( &tmplibnm..RHWORKN&newid. , cnt_TPT_FLG_N);
        %if &cnt_TPT_FLG_N. > 0 %then
        %do;
            /* Step4b: Check if there are records in rhWorkSet_n with Treatment_sk = -1 */
            proc sql noprint;
                select count(*) into :cnt_trt_sk_1 from &tmplibnm..RHWORKN&newid. t where t.TREATMENT_SK =-1
            ;quit;

            %if &cnt_trt_sk_1. > 0 %then
            %do;
                /* Step4c: Convert the Treatment_SK -1 into actual treatments for that cell package ;*/
                %let TblNames=&TblNames. %sysfunc(strip(RHWORKSET_MAPTRTSK_N&newid.));
                proc sql;
                    create table    &tmplibnm..RHWORKSET_MAPTRTSK_N&newid. as
                        select      t.CELL_PACKAGE_SK ,
                                    cpt.TREATMENT_SK,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        t.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    t.RESPONSE_CD,
                                    t.RESPONSE_DTTM
                        from        &tmplibnm..RHWORKN&newid. t
                        inner join  &libnm..&cpt_tbl. cpt
                        on          cpt.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
                        where       t.TREATMENT_SK =-1
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
                /*Step4d Remove the treatment_sk = -1 records from rhWorkSet_n      */
                /*    and add mapped treatment_sk records from rhWorkSet_maptrtsk_N */
                proc sql;
                    delete from &tmplibnm..RHWORKN&newid. where TREATMENT_SK =-1;
                quit;
                %if &SQLRC ne 0  %then %do;%let MAError =738; %goto exit; %end;
                proc sql;
                    insert into &tmplibnm..RHWORKN&newid.
                        select  CELL_PACKAGE_SK ,
                                    TREATMENT_SK,
                                    &sub_key_cs.,
                                    RESPONSE_CD,
                                    RESPONSE_DTTM,
                                    'N' as TPT_FLG
                        from &tmplibnm..RHWORKSET_MAPTRTSK_N&newid.
                ;quit;
                %if &SQLRC ne 0  %then %do;%let MAError =739; %goto exit; %end;
            %end;

            /* Step4e: join the rhWorkSet_n table to Cell_package_X table to get Response_type_cd */
            %let TblNames=&TblNames. %sysfunc(strip(RH_PERF_YN_TPT_N&newid.));
            proc sql;
                create table    &tmplibnm..RH_PERF_YN_TPT_N&newid. as
                    select      t.*,
                                xr.RESPONSE_TYPE_CD
                    from        &tmplibnm..RHWORKN&newid. t
                    inner join  &libnm..&crxcp_tbl. xr
                    on          xr.CELL_PACKAGE_SK = t.CELL_PACKAGE_SK
                    and         xr.RESPONSE_CD = t.RESPONSE_CD
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

    /* Step5- Generate Summary tables for Cell Packages With Treatment Flag = N */

            /* Step5a: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Cell_Package Level */
            %let TblNames=&TblNames. %sysfunc(strip(RH_CP_B4SUMMARY_n&newid.));
            proc sql;
                create table    &tmplibnm..RH_CP_B4SUMMARY_n&newid. as
                    select      CELL_PACKAGE_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD, count(*) as DUMMYCNT
                    from        &tmplibnm..RH_PERF_YN_TPT_N&newid.
                    group by    CELL_PACKAGE_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step5b: TOTAL_RESPONSE_CNT  at cell package level */
            %let TblNames=&TblNames. %sysfunc(strip(CP_TOTAL_RESP_N&newid.));
            %* S0782605 - use proc sql to get counts instead of proc summary;
            proc sql;
                create table    &tmplibnm..CP_TOTAL_RESP_n&newid. as
                    select      CELL_PACKAGE_SK , count(*) as T_RESP_CNT
                    from        &tmplibnm..RH_CP_B4SUMMARY_n&newid.
                    group by    CELL_PACKAGE_SK ;
            quit;

            /* Step5c: TOTAL_SUCCESSFUL_RESPONSE_CNT at cell package level */
            %let TblNames=&TblNames. %sysfunc(strip(CP_SUCCESS_RESP_N&newid.));
            proc sql;
                create table    &tmplibnm..CP_SUCCESS_RESP_n&newid. as
                    select      CELL_PACKAGE_SK , count(*) as S_RESP_CNT
                    from        &tmplibnm..RH_CP_B4SUMMARY_n&newid.
                    where       RESPONSE_TYPE_CD = "&s_rsp_cd"
                    group by    CELL_PACKAGE_SK ;
            quit;

            /* Step5d: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment Level */
            %let TblNames=&TblNames. %sysfunc(strip(RH_CPT_B4SUMMARY_n&newid.));
            proc sql;
                create table &tmplibnm..RH_CPT_B4SUMMARY_n&newid. as
                    select      CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD , count(*) as DUMMYCNT
                    from        &tmplibnm..RH_PERF_YN_TPT_N&newid.
                    group by    CELL_PACKAGE_SK , TREATMENT_SK , &sub_key_cs. , RESPONSE_CD , RESPONSE_TYPE_CD
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step5e: TOTAL_RESPONSE_CNT  at Treatment level */
            %let TblNames=&TblNames. %sysfunc(strip(CPT_TOTAL_RESP_N&newid.));
            %* S0782605 - use proc sql to get counts instead of proc summary;
            proc sql;
                create table    &tmplibnm..CPT_TOTAL_RESP_n&newid. as
                    select      CELL_PACKAGE_SK , TREATMENT_SK , count(*) as T_RESP_CNT
                    from        &tmplibnm..RH_CPT_B4SUMMARY_N&newid.
                    group by    CELL_PACKAGE_SK , TREATMENT_SK ;
            quit;

            /* Step5f: TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment level */
            %let TblNames=&TblNames. %sysfunc(strip(CPT_SUCCESS_RESP_N&newid.));
            proc sql;
                create table    &tmplibnm..CPT_SUCCESS_RESP_n&newid. as
                    select      CELL_PACKAGE_SK , TREATMENT_SK , count(*) as S_RESP_CNT
                    from        &tmplibnm..RH_CPT_B4SUMMARY_n&newid.
                    where       RESPONSE_TYPE_CD = "&s_rsp_cd"
                    group by    CELL_PACKAGE_SK , TREATMENT_SK ;
            quit;

            /* Step5g: Merge Cell_Package Summary columns */
            /*option nodsnferr used to avoid error occurance in case one of the merging tables not present */
            option nodsnferr;
            %let TblNames=&TblNames. %sysfunc(strip(CP_RESP_COUNT_N&newid.));
            data &tmplibnm..CP_RESP_COUNT_N&newid.;
                merge &tmplibnm..CP_TOTAL_RESP_N&newid.(in=tr) &tmplibnm..CP_SUCCESS_RESP_N&newid. (in=sr);
                by CELL_PACKAGE_SK;
            run;

            /* Step5h: Merge Cell_Package_Treatment Summary columns */
            %let TblNames=&TblNames. %sysfunc(strip(CPT_RESP_COUNT_N&newid.));
            data &tmplibnm..CPT_RESP_COUNT_N&newid.;
                merge &tmplibnm..CPT_TOTAL_RESP_N&newid.(in=tr) &tmplibnm..CPT_SUCCESS_RESP_N&newid. (in=sr);
                by CELL_PACKAGE_SK;
            run;
        %end;

    /* Step6- Merge the Summary tables from Step3 & Step5           */

        /* Step6a: Merge Cell_Package Summary tables(summary of treatment_flag Y + summary of treatment_flag N) */
        /*option nodsnferr used to avoid error occurance in case one of the data tables not present */
        option nodsnferr;
        %let TblNames=&TblNames. %sysfunc(strip(CP_RESPONSES&newid.));
        data &tmplibnm..CP_RESPONSES&newid.;
            set &tmplibnm..CP_RESP_COUNT&newid. &tmplibnm..CP_RESP_COUNT_N&newid. ;
        run;

        /* Step6b: Merge Cell_Package_treatment Summary tables(summary of treatment_flag Y + summary of treatment_flag N) */
        %let TblNames=&TblNames. %sysfunc(strip(CPT_RESPONSES&newid.));
        data &tmplibnm..CPT_RESPONSES&newid.;
            set &tmplibnm..CPT_RESP_COUNT&newid. &tmplibnm..CPT_RESP_COUNT_N&newid.;
        run;
    %end; /* End of Else Aster block */

    /* Step6c:Update the Null summary columns with zero to avoid update by null in the Target table */
    /*Update only for following engines, as they are using proc sql update target table where NULLS must be set to 0*/
    %if &dbengine. = ORACLE or  &dbengine. = DB2 or &dbengine. = TERADATA %then
    %do;
        proc sql;
            update &tmplibnm..CP_RESPONSES&newid.  set T_RESP_CNT = 0 where T_RESP_CNT is null;
            update &tmplibnm..CP_RESPONSES&newid.  set S_RESP_CNT = 0 where S_RESP_CNT is null;
            update &tmplibnm..CPT_RESPONSES&newid. set T_RESP_CNT = 0 where T_RESP_CNT is null;
            update &tmplibnm..CPT_RESPONSES&newid. set S_RESP_CNT = 0 where S_RESP_CNT is null;
        quit;
        %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
    %end;

/* Step7: Update Target tables                                  */

    /* Step7a: Check if cp_responses contains any record then only update the target table */
    /* to avoid executing update statement in case there is nothing to update               */
    /* S0594364 */
    %marowcnt( &tmplibnm..CP_RESPONSES&newid. , cnt_cp_responses);
    option dsnferr;
    %if &cnt_cp_responses. > 0 %then
    %do;
        /*Update Total_Response_Cnt*/
        %let pdttm=%sysfunc(datetime(),datetime20);
        /* Step7b: Update Cell_Package target table */
        %if &MADebug = Y %then  %put Step7b Start Time:%sysfunc(datetime(),datetime20);
        /*The SQL_IP_SPEEDO can convert update statments for following database so use PROC SQL Update*/
        %if &dbengine. = ORACLE or  &dbengine. = DB2 or &dbengine. = TERADATA or &dbengine. = SASSPDS %then
        %do;
            proc sql;
                update  &libnm..&cp_tbl.
                    set TOTAL_RESPONSE_CNT =
                                        (select TOTAL_RESPONSE_CNT + T_RESP_CNT
                                         from &tmplibnm..CP_RESPONSES&newid. cpr
                                         where cpr.CELL_PACKAGE_SK = &cp_tbl..CELL_PACKAGE_SK),
                        PROCESSED_DTTM ="&pdttm"dt
                    where  exists   (select 1 from &tmplibnm..CP_RESPONSES&newid. cprc
                                     where cprc.CELL_PACKAGE_SK =&cp_tbl..CELL_PACKAGE_SK)
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            /*Update Total_Successful_Response_Cnt*/
            proc sql;
                update  &libnm..&cp_tbl.
                    set TOTAL_SUCCESSFUL_RESPONSE_CNT=
                                        (select TOTAL_SUCCESSFUL_RESPONSE_CNT + S_RESP_CNT
                                         from &tmplibnm..CP_RESPONSES&newid. cpr
                                         where cpr.CELL_PACKAGE_SK = &cp_tbl..CELL_PACKAGE_SK),
                        PROCESSED_DTTM ="&pdttm"dt
                    where  exists   (select 1 from &tmplibnm..CP_RESPONSES&newid. cprc
                                     where cprc.CELL_PACKAGE_SK =&cp_tbl..CELL_PACKAGE_SK)
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %end;
        /* Else use generic update using Data Modify */
        %else
        %do;
            data &libnm..&cp_tbl.;
                modify &libnm..&cp_tbl. &tmplibnm..CP_RESPONSES&newid.;
                by CELL_PACKAGE_SK ;
                select (_IORC_);
                    when (%SYSRC(_SOK))
                    do;
                        TOTAL_RESPONSE_CNT=SUM(TOTAL_RESPONSE_CNT, T_RESP_CNT);
                        TOTAL_SUCCESSFUL_RESPONSE_CNT=SUM(TOTAL_SUCCESSFUL_RESPONSE_CNT,S_RESP_CNT);
                        PROCESSED_DTTM="&pdttm"dt;
                        replace;
                    end;
                    otherwise do;stop;end;
                end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
                %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
                %end;
            %end;
        %end;
        %if &MADebug = Y %then  %put Step7b End Time:%sysfunc(datetime(),datetime20);
    %end;

    /* Step7c: Check if cpt_responses contains any record then only update the target table,to avoid executing update statement in case there is nothing to update */
    /* S0594364 */
    %marowcnt( &tmplibnm..CPT_RESPONSES&newid. , cnt_cpt_responses);
    %if &cnt_cpt_responses. > 0 %then
    %do;
        /* Update Total_Response_Cnt */
        %let pdttm=%sysfunc(datetime(),datetime20);
        /* Step7d: Update Cell_Package_Treatment target table */
        %if &MADebug = Y %then  %put Step7d Start Time:%sysfunc(datetime(),datetime20);
        /* The SQL_IP_SPEEDO can convert update statments for following database so use PROC SQL Update */
        %if &dbengine. = ORACLE or  &dbengine. = DB2 or &dbengine. = TERADATA or &dbengine. = SASSPDS %then
        %do;
            proc sql;
                update  &libnm..&cpt_tbl.
                    set TOTAL_RESPONSE_CNT=
                                        (select TOTAL_RESPONSE_CNT + T_RESP_CNT
                                         from &tmplibnm..CPT_RESPONSES&newid. cpr1
                                         where cpr1.CELL_PACKAGE_SK = &cpt_tbl..CELL_PACKAGE_SK and   cpr1.TREATMENT_SK= &cpt_tbl..TREATMENT_SK ),
                        PROCESSED_DTTM ="&pdttm"dt
                    where  exists   (select 1 from &tmplibnm..CPT_RESPONSES&newid. cprc
                                     where cprc.CELL_PACKAGE_SK =&cpt_tbl..CELL_PACKAGE_SK and cprc.TREATMENT_SK= &cpt_tbl..TREATMENT_SK )
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;

            /* Update Total_Successful_Respone_Cnt */
            proc sql;
                update  &libnm..&cpt_tbl.
                    set TOTAL_SUCCESSFUL_RESPONSE_CNT=
                                        (select TOTAL_SUCCESSFUL_RESPONSE_CNT + S_RESP_CNT
                                         from &tmplibnm..CPT_RESPONSES&newid. cpr2
                                         where cpr2.CELL_PACKAGE_SK = &cpt_tbl..CELL_PACKAGE_SK and   cpr2.TREATMENT_SK= &cpt_tbl..TREATMENT_SK ),
                        PROCESSED_DTTM ="&pdttm"dt
                    where  exists   (select 1 from &tmplibnm..CPT_RESPONSES&newid. cprc
                                     where cprc.CELL_PACKAGE_SK =&cpt_tbl..CELL_PACKAGE_SK and cprc.TREATMENT_SK= &cpt_tbl..TREATMENT_SK )
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =737; %goto exit; %end;
        %end;
        /* Else use generic update using Data Modify */
        %else
        %do;
            data &libnm..&cpt_tbl.;
                modify &libnm..&cpt_tbl. &tmplibnm..CPT_RESPONSES&newid.;
                by CELL_PACKAGE_SK TREATMENT_SK;
                select (_IORC_);
                    when (%SYSRC(_SOK))
                    do;
                        TOTAL_RESPONSE_CNT = SUM(TOTAL_RESPONSE_CNT, T_RESP_CNT);
                        TOTAL_SUCCESSFUL_RESPONSE_CNT = SUM(TOTAL_SUCCESSFUL_RESPONSE_CNT,S_RESP_CNT);
                        PROCESSED_DTTM="&pdttm"dt;
                        replace;
                    end;
                    otherwise do;stop;end;
                end;
            run;
            /* Exit the process if any error */
            %if %symexist(SYSDBRC) %then
            %do;%if &SYSDBRC. ^= 0 %then
                %do; %let SYSCC = 100000; %let MAError = 766; %goto exit;
                %end;
            %end;
        %end;
        %if &MADebug = Y %then  %put Step7d End Time:%sysfunc(datetime(),datetime20);
    %end;

    /* Step8- Clear the work tables */
%exit:
    /*if MAError is set then set the MAMsg*/
    %if  &MAError.  ne %str() %then
    %do;
        %if &MAError = 736 %then %let MAMsg = Macro[&sysmacroname.]. Error in Create table Statement SQLRC[&SQLRC]. Check the log for details;
        %if &MAError = 737 %then %let MAMsg = Macro[&sysmacroname.]. Error in Update table Statement SQLRC[&SQLRC]. Check the log for details;
        %if &MAError = 738 %then %let MAMsg = Macro[&sysmacroname.]. Error in Delete table Statement SQLRC[&SQLRC]. Check the log for details;
        %if &MAError = 739 %then %let MAMsg = Macro[&sysmacroname.]. Error in Insert into table Statement SQLRC[&SQLRC]. Check the log for details;
        %if &MAError = 766 %then %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
    /* Delete the temporary tables */
    %macdidel(libexist=Y, tblnms=&TblNames., templibname=&tmplibnm.);

    %if &MADebug = Y %then
    %do;
        %put marhcnt Execution End Time:%sysfunc(datetime(),datetime20);
        %let et=%sysfunc(datetime());
        /* print the time taken for execution; */
        %mautlpt(start_tm=&st., end_tm=&et., proc_nm='marhcnt');
    %end;
    %return;
%mend;
