/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/*****************************************************************************************/
/* NAME: matmtudf.sas                                                                    */
/* VERSION: 5.4                                                                        */
/*                                                                                       */
/* DESCRIPTION: Macro to update the following data model tables                          */
/*              CI_TREATMENT                                                             */
/*              CI_TREATEMENT_CHAR_UDF                                                   */
/*              CI_TREATEMENT_NUM_UDF                                                    */
/*              CI_TREATEMENT_DATE_UDF                                                   */
/*                                                                                       */
/*  Parameters:                                                                          */
/*             ct          - Physical table name for Treatment                           */
/*             ct_char_udf - Physical table name for Treatment_Char_Udf                  */
/*             ct_num_udf  - Physical table name for Treatment_Num_Udf                   */
/*             ct_dt_udf   - Physical table name for Treatment_Date_Udf                  */
/*                                                                                       */
/* PRODUCT: MA                                                                           */
/* USAGE: %matmtudf                                                                      */
/*                                                                                       */
/*****************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   18JUL2008 - First version of the macro.                                             */
/*                                                                                       */
/* S0526823 - Processed_Dttm added for CI_TREATMENT table                                */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 14Oct2008  S0504678   Update <TYPE>_Set_No column name to TREATMENT_HASH_VAL in       */
/*                       all TREATMENT_UDF tables                                        */
/*                                                                                       */
/* 14May2009  S0575393  Current_Version_Flg added to CI_TREATMENT table                  */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*                                                                                       */
/* 30Jun2009  S0575393  Current_Version_Flg should not be set back from N to Y           */
/*                                                                                       */
/* 06DEC2011  S0767119  Upserts changed to INSERT only                                   */
/*                                                                                       */
/* 06Jan2012  S0782404   Comment REPLACE in TMT_CHAR_UDF table processing                */
/*****************************************************************************************/
%macro matmtudf(ct=, ct_char_udf=, ct_num_udf=, ct_dt_udf=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: F0MA541;
    %put Last Modified on 06Jan2012;
    %put MATMTUDF macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Update the CI_TREATMENT table */
/* S0575393: Publish XML will have only the latest version of the Treatment per the campaign.
   Set the current_version_flg to N in the input set if the treatment_sk already marked as
   current_version N and in the input set it is coming as Y

   This scenario can happen when the treatment version is revised and published.  But a campaign
   which has got the older version didn't accept the updates and tries to publish */

proc sql;
    /* Pulling down summary treatment information */
    CREATE TABLE tmt_wh AS
        SELECT
            treatment_sk, treatment_cd, current_version_flg
        FROM
            &libnm..&ct. (&dsreadopts.)
    ;
proc sql;
    /* Marking non-current the ones already non-current in the warehouse */
    update
        work.&ct.
    set
        current_version_flg ='N'
    where
        treatment_sk IN (
            select
                treatment_sk
            from
                tmt_wh t
            where
                t.current_version_flg='N'
        )
    ;
    /* Identifying only those records that need uploaded */
    CREATE TABLE new_tmt AS
        SELECT
            inp.*
        FROM
            work.&ct. inp
        LEFT JOIN
            tmt_wh wh ON wh.treatment_sk = inp.treatment_sk
        WHERE
            wh.treatment_sk = .
    ;
    /* Retrieve the new treatment_sk and treatment_cds */
    SELECT
        treatment_sk, treatment_cd
    INTO
        :treatment_sk1 - :treatment_sk9999, :treatment_cd1 - :treatment_cd9999
    FROM
        new_tmt
    ;
/* Acquire only update row locks per treatment_cd. */
/* Switches previous SKs to current_version_flg=N  */
%do i = 1 %to &SQLOBS;
proc sql;
    /* Avoid Y -> N -> Y via UPDATE and DATA/MODIFY processing */
    update &libnm..&ct. t
        set
            current_version_flg='N', processed_dttm="&pdttm"dt
            where treatment_cd = "&&treatment_cd&i" and current_version_flg='Y'
            and treatment_sk ^= &&treatment_sk&i
        ;
%end;

/* b) Insert records to CI_TREATMENT table */
data &libnm..&ct.;
    modify &libnm..&ct. (&dswriteopts.) new_tmt UPDATEMODE=NOMISSINGCHECK;
    by TREATMENT_SK;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            /*
             * This is now an APPEND only operation.  If another thread
             * made this row appear before we got around to it, just
             * ignore.
             *
            processed_dttm="&pdttm"dt;
            replace;
             */
        end;
        when (%SYSRC(_DSENMR))
        do;
            processed_dttm="&pdttm"dt;
            _error_= 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC = 100000;
        %let MAError = 839;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =839;
    %let MAMsg =Macro[&sysmacroname.]. Data Modfiy Stmt error while attempting to update Treatment table . Check the log for details;
%end;

/* Step 2: Update all Treatment_UDF tables */

/* S0782404 Identify only those that are new */
proc sql;
  CREATE TABLE new_char_udf AS
    SELECT cudf.* FROM work.&ct_char_udf cudf INNER JOIN new_tmt t ON t.treatment_sk = cudf.treatment_sk;
  CREATE TABLE new_num_udf AS
    SELECT nudf.* FROM work.&ct_num_udf nudf INNER JOIN new_tmt t ON t.treatment_sk = nudf.treatment_sk;
  CREATE TABLE new_date_udf AS
    SELECT dudf.* FROM work.&ct_dt_udf dudf INNER JOIN new_tmt t ON t.treatment_sk = dudf.treatment_sk;
quit;

/* Treatment_Char_Udf: Inserts for Treatment_Char_Udf */
%marowcnt(new_char_udf, tmt_cudf_cnt);

%if &tmt_cudf_cnt. > 0 %then
%do;
    data &libnm..&ct_char_udf.;
        modify &libnm..&ct_char_udf. (&dswriteopts.) new_char_udf UPDATEMODE=NOMISSINGCHECK;
        by TREATMENT_SK CHAR_UDF_NM TREATMENT_HASH_VAL;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                /*
                 * UDFs can't change without altering the TREATMENT_SK.  Do not replace.
                 *
                processed_dttm="&pdttm"dt;
                replace;
                 */
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC = 100000;
            %let MAError = 863;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =863;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update TREATMENT_CHAR_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;


/* Treatment_Num_Udf: Inserts for Treatment_Num_Udf */
%marowcnt(new_num_udf, tmt_nudf_cnt);

%if &tmt_nudf_cnt. > 0 %then
%do;
    data &libnm..&ct_num_udf.;
        modify &libnm..&ct_num_udf. (&dswriteopts.) new_num_udf UPDATEMODE=NOMISSINGCHECK;
        by TREATMENT_SK NUM_UDF_NM TREATMENT_HASH_VAL;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                /*
                 * UDFs can't change without altering the TREATMENT_SK.  Do not replace.
                 *
                processed_dttm="&pdttm"dt;
                replace;
                 */
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC = 100000;
            %let MAError = 864;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =864;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update TREATMENT_NUM_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* Treatment_Date_Udf: Inserts for Treatment_Date_Udf */
%marowcnt(new_date_udf, tmt_dudf_cnt);

%if &tmt_dudf_cnt. > 0 %then
%do;
    data &libnm..&ct_dt_udf.;
        modify &libnm..&ct_dt_udf. (&dswriteopts.) new_date_udf UPDATEMODE=NOMISSINGCHECK;
        by TREATMENT_SK DATE_UDF_NM TREATMENT_HASH_VAL;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                /*
                 * UDFs can't change without altering the TREATMENT_SK.  Do not replace.
                 *
                processed_dttm="&pdttm"dt;
                replace;
                 */
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC = 100000;
            %let MAError = 865;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =865;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update TREATMENT_DATE_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%if &MADebug. eq Y %then
%do;
    %put MATMTUDF macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
