/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: matrnudf.sas                                                               */
/* VERSION: 5.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: Generic macro to insert / update following Extension table records  */
/*              CI_CAMPAIGN_EXT                                                     */
/*              CI_COMMUNICATION_EXT                                                */
/*              CI_TREATMENT_EXT                                                    */
/*                                                                                  */
/* Parameter:                                                                       */
/*    char_intbl    - Physical table name for Char_Udf input table                  */
/*    num_intbl     - Physical table name for Num_Udf input table                   */
/*    dt_intbl      - Physical table name for Dt_udf input table                    */
/*    byvar         - By variable grouping for transpose                            */
/*    ext_tbl       - Physcial table name for Extension table                       */
/*    parent_tbl    - Parent table name (Campaign/Communication/Treatment)          */
/*    nullcheck     - Null Check string to eliminate the NULL PK columns in         */
/*                    transposed table before updating the target table             */
/*                                                                                  */
/************************************************************************************/
/* PRODUCT: MA                                                                      */
/* USAGE: %matrnudf                                                                 */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - First version of the macro.                                             */
/*                                                                                       */
/*  S0511916 - Fixed the comm_trans record count check, which was causing the issue      */
/*                                                                                       */
/*  S0504678 - Treatment_UDF changes for CI5.3.  Part of this modification this          */
/*  code is refactored to be more generic now.  So that all extension tables             */
/*  can be updated using this single macro                                               */
/*                                                                                       */
/*  S0526823 - Renamed the user defined variable names,as the single character var       */
/*  names could potentialy clash with the input table column name                        */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 15Oct2008  S0506508   Added CHECKLIST_ITEM and associated UDF and Extn table publish  */
/*                                                                                       */
/* 01Dec2008  S0504678   Changes to support Dynamic_Treatment_Ext table population       */
/*                       with in Dynamic Treatments Stp call.                            */
/*                                                                                       */
/* 16Jan2009  S0504678   Changes to support all DBs.                                     */
/*                                                                                       */
/* 17Jun2009  S0571190   Exclude Dynamic Treatment records getting populated to          */
/*                       CI_Treatment_Ext table                                          */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*            S0504678   New macro parameters libstmt and dytmt_ext added to the macro   */
/*                       call.Dytmt_ext flag need to be set to Y only for updating       */
/*                       Dyn_treat_attr_ext  table.So it is defined as N by default for  */
/*                       other publish calls                                             */
/*            S0571190   New macro parameter exclude_dntmt added so to eliminate the     */
/*                       Treatment_hash_val=X getting populated in to the                */
/*                       ci_treatment_ext table . This variable is defaulted to N for all*/
/*                       the processing except for Treatment_Ext population where it is  */
/*                       set to Y                                                        */
/* 28Jun2010 S0594364    Use a common macro for getting the row count of tables          */
/* 06Jul2010 S0682253    skip datetime converions for null date udf                      */
/* 14Aug2013 S0992864    add dswriteopts in data modify                                  */
/*****************************************************************************************/
%macro matrnudf(char_intbl=, num_intbl=, dt_intbl=, byvar=, ext_tbl=, parent_tbl=, nullcheck=,
                 libstmt=, dytmt_ext=N, exclude_dytmt=N);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put  MATRNUDF - Current Version of the code: 541MA09;
    %put  Last Modified on 14Aug2013;
    %put  MATRNUDF macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* S0504678 Assign the required libraries if not exists */
%if %upcase(&dytmt_ext.)=Y %then
%do;
    %let _libnm = %upcase(%qscan(&libstmt.,2));
    %let _dbengine = %upcase(%qscan(&libstmt.,3));


    %if &_dbengine. eq SASSPDS %then
    %do;
        %macrtlib(%unquote(&libstmt.), MATRNUDF);
    %end;
    %else
    %do;/* Add reread_exposure=yes to the libstatment for the proc modify step */
        %let libstmt= %sysfunc(strip(&libstmt.)); /* Remove spaces after semicolon */

        %let semicolpos= %index(&libstmt.,%str(;));
        %let liblength= %length(&libstmt.);

        %if &semicolpos. > 0 %then
        %do;
            /* Remove the semicolon if it part of the libname */
            %if %substr(%superq(libstmt),&liblength.) eq %str(;) %then
                %let libstmt=%substr(%superq(libstmt),1,%eval(&liblength.-1));
        %end;
            /* S0970363 - add the required library options */
            %madsopts(%str(&libstmt.), libstmt, operation=DT);

            %macrtlib(%unquote(&libstmt. REREAD_EXPOSURE=YES), MATRNUDF);
    %end;

    %if &MAError. ne %str() %then
    %do;
        return;
    %end;
%end;
%else
%do;
    %let _libnm=&libnm.;
%end;

/* Char_Udf: Transpose and create a temp output table */

/* Proc transpose procedure looks for the a case sensitive value in the id column
so to transpose the values correctly.  This additional data step is introduced to
upcase the extn_column_nm values before using transpose procedure*/

/* S0571190 - Retain only static treatments "Treatment_hash_val = S" from Treamtn_Char_udf table */
data work.&char_intbl.;
    set work.&char_intbl.;
    char_ext_column_nm=upcase(char_ext_column_nm);
    %if &exclude_dytmt. eq Y %then
    %do;
        where treatment_hash_val eq 'S' and char_ext_column_nm ne '';
    %end;
    %else
    %do;
        where char_ext_column_nm ne '';
    %end;
run;

proc sort data=work.&char_intbl.;
    by &byvar.;
run;

proc transpose data=work.&char_intbl.
               out=work.char_udf_trans (drop=_NAME_ _LABEL_);
    by &byvar.;
    id char_ext_column_nm;
    var char_udf_val;
run;

/* Exit the process if any error */
%if &SYSERR.> 4  %then
%do;
    %let MAError =851;
    %let MAMsg = Macro[&sysmacroname.]. Proc Transpose error while attempting to Convert &parent_tbl._CHAR_UDF records SQLRC[&sqlrc]. Check the log for details;
    %return;
%end;

/* Num_Udf: Transpose and create a temp output table */
/* S0571190 - Retain only static treatments "Treatment_hash_val = S" from Treamtn_num_udf table */
data work.&num_intbl. ;
    set work.&num_intbl. ;
    num_ext_column_nm=upcase(num_ext_column_nm);
    %if &exclude_dytmt. eq Y %then
    %do;
        where treatment_hash_val eq 'S' and num_ext_column_nm ne '';
    %end;
    %else
    %do;
        where num_ext_column_nm ne '';
    %end;
run;

proc sort data=work.&num_intbl.;
    by &byvar.;
run;

proc transpose data=work.&num_intbl.
               out=work.num_udf_trans (drop=_NAME_ _LABEL_);
    by &byvar.;
    id num_ext_column_nm;
    var num_udf_val;
run;


/* Exit the process if any error */
%if &SYSERR.> 4  %then
%do;
    %let MAError =852;
    %let MAMsg = Macro[&sysmacroname.]. Proc Transpose error while attempting to Convert &parent_tbl._NUM_UDF records SQLRC[&sqlrc]. Check the log for details;
    %return;
%end;

/* Date_Udf: Transpose and create a temp output table */
/* S0571190 - Retain only static treatments "Treatment_hash_val = S" from Treamtn_date_udf table */
data work.&dt_intbl. ;
    set work.&dt_intbl. ;
    date_ext_column_nm=upcase(date_ext_column_nm);
    %if &exclude_dytmt. eq Y %then
    %do;
        where treatment_hash_val eq 'S' and date_ext_column_nm ne '';
    %end;
    %else
    %do;
        where date_ext_column_nm ne '';
    %end;
run;

/* S0504678 New step introduced to convert the date_udf_val which is being sent as string
   to a number equvalent for populating dyn_treat_attr_ext table. In the other publish process
   the date_udf_val is sent as number */
/* S0682253 */
%if %upcase(&dytmt_ext.)=Y %then
%do;
    data work.&dt_intbl. (rename=dt_val=date_udf_val);
        set work.&dt_intbl.;
        length dt_val 8.;
        if date_udf_val ne '' then /* skip the datetime conversion for null date udf */
        do;
            udf_dt=scan(date_udf_val,1,'');
            udf_tm=scan(date_udf_val,2,'');
            dt_val=input( put(input(udf_dt,yymmdd10.),date9.)||':'||put(input(udf_tm,time9.),tod9.),datetime20.) ;
        end;
        drop date_udf_val udf_dt udf_tm;
    run;
%end;

proc sort data=work.&dt_intbl.;
    by &byvar.;
run;

proc transpose data=work.&dt_intbl.
               out=work.dt_udf_trans (drop=_NAME_ _LABEL_);
    by &byvar.;
    id date_ext_column_nm;
    var date_udf_val;
run;

/* Exit the process if any error */
%if &SYSERR.> 4  %then
%do;
    %let MAError =853;
    %let MAMsg = Macro[&sysmacroname.]. Proc Transpose error while attempting to Convert &parent_tbl._DATE_UDF records SQLRC[&sqlrc]. Check the log for details;
    %return;
%end;

/* Merge the above three temp table to create a final table to update Campaign_Ext */

proc sort data=work.char_udf_trans;
    by &byvar.;
run;

proc sort data=work.num_udf_trans;
    by &byvar.;
run;

proc sort data=work.dt_udf_trans;
    by &byvar.;
run;

data work.transposed_tbl(where=(&nullcheck.)) ;
    merge work.char_udf_trans (in=_l_C)
          work.num_udf_trans (in=_l_N)
          work.dt_udf_trans (in=_l_D);
    by &byvar.;
    if _l_C or _l_N or _l_D then
    do;
        output;
    end;
run;

/* Exit the process if any error */
%if &SYSERR.> 4  %then
%do;
    %let MAError =854;
    %let MAMsg = Macro[&sysmacroname.]. Data Step error while attempting to Convert &parent_tbl._UDF records SQLRC[&sqlrc]. Check the log for details;
    %return;
%end;

/* Campaign_Ext: Update or Insert the transposed  campaign udf tables to respective extension table */
/* S0594364 */
%marowcnt( work.transposed_tbl , trans_cnt);
%if &trans_cnt. > 0 %then
%do;
    data &_libnm..&ext_tbl.;
        modify &_libnm..&ext_tbl. (&dswriteopts.) work.transposed_tbl UPDATEMODE=NOMISSINGCHECK;
        by &byvar.;
        select (_IORC_);
            when (%SYSRC(_SOK))
            do;
                /* S0504678 There will not be any updates to the Dynamic Treatment UDF val.
                   So always insert new records and discard the updates if any exists*/
                %if %upcase(&dytmt_ext.)=N %then
                %do;
                    replace;
                %end;
            end;
            when (%SYSRC(_DSENMR))
            do;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;
%end;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC = 100000;
        %let MAError = 855;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR.> 4  %then
%do;
    %let MAError =855;
    %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to update &parent_tbl._EXT records SQLRC[&sqlrc]. Check the log for details;
%end;

%put MATRNUDF macro End Time - %sysfunc(datetime(), datetime24.2);

%mend;
